<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// common.lib.php
// Common Library

// Activate GZip Output Buffer if it's not active yet
switch (rc('gzip'))
{
	case 'on':
		@ob_start('ob_gzhandler');
		break;
	case 'auto':
		if (function_exists('ob_get_level'))
			if (!ob_get_level()) ob_start('ob_gzhandler');
		break;
}


// Global Count*() arrays, e.g. an entire Post-count list will be stored to an array from one DB query
$PostsPerForum = array();
$ThreadsPerForum = array();
$NewForums = array();
$PostsByUser = array();
$PostsByThread = array();

$ACL = array();

// Load other libraries
require_once($libpath . 'forum.lib.php');
require_once($libpath . 'thread.lib.php');
require_once($libpath . 'post.lib.php');
require_once($libpath . 'message.lib.php');
require_once($libpath . 'mail.lib.php');


// Change current Forum/Activity ID for this guest at later time
function SetGuestLastForum($id)
{
	global $LoginUserID, $db;
	if ($LoginUserID) return;

	// hmm, it's not a good idea to update the time twice here, but is better than not.... :/
	return $db->ChangeRecord(array('LastActivity' => time(), 'LastForum' => $id), 'Session="' . session_id() . '"', 'Guests');
}


// ------------------------------ GLOBAL DATA INITIALISATION ------------------------------

// Unquote magic_quote'd data if needed
set_magic_quotes_runtime(0);

if (get_magic_quotes_gpc())
{
	while (list ($key, $value) = each ($_GET))
	{
		$_GET[$key] = str_replace('\\\'', '\'', $_GET[$key]);
		$_GET[$key] = str_replace('\\"', '"', $_GET[$key]);
		$_GET[$key] = str_replace('\\\\', '\\', $_GET[$key]);
	}
	while (list ($key, $value) = each ($_POST))
	{
		$_POST[$key] = str_replace('\\\'', '\'', $_POST[$key]);
		$_POST[$key] = str_replace('\\"', '"', $_POST[$key]);
		$_POST[$key] = str_replace('\\\\', '\\', $_POST[$key]);
	}
	while (list ($key, $value) = each ($_COOKIE))
	{
		$_COOKIE[$key] = str_replace('\\\'', '\'', $_COOKIE[$key]);
		$_COOKIE[$key] = str_replace('\\"', '"', $_COOKIE[$key]);
		$_COOKIE[$key] = str_replace('\\\\', '\\', $_COOKIE[$key]);
	}
}


// ------------------------------ HTML OUTPUT ------------------------------

// List messages for a Forum
//
// threadid = >0 if we're in a Thread
// page = page in the Thread
//
function ListMessages($forumid, $threadid = 0, $page = 0)
{
	global $MODULE, $ME, $IMG, $LoginUserID, $T;

	$message = new IMessage;
	$newonly = ($_GET['allmessages'] != 1) && ($LoginUserID > 0);
	$num = 0;
	if ($message->Find($forumid, $newonly, $threadid > 0))
	{
		if (!$num) BeginMessageTable(1);

		BeginMessageTable(($message->GetOptions() & 1) * 2);
		do
		{
			if ($num) MessageDelimiter();
			ShowMessage($message, false, ($threadid > 0 ? -$threadid : $forumid), $page);
			$num++;
		}
		while ($message->FindNext());
	}

	if ($num > 0) EndMessageTable();

	$x = '';
	if (!$threadid && CheckRights('editmessage', $forumid))
	{
		if ($x != '') $x .= ' &nbsp; ';
		$x .= '<a href="' . $MODULE['post'] . PD . 'message=-1&amp;forum=' . $forumid . sid() . '"><img ' . $IMG['add'] . ' /> ' . $T['compose_new_message'] . '</a>';
	}
	if (!$threadid && $message->Count($forumid, $threadid > 0) > $num)
	{
		if ($x != '') $x .= ' &nbsp; ';
		$x .= '<a href="' . $MODULE['main'] . PD . 'id=' . $forumid . '&amp;allmessages=1' . sid() . '"><img ' . $IMG['message'] . ' /> ' . $T['show_all_messages'] . '</a>';
	}
	if ($x != '') echo '<div class="actionbar_r" style="text-align:right;">' . $x . '</div>';
	elseif ($num) echo '<br />';
}

// This belongs to EditThread, but is also used by other functions
//
// disable_cats = make category options disabled (not for selecting)
//
function ListForumsRec($curr, $disable_cats = false, $parent = 0, $level = 0, $nextAvailBlock = true)
{
	global $output;

	$forum = new IForum;

	// This flag determines how to draw the forums tree 'lines'.
	// Unicode produces a real tree structure, but needs the client to have a unicode font installed.
	// Non-unicode uses simple characters.
	$unicodeTree = false;

	if ($unicodeTree)
	{
		$pad = '';
		for ($n = 0; $n < $level - 1; $n++) $pad .= ($nextAvailBlock ? '&#x2502; &nbsp; ' : '&nbsp;&nbsp; &nbsp; ');
		$pad2 = $pad;
		if ($level > 0)
		{
			$pad2 .= '&#x2514;&#x2500; ';
			$pad .= '&#x251C;&#x2500; ';
		}
	}
	else
	{
		$pad = '';
		for ($n = 0; $n < $level; $n++) $pad .= '| &nbsp; ';
			// also good:  '&#183; &nbsp; '  '| &nbsp; '  '&#150;&#150; '
		$pad2 = $pad;
	}

	$forums = $forum->GetChildA($parent);
	if (!is_array($forums)) return;
	$count = sizeof($forums);
	for ($n = 0; $n < $count; $n++)
	{
		$forum->LoadFromRecord($forums[$n]);

		$id = $forum->GetID();
		if (!CheckRights('viewforum', $id))
		{
			$nextAvail = $n < $count - 1;
			continue;
		}

		$classname = 'forum';
		if ($forum->IsCategory()) $classname = 'category';
		if ($curr == $id) $classname = 'current';

		$isCat = $forum->IsCategory();
		$name = $forum->GetName();
		$nextAvail = $n < $count - 1;

		$output .= '<option value="' . $id . '"' . ($curr == $id ? ' selected="selected"' : '') .
			' class="' . $classname . '"' .
			($disable_cats && $isCat ? ' disabled="disabled"' : '') .
			'>' . ($nextAvail ? $pad : $pad2) . t2h(' ' . $name) . '</option>';

		if ($forums[$n]['ChildCount'] > 0)
			ListForumsRec($curr, $disable_cats, $id, $level + 1, $nextAvail);
	}
}

// More abstract version to ListForumsRec
// Recursively reads all forum IDs from $parent on into $forums array.
//
function GetForumsRec($parent = 0, $level = 0)
{
	global $forums;

	$forum = new IForum;
	if (!$level) $forums = array();

	if ($forum->GetChild($parent)) do
	{
		if (!CheckRights('viewforum', $forum->GetID())) continue;
		array_push($forums, $forum->GetID());
		GetForumsRec($forum->GetID(), $level + 1);
	} while ($forum->GetNextChild());
}

// Count Posts per Thread for ALL Threads
//
function CountThreadPosts()
{
	global $PostsByThread, $db;

	$pre = $db->tblprefix;
	$sql = "SELECT {$pre}Posts.Thread, count(*) " .
		"FROM {$pre}Posts " .
		"GROUP BY Thread";

	$db->Exec($sql);

	$PostsByThread = array();
	while ($record = $db->GetRecord())
	{
		$PostsByThread[$record[0]] = $record[1];
	}
}

// Return number of Posts in a Thread
// Reads value from global array, so expects CountThreadPosts() to be called before
//
function GetPostsByThread($threadid)
{
	global $PostsByThread;
	if (isset($PostsByThread[$threadid]))
		return $PostsByThread[$threadid];
	else
		return 0;
}

// Count users who have read in threads for a set of threads in one step
// Input is an array(UserID)
// Returns an array(ThreadID => count)
//
function CountUserViews($threadids)
{
	global $db;
	if (!is_array($threadids)) return array();
	if (!sizeof($threadids)) return array();

	$a = array();
	$ids = join(',', $threadids);
	if ($record = $db->FastQuery('ThreadWatch', 'Thread, count(*) AS cnt', 'Thread IN (' . $ids . ')', '', '', /*group*/ 'Thread')) do
	{
		$a[$record['Thread']] = $record['cnt'];
	}
	while ($record = @mysql_fetch_array($db->result));
	#while ($record = $db->GetRecord());

	return $a;
}

// Count distinct users who have posted in a thread for a set of threads in one step
// Input is an array(UserID)
// Returns an array(ThreadID => count)
//
function CountReplyUsers($threadids)
{
	global $db;
	if (!is_array($threadids)) return array();
	if (!sizeof($threadids)) return array();

	$a = array();
	$ids = join(',', $threadids);
	if ($record = $db->FastQuery('Posts', 'Thread, count(distinct User) as cnt', 'Thread in (' . $ids . ')', '', '', /*group*/ 'Thread')) do
	{
		$a[$record['Thread']] = $record['cnt'];
	}
	while ($record = @mysql_fetch_array($db->result));
	#while ($record = $db->GetRecord());

	return $a;
}

// Find all Threads that contain any posts of the current User
//
// threads = array of Threads to search among
//
// Returns an array of Threads found
//
function FindUsersPosts($threads)
{
	global $LoginUserID, $db;

	if (!is_array($threads)) return false;
	if (!$threads) return false;

	$list = join(',', $threads);

	$a = $db->FastQuery1stArray('Posts', 'Thread', 'Thread IN (' . $list . ') AND User=' . $LoginUserID);
	if (!is_array($a)) $a = array();
	return $a;
}

// List threads
//
// where =
//   (int): Forum to list Threads from
//   (string): SQL restriction of Threads to show
// page = Page Number to display (-1: last page)
// editthread = Thread ID to edit
// get = GET params to add to page links (i.e. Search definition)
// order = SQL ORDER parameter
// limit = SQL LIMIT parameter
// write = enable "write access" at all? this means: icons for Thread editing. recommended to false on search queries etc.
//
function ListThreads($where, $page = 1, $editthread = 0, $get = '', $order = '', $limit = '', $write = true)
{
	function EditThread(&$thread)
	{
		global $ME, $IMG, $TABLE, $toplevel, $output, $T;

		$show_view_count = rc('count_thread_views');

		$is_moved = $thread->GetOptions() & 8;

		$output .= '<form name="form" action="' . $ME . PD . 'id=' . $toplevel . sid() . '" method="post"><tr valign="top">';

		$output .= '<td class="list_editforum" valign="middle"><img ' . $IMG['editthread'] . ' vspace="1" hspace="2" /></td>';
		$output .= '<td class="list_editforum" colspan="' . ($show_view_count ? 4 : 3) . '">';
		$output .= '<input type="hidden" name="action" value="editthread" />';
		$output .= '<input type="hidden" name="ID" value="' . $thread->GetID() . '" />';

		$output .= '<table cellspacing="' . $TABLE['thread']['space'] . '" cellpadding="' . $TABLE['thread']['pad'] . '" border="' . $TABLE['thread']['border'] . '">';
		$output .= '<tr><td>' . $T['topic'] . ':</td><td><input type="text" class="text" name="Subject" size="50" value="' . htmlquote($thread->GetSubject()) . '" /></td></tr>';

		$output .= '<tr><td>' . $T['description'] . ':&nbsp;</td><td><input type="text" class="text" name="Desc" size="50" value="' . htmlquote($thread->GetDesc()) . '" /></td></tr>';

		$output .= '<tr><td>' . $T['options'] . ':&nbsp;</td><td>';
		if (!$is_moved) $output .= '<input type="checkbox" class="radio" name="CloseThread" id="CloseThread" value="1"' . ($thread->IsClosed() ? ' checked="checked"' : '') . ' /><label for="CloseThread">' . $T['closed'] . '</label> &nbsp; ';
		$output .= '<input type="checkbox" class="radio" name="ImportantThread" id="ImportantThread" value="1"' . ($thread->IsImportant() ? ' checked="checked"' : '') . ' /><label for="ImportantThread">' . $T['important'] . '</label></td></tr>';

		if (!$is_moved)
		{
			$output .= '<tr><td>' . $T['forum'] . ':</td><td>';

			$output .= '<select name="Forum" onchange="document.form.MovedLink.checked=true;">';
			ListForumsRec($thread->GetForum(), true);
			$output .= '</select>';

			$output .= ' &nbsp; ';
			$output .= '<input type="checkbox" class="radio" name="MovedLink" id="MovedLink" value="1" /><label for="MovedLink">' . $T['moved_note'] . '?</label></td></tr>';
		}
		$output .= '</table>';

		$output .= '</td>';

		$output .= '<td class="list_editforum" colspan="2" align="right">';
		$output .= '<input type="submit" class="button" value="' . $T['save'] . '" /><br />';
		$output .= '<img ' . $IMG['empty'] . ' width="1" height="18" /><a href="' . $ME . PD . 'id=' . $toplevel . sid() . '">' . $T['cancel'] . '</a><br /><br />';
		$output .= '<input type="checkbox" class="radio" name="Remove" id="Remove" value="1" ' . suredelete() . ' /><label for="Remove">' . $T['delete'] . '?</label>';
		$output .= '</td>';

		$output .= '</tr></form>';
	}

	global $MODULE, $ME, $IMG, $TABLE, $LoginUser, $LoginUserID, $output, $T, $user_cache, $CFG;

	$threads_per_page = rc('threads_per_page');
	$mark_own_posts = rc('own_posts_in_threadlist');
	$advanced_counter = rc('advanced_thread_counter');
	$show_view_count = rc('count_thread_views');
	if ($get != '') $get = '&amp;' . $get;

	if (is_int($where) && !CheckRights('viewforum', $where)) return false;

	CountThreadPosts();

	$forum = new IForum;
	$thread = new IThread;

	$user = new IUser;
	$post = new IPost;

	if (is_int($where))
	{
		$forumid = $where;
		$where = 'Forum=' . $where;
		$show_forum = false;
	}
	elseif (is_string($where))
	{
		$forumid = -1;
		$show_forum = true;   // is a string -> is a WHERE definition (over all Forums) -> display Thread's Forum
	}
	else
	{
		return false;   // input type not supported
	}

	# ---------- PAGE SELECTION ----------
	$thread_count = $thread->Count($where);
	if (!$threads_per_page)
		$page_count = 1;
	elseif ($thread_count > $threads_per_page)
		$page_count = ceil($thread_count / $threads_per_page);
	else
		$page_count = 1;

	if ($page > $page_count) $page = $page_count;
	if ($page == -1) $page = $page_count;

	if ($page_count > 1 && $limit == '')
	{
		$boundary = 3;   // pages to show at the boundaries (begin/end)
		$middle = 2;     // pages to show around current page in the middle

		$page_str = '<td class="actionbar_l"><img ' . $IMG['page'] . ' /> ' . $T['page'] . ':&nbsp;';
		if ($page > 1)
			$page_str .= '<a href="' . $ME . PD . 'id=' . $forumid . '&amp;page=' . ($page - 1) . $get . sid() . '">&nbsp;&#xAB;&nbsp;' . $T['prev_page2'] . '&nbsp;&nbsp;</a>';
		$state = 0;
		for ($n = 1; $n <= $page_count; $n++)
		{
			// show pages if they were the only one replaced by '...'
			$med_left = ($n == $boundary + 1 && $n == $page - $middle - 1);
			$med_right = ($n == $page + $middle + 1 && $n == $page_count - ($boundary - 1) - 1);

			if ($n <= $boundary || $med_left ||  abs($page - $n) <= $middle || $med_right || $n > $page_count - $boundary)
			{
				if ($n == $page)      $page_str .= '&nbsp;<b>' . $n . '</b>&nbsp;';
				else                  $page_str .= '<a href="' . $ME . PD . 'id=' . $forumid . '&amp;page=' . $n . $get . sid() . '">&nbsp;' . $n . '&nbsp;</a>';
			}
			else
			{
				if ($n < $page && $state < 1) { $page_str .= '...'; $state = 1; }
				if ($n > $page && $state < 2) { $page_str .= '...'; $state = 2; }
			}
		}
		if ($page < $page_count)
			$page_str .= '<a href="' . $ME . PD . 'id=' . $forumid . '&amp;page=' . ($page + 1) . $get . sid() . '">&nbsp;&nbsp;' . $T['next_page2'] . '&nbsp;&#xBB;&nbsp;</a>';
		$page_str .= '</td>';
	}


	$output = '';

	$output .= '<table cellspacing="' . $TABLE['thread']['space'] . '" cellpadding="' . $TABLE['thread']['pad'] . '" border="' . $TABLE['thread']['border'] . '" width="100%"' . $TABLE['thread']['fixed'] . '>';
	// now for IE compatibility or so...
	// TODO: is this necessary?
	$output .= '<colgroup>';
	$output .= '<col width="' . $TABLE['thread']['col_icon'] . '" />';
	$output .= '<col />';
	if ($show_forum) $output .= '<col align="center" />';
	$output .= '<col width="' . $TABLE['thread']['col_date'] . '" />';
	if ($show_view_count) $output .= '<col width="' . $TABLE['thread']['col_views'] . '" align="center" />';
	$output .= '<col width="' . $TABLE['thread']['col_replies'] . '" align="center" />';
	$output .= '<col width="' . $TABLE['thread']['col_lastpost'] . '" align="right" />';
	$output .= '<col width="' . $TABLE['thread']['col_gotopost'] . '" align="right" />';
	$output .= '</colgroup>';

	# ---------- LOOK FOR THREADS ON THIS PAGE or accept $limit parameter ----------
	if ($limit == '')
	{
		if ($threads_per_page)
			$limit = (($page - 1) * $threads_per_page) . ',' . $threads_per_page;
		else
			$limit = '';
	}
	if ($order == '')
	{
		$order = ($get == '' ? '(Options & 2) DESC, ' : '') . 'LastPostDate DESC';
	}
	$threads = $thread->FindArray($where, $order, $limit);

	// read all threads' first posts' users into the cache
	$users = array();
	foreach($threads as $record)
		if (!in_array($record['User'], $users) && $record['User'] > 0 && !isset($user_cache[$record['User']]))
			array_push($users, $record['User']);
	build_entire_user_cache($users);

	$a = array();
	foreach($threads as $record) array_push($a, $record['ID']);
	$user_reads = ReadUserReads($a);
	$last_posts = GetLastPost($a);
	if ($mark_own_posts) $users_posts = FindUsersPosts($a);
	if ($advanced_counter && $show_view_count) $count_userviews = CountUserViews($a);
	if ($advanced_counter) $count_replyusers = CountReplyUsers($a);

	$n = 0;
	$new_threads = 0;
	$prev_important = false;
	foreach($threads as $record)
	{
		$impclass = '';

		$thread->LoadFromRecord($record);   // load thread as if it came directly from the database record

		if (!CheckRights('viewforum', $thread->GetForum(), $thread->GetID())) continue;

		// is this a 'moved' thread that expired?
		if ($thread->IsMovedExpired())
		{
			$thread->RemoveAllPosts();
			continue;
		}

		if (!$n)
		{
			$output .= '<tr valign="top">';
			$output .= '<td class="list_thread_head_left">&nbsp;</td>';
			$output .= '<td class="list_thread_head_0">' . $T['topic'] . '</td>';
			if ($show_forum) $output .= '<td class="list_thread_head_0" align="center">' . $T['forum'] . '</td>';
			$output .= '<td class="list_thread_head_0">' .
				(rc('display_thread_startdate') ? $T['date'] . ', ' : '') .
				$T['author'] . '</td>';
			if ($show_view_count) $output .= '<td class="list_thread_head_0" align="center">' . $T['views'] . '</td>';
			$output .= '<td class="list_thread_head_0" align="center">' . $T['replies'] . '</td>';
			$output .= '<td class="list_thread_head_right" align="right" colspan="2">' . $T['last_post'] . '</td>';
			$output .= '</tr>';
		}

		if ($forumid != -1 && $prev_important && !$thread->IsImportant())
		{
			$prev_important = false;

			$output .= '<tr><td class="list_post_old_delimiter" colspan="' . (6 + ($show_forum ? 1 : 0) + ($show_view_count ? 1 : 0)) . '">';
			if ($TABLE['post']['line'] == 1) $output .= '<hr />';
			else $output .= '<img ' . $IMG['empty'] . ' width="1" height="1" />';
			$output .= '</td></tr>';
		}
		elseif ($thread->IsImportant())
		{
			$prev_important = true;
			$impclass = '_important';
		}

		$output .= '<tr>';

		//$lastpost = GetLastPost("Thread=" . $thread->GetID());
		$lastpost = $last_posts[$thread->GetID()];
		//$lastread = $thread->GetLastRead();
		$lastread = $user_reads[$thread->GetID()]['LastRead'];

		$target = $thread->GetID();
		if ($thread->IsMoved()) $target = intval($thread->GetQuestion());

		# ICON
		$img = 'thread';
		$title = $T['topic'];
		$jmp_new = '';

		$flag_moved = $thread->IsMoved();
		$flag_new = $lastpost['Date'] > $lastread && $LoginUserID;
		$flag_own = $mark_own_posts && $LoginUserID > 0 && in_array($thread->GetID(), $users_posts);
		$flag_closed = $thread->IsClosed();
		$flag_imp = $thread->IsImportant();

		if ($flag_moved) $flag_new = $flag_own = $flag_closed = $flag_imp = false;
		#if ($flag_imp) $flag_own = false;

		if ($flag_moved)
		{
			$img .= '_moved';
			$title = $T['img_moved_thread'];
		}

		if ($flag_imp)
		{
			$title = $T['img_important_thread'];
		}

		if ($flag_closed)
		{
			$title = $T['img_closed_thread'];
		}

		if ($flag_new)
		{
			$img .= '_new';
			$title .= ', ' . $T['img_new_posts'];
			$new_threads++;

			// direct link to 1st unread post, only if user has already read something there before
			if ($lastread)
			{
				$jmp_new = MakePostLink($thread->FirstUnreadPost($lastread), 0, 0, true) . '<img ' . $IMG['unread'] . ' title="' . $T['goto_unread_post'] . '" />&nbsp;';
			}
		}

		if ($flag_own)
		{
			$img .= '_own';
			$title .= ', ' . $T['img_own_posts'];
		}

		if ($flag_closed)
		{
			$img .= '_closed';
		}

		if ($flag_imp)
		{
			$img .= '_important';
		}

		/*
		if (GetPostsByThread($thread->GetID()) > rc('hot_thread_posts') && rc('hot_thread_posts') ||
			$thread->GetViews() > rc('hot_thread_views') && rc('hot_thread_views'))
		{
			$img .= '_hot';
			$title = $T['hot_thread'];
		}
		*/

		$output .= '<td class="list_thread' . $impclass . '">';
		if ($thread->GetID() == $editthread && CheckRights('editpost', $thread->GetForum(), $thread->GetID(), 0))
			$output .= '<a name="here"></a>';
		$output .= '<a href="' . $MODULE['thread'] . PD . 'id=' . $target . sid() . '"><img ' . $IMG[$img] . ' title="' . $title . '" style="margin: 4px 0px;" /></a>';
		$output .= '</td>';

		# SUBJECT
		$output .= '<td class="list_thread' . $impclass . '">';
		if ($thread->HasAttach()) $output .= '<img ' . $IMG['attach'] . ' /> ';
		if ($thread->IsMoved()) $output .= '<small>' . $T['moved'] . ':</small> ';
		if ($jmp_new)
			$output .= $jmp_new;
		else
			$output .= ($CFG['mod_rewrite'] ?
					'<a href="' . $target . sid(3) . '">' :
					'<a href="' . $MODULE['thread'] . PD . 'id=' . $target . sid() . '">');
		$output .= '<b>' . t2h($thread->GetSubject()) . '</b></a>';
		if (rc('posts_per_page') > 0 && GetPostsByThread($thread->GetID()) > rc('posts_per_page'))
		{
			$boundary = 3;

			$thread_page_count = ceil(GetPostsByThread($thread->GetID()) / rc('posts_per_page'));
			#$output .= ' (<img ' . $IMG['page'] . ' title="' . $T['select_page'] . '" />';
			$output .= ' &nbsp; <img ' . $IMG['page'] . ' title="' . $T['select_page'] . '" />';
			$state = 0;
			for ($m = 1; $m <= $thread_page_count; $m++)
			{
				// show pages if they were the only one replaced by '...'
				$med = ($m == $boundary + 1 && $m == $thread_page_count - ($boundary - 1) - 1);

				if ($m <= $boundary || $med || $m > $thread_page_count - $boundary)
				{
					$output .= '<a href="' . $MODULE['thread'] . PD . 'id=' . $thread->GetID() . '&amp;page=' . $m . sid() . '"> ' . $m . '</a>';
				}
				else
				{
					if ($state < 1) { $output .= ' ...'; $state = 1; }
				}
			}
			#$output .= ')';
		}
		if ($thread->IsMoved())
		{
			// Show the target forum where this thread was moved into
			$t_thread = new IThread;
			$t_thread->Load($target);
			$target_id = $t_thread->GetForum();
			$t_forum = new IForum;
			$t_forum->Load($target_id);
			$target_name = $t_forum->GetName();

			$output .= ' &nbsp; <small>(' . $T['forum'] . ': <a href="' . $MODULE['main'] . PD . 'id=' . $target_id . sid() . '">' . t2h($target_name) . '</a>)</small> ';
		}

		if ($thread->IsImportant() && $thread->HasPoll() && $thread->GetVotes()) $output .= ' &nbsp; <small>' . $T['important_poll'] . '</small>';
		elseif ($thread->IsImportant()) $output .= ' &nbsp; <small>' . $T['important_thread'] . '</small>';
		elseif ($thread->HasPoll() && $thread->GetVotes()) $output .= ' &nbsp; <small>' . $T['poll'] . '</small>';

		if (rc('show_bookmarked_thread') && $thread->IsWatched() & 128) $output .= ' &nbsp; <img ' . $IMG['bookmark']. ' title="' . $T['bookmark'] . '" />';

		if (CheckRights('editpost', $thread->GetForum(), $thread->GetID(), 0) && $write) $output .= ' &nbsp; <a href="' . $MODULE['main'] . PD . 'id=' . $forumid . '&amp;page=' . $page . '&amp;editthread=' . $thread->GetID() . sid() . '#here" class="admin"><img ' . $IMG['edit'] . ' title="' . $T['change_thread'] . '" /></a>';

		if ($desc = $thread->GetDesc()) $output .= '<div class="desc">' . t2h($desc) . '</div>';
		$output .= '</td>';

		# FORUM
		if ($show_forum)
		{
			$forum = new IForum;
			$forum->Load($thread->GetForum());

			$output .= '<td class="list_thread' . $impclass . '" align="center">';
			$output .= '<small><a href="' . $MODULE['main'] . PD . 'id=' . $thread->GetForum() . sid() . '">' . t2h($forum->GetName()) . '</a></small></td>';
		}

		if ($thread->IsMoved())
		{
			$output .= '<td class="list_thread' . $impclass . '" colspan="5">&nbsp;</td>';
		}
		else
		{
			# AUTHOR, DATE
			$output .= '<td class="list_thread' . $impclass . '"><small>';
			if (rc('display_thread_startdate'))
			{
				$output .= FriendlyDate($thread->GetDate(), 1, 3);
				#$output .= FriendlyDate($thread->GetDate(), 2, 3, true, 4);
				# use this to display thread start dates in a relative form
				$output .= '<br />' . $T['by'] . ' ';
			}
			if ($user->Load($thread->GetUser()))
				$output .= '<a href="' . $MODULE['cp'] . PD . 'id=' . $user->GetID() . sid() . '">' . t2h(str_limit($user->GetName(), 20, true)) . '</a>';
			else
				$output .= t2h(str_limit($thread->GetUserName(), 20, true));
			$output .= '</small></td>';

			# VIEWS
			if ($show_view_count)
			{
				$views = $thread->GetViews();
				if ($advanced_counter)
				{
					$cuv = $count_userviews[$thread->GetID()];
					if (!$cuv) $cuv = 0;
					if ($views) $views .= '&nbsp;<small>(' . /*$thread->CountUsers()*/ $cuv . ')</small>';
				}
				$output .= '<td class="list_thread' . $impclass . '" align="center">' . ($views ? $views : '&nbsp;') . '</td>';
			}

			# REPLIES
			$replies = GetPostsByThread($thread->GetID()) - 1;
			if ($advanced_counter)
				if ($replies) $replies .= '&nbsp;<small>(' . $count_replyusers[$thread->GetID()] . ')</small>';
			$output .= '<td class="list_thread' . $impclass . '" align="center">' . ($replies ? $replies : '&nbsp;') . '</td>';

			# LAST POST
			if ($lastpost && $replies)
			{
				$output .= '<td class="list_thread' . $impclass . '" align="right"><small>';
				#$output .= MakePostLink($lastpost, 1, -1);
				$output .= FriendlyDate($lastpost['Date'], 1, 3);

				if (rc('display_thread_lastposter'))
				{
					$output .= '<br /><small>' . $T['by'] . '</small> ';
					if ($lastpost['User'] > 0)
					{
						//$output .= '<a href="' . $MODULE['cp'] . PD . 'id=' . $lastpost['User'] . sid() . '">' . t2h(str_limit($lastpost['UserName'], 20, true)) . '</a>';

						if ($user->Load($lastpost['User']))
							$output .= '<a href="' . $MODULE['cp'] . PD . 'id=' . $lastpost['User'] . sid() . '">' .
								t2h(str_limit($lastpost['UserName'], 20, true)) . '</a>';
						else
							$output .= t2h(str_limit($lastpost['UserName'], 20, true));
					}
					else
					{
						$output .= t2h(str_limit($lastpost['UserName'], 20, true));
					}
				}
				$output .= '</small></td>';
			}
			else
			{
				$output .= '<td class="list_thread' . $impclass . '">&nbsp;</td>';
			}

			# LAST POST ICON
			$output .= '<td class="list_thread' . $impclass . '" align="right">';
			if ($lastpost && $replies)
				$output .= MakePostLink($lastpost, 0, -1);
			else
				$output .= '&nbsp;';
			$output .= '</td>';
		}

		$output .= '</tr>';

		if ($thread->GetID() == $editthread && CheckRights('editpost', $thread->GetForum(), $thread->GetID(), 0))
			EditThread($thread);

		$n++;
	}  // end foreach

	if (!$n)
	{
		$output .= '<tr><td colspan="' . (6 + ($show_forum ? 1 : 0) + ($show_view_count ? 1 : 0)) . '" class="list_thread_head_0" align="center">';
		if ($show_forum)
			$output .= $T['no_threads_found'];
		else
			$output .= $T['no_threads_in_forum'];
		$output .= '</td></tr>';
	}

	$output .= '</table>';


	# ---------- THREAD ACTIONS ----------
	$x = '';
	if ($LoginUserID && $forumid > 0)
	{
		$mode = $forum->IsWatched($forumid);
		if ($LoginUser->GetEMail() != '' ||
			$LoginUser->GetICQ() != '' && $CFG['enable_icq'] ||
			$LoginUser->GetJabber() != '' && $CFG['enable_jabber'])
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$x .= '<img ' . $IMG['notify'] . ' /> ' . $T['notification'] . ':';
		}
		if ($LoginUser->GetEMail() != '')
		{
			if ($mode & 1)
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;unwatch=1&amp;page=' . $page . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['e_mail'] . '</a>';
			else
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;watch=1&amp;page=' . $page . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['e_mail'] . '</a>';
		}
		if ($LoginUser->GetJabber() != '' && $CFG['enable_jabber'])
		{
			if ($mode & 4)
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;unwatch=4&amp;page=' . $page . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['jabber'] . '</a>';
			else
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;watch=4&amp;page=' . $page . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['jabber'] . '</a>';
		}
		if ($LoginUser->GetICQ() != '' && $CFG['enable_icq'])
		{
			if ($mode & 2)
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;unwatch=2&amp;page=' . $page . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['icq'] . '</a>';
			else
				$x .= ' <a href="' . $ME . PD . 'id=' . $forumid . '&amp;watch=2&amp;page=' . $page . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['icq'] . '</a>';
		}
	}
	if ($LoginUserID && $forumid > 0 && $n && $new_threads)
	{
		if ($x != '') $x .= ' &nbsp; &nbsp; ';
		$x .= '<a href="' . $ME . PD . 'id=' . $forumid . '&amp;allthreadsread=1' . sid() . '">' . $T['mark_all_threads_read'] . '</a>';
	}
	if (CheckRights('addthread', $forumid) && CheckRights('writeforum', $forumid) && $forumid > 0)
	{
		if ($x != '') $x .= ' &nbsp; &nbsp; ';
		$x .= '<a href="' . $MODULE['post'] . PD . 'forum=' . $forumid . sid() . '"><img ' . $IMG['write'] . ' /> ' . $T['create_new_thread'] . '</a>';
	}

	# ---------- COMPOSITION OF PAGES AND ACTIONS ----------
	if ($page_count > 1 || $x != '')
	{
		$actionbar = '<table cellspacing="0" cellpadding="0" width="100%" class="actiontable">';
		$actionbar .= '<tr>';
		if ($page_count > 1) $actionbar .= $page_str;
		if ($x != '') $actionbar .= '<td class="actionbar_r" align="right">' . $x . '</td>';
		$actionbar .= '</tr></table>';
	}
	else
		$actionbar = '';

	echo $actionbar;
	echo $output;
	if ($n >= 12) echo $actionbar;

	return ($n > 0);
}

// List postings (and poll, if available)
//
// where =
//   (int): Thread to list Posts from
//   (string): SQL restriction of Posts to show
// page = Page Number to display
// get = GET params to add to page links (i.e. Search definition)
// write = enable "write access" at all? this means: icons for Thread editing. recommended to false on search queries etc.
// hightlight = array of words to highlight, from search i.e.
// give the number of posts to display, save the count query (good for search when we already have the count)
//
// Exports: $page_count
//
function ListPosts($where, $page = 1, $get = '', $order = 'Date ASC', $write = true, $highlight = false, $post_count = false)
{
	global $MODULE, $ME, $IMG, $TABLE, $abbc_cfg, $abbc_scan, $LoginUserID, $LoginUserName, $LoginUser, $CFG, $db, $T, $NEWLAYOUT, $page_count;

	$posts_per_page = rc('posts_per_page');
	if ($get != '') $get = '&amp;' . $get;

	if (is_int($where))
	{
		$threadid = $where;
		$where = 'Thread=' . $where;
		$show_thread = false;
	}
	elseif (is_string($where))
	{
		$threadid = -1;
		$show_thread = true;   // is a string -> is a WHERE definition (over all Threads) -> display Post's Thread
	}
	else
	{
		return false;   // input type not supported
	}

	$thread = new IThread;
	$user = new IUser;
	$post = new IPost;

	if ($threadid > 0)
	{
		$thread->Load($threadid);
		$writeaccess = CheckRights('writeforum', $thread->GetForum());

		if ($_GET['nocount'] != 1) $thread->IncViews();
	}

	CountUserPosts();

	# ---------- PAGE SELECTION ----------
	if (!$post_count)
		$post_count = $post->Count($where);

	if (!$posts_per_page)
		$page_count = 1;
	elseif ($post_count > $posts_per_page)
		$page_count = ceil($post_count / $posts_per_page);
	else
		$page_count = 1;

	if ($page > $page_count) $page = $page_count;
	if ($page == -1) $page = $page_count;

	if ($page_count > 1)
	{
		$boundary = 3;   // pages to show at the boundaries (begin/end)
		$middle = 2;     // pages to show around current page in the middle

		$page_str = '<td class="actionbar_l"><img ' . $IMG['page'] . ' /> ' . $T['page'] . ':&nbsp;';
		if ($page > 1)
			$page_str .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . ($page - 1) . '&amp;nocount=1' . $get . sid() . '">&nbsp;&#xAB;&nbsp;' . $T['prev_page2'] . '&nbsp;&nbsp;</a>';
		$state = 0;
		for ($n = 1; $n <= $page_count; $n++)
		{
			// show pages if they were the only one replaced by '...'
			$med_left = ($n == $boundary + 1 && $n == $page - $middle - 1);
			$med_right = ($n == $page + $middle + 1 && $n == $page_count - ($boundary - 1) - 1);

			if ($n <= $boundary || $med_left ||  abs($page - $n) <= $middle || $med_right || $n > $page_count - $boundary)
			{
				if ($n == $page)      $page_str .= '&nbsp;<b>' . $n . '</b>&nbsp;';
				else                  $page_str .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . $n . '&amp;nocount=1' . $get . sid() . '">&nbsp;' . $n . '&nbsp;</a>';
			}
			else
			{
				if ($n < $page && $state < 1) { $page_str .= '...'; $state = 1; }
				if ($n > $page && $state < 2) { $page_str .= '...'; $state = 2; }
			}
		}
		if ($page < $page_count)
			$page_str .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . ($page + 1) . '&amp;nocount=1' . $get . sid() . '">&nbsp;&nbsp;' . $T['next_page2'] . '&nbsp;&#xBB;&nbsp;</a>';
		$page_str .= '</td>';
	}

	# ---------- THREAD ACTIONS ----------
	$x = '';
	if ($threadid > 0)
	{
		if (CheckRights('closethread', $thread->GetForum(), $thread->GetID()) &&
		    CheckRights('importantthread', $thread->GetForum(), $thread->GetID()) &&
		    !$thread->IsClosed())
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;split=1' . sid() . '"><img ' . $IMG['split'] . ' title="' . $T['split_thread'] . '" /> ' . $T['split_thread'] . '</a>';
		}
		if (CheckRights('closethread', $thread->GetForum(), $thread->GetID()) && !$thread->IsClosed())
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;close=1&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['lock'] . ' title="' . $T['close_thread'] . '" /> ' . $T['close_thread'] . '</a>';
		}
		if ($thread->IsClosed())
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$x .= $T['thread_is_closed'];
			if (CheckRights('closethread', $thread->GetForum(), $thread->GetID()))
				$x .= ' (<a href="' . $ME . PD . 'id=' . $threadid . '&amp;close=0&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['unlock'] . ' title="' . $T['open_thread_again'] . '" /> ' . $T['open'] . '</a>)';
		}
		if ($LoginUserID)
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$mode = $thread->IsWatched();
			if ($mode & 128)
			{
				$x .= '<img ' . $IMG['bookmark']. ' /> ';
				$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;unwatch=128&amp;page=' . $page . '&amp;nocount=1' . sid() . '">' . $T['remove_from_bookmarks'] . '</a>';
			}
			else
			{
				$x .= '<img ' . $IMG['nobookmark']. ' /> ';
				$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;watch=128&amp;page=' . $page . '&amp;nocount=1' . sid() . '">' . $T['add_to_bookmarks'] . '</a>';
			}
			$x .= ' (<a href="' . $MODULE[cp] . PD . 'item=4' . sid() . '">' . $T['show_bookmarks'] . '</a>)';
		}
	}
	$more_actions = $x;

	$x = '';
	if ($threadid > 0)
	{
		/*if (!rc('small_layout'))
		{
			$x = $more_actions;
			$more_actions = '';
		}*/   // this takes up too much space...

		if ($LoginUserID)
		{
			$mode = $thread->IsWatched();
			if ($LoginUser->GetEMail() != '' ||
				$LoginUser->GetICQ() != '' && $CFG['enable_icq'] ||
				$LoginUser->GetJabber() != '' && $CFG['enable_jabber'])
			{
				if ($x != '') $x .= ' &nbsp; &nbsp; ';
				$x .= '<img ' . $IMG['notify'] . ' /> ' . $T['notification'] . ':';
			}
			if ($LoginUser->GetEMail() != '')
			{
				if ($mode & 1)
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;unwatch=1&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['e_mail'] . '</a>';
				else
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;watch=1&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['e_mail'] . '</a>';
			}
			if ($LoginUser->GetJabber() != '' && $CFG['enable_jabber'])
			{
				if ($mode & 4)
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;unwatch=4&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['jabber'] . '</a>';
				else
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;watch=4&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['jabber'] . '</a>';
			}
			if ($LoginUser->GetICQ() != '' && $CFG['enable_icq'])
			{
				if ($mode & 2)
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;unwatch=2&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap1'] . ' /> ' . $T['icq'] . '</a>';
				else
					$x .= ' <a href="' . $ME . PD . 'id=' . $threadid . '&amp;watch=2&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['tap0'] . ' /> ' . $T['icq'] . '</a>';
			}
		}
		if ($writeaccess && (!$thread->IsClosed() || CheckRights('closethread')))
		{
			if ($x != '') $x .= ' &nbsp; &nbsp; ';
			$x .= '<a href="' . $MODULE['post'] . PD . 'thread=' . $threadid . sid() . '"><img ' . $IMG['write'] . ' /> ' . $T['create_new_post'] . '</a>';
		}
	}

	# ---------- COMPOSITION OF PAGES AND ACTIONS ----------
	if ($page_count > 1 || $x != '')
	{
		$actionbar = '<table cellspacing="0" cellpadding="0" width="100%" class="actiontable">';
		$actionbar .= '<tr>';
		if ($page_count > 1) $actionbar .= $page_str;
		if ($x != '') $actionbar .= '<td class="actionbar_r" align="right">' . $x . '</td>';
		$actionbar .= '</tr></table>';
	}
	else
		$actionbar = '';

	echo $actionbar;

	# ---------- POLL ----------
	if ($threadid > 0 && $thread->HasPoll() && $thread->GetVotes())
	{
		$UserVoteNum = $thread->HasUserVoted();
		if (!$LoginUserID ||
			$_GET['showvotes'] ||
			$UserVoteNum ||
			$thread->IsPollEnded() ||
			($thread->IsClosed() && !CheckRights('closethread')) ||
			!CheckRights('allowvoting', $thread->GetForum(), $threadid))
		{
			# SHOW VOTES
			echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%">';
			echo '<tr>';
			echo '<td class="list_post_question_left"><b>' . t2h($thread->GetQuestion()) . '</b></td>';
			echo '<td class="list_post_question_right" width="8">&nbsp;</td>';
			echo '</tr>';
			echo '</table>';

			echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%" class="poll_table">';

			echo '<colgroup>';
			echo '<col width="' . $TABLE['post']['col_polloption'] . '" />';
			echo '<col width="' . $TABLE['post']['col_pollgraph'] . '" />';
			echo '<col />';
			echo '</colgroup>';

			echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';

			$votes = $thread->GetVotes();
			if ($votes)
			{
				$max = 0;
				$sum = 0;
				foreach ($votes as $vote)
				{
					if ($vote['Votes'] > $max) $max = $vote['Votes'];
					$sum += $vote['Votes'];
				}

				$maxwidth = $TABLE['post']['col_pollgraph'] - 6;

				foreach ($votes as $vote)
				{
					$percent = $sum > 0 ? round($vote['Votes'] / $sum * 100, 0) : 0;

					if ($vote['Votes'] == 0)
					{
						$graph = '<img ' . $IMG['graph0'] . ' width="' . $maxwidth . '" height="10" />';
					}
					elseif ($vote['Votes'] == $max)
					{
						//$graph = '<span class="graph1" style="width:300px;"></span>';
						$graph = '<img ' . $IMG['graph1'] . ' width="' . $maxwidth . '" height="10" />';
					}
					elseif ($vote['Votes'])
					{
						$g1w = round($vote['Votes'] * ($maxwidth - 2) / $max);
						$g0w = ($maxwidth - 2) - $g1w;

						/*$graph = '<span class="graph1" style="width:' . $g1w . 'px;"></span>' .
							'<span class="graph0" style="width:' . $g0w . 'px;"></span>';*/   // mozilla can't render this :(
						$graph = '<img ' . $IMG['graph1'] . ' width="' . $g1w . '" height="10" />' .
							'<img ' . $IMG['graph1b'] . ' width="2" height="10" />';
						if ($g0w) $graph .= '<img ' . $IMG['graph0'] . ' width="' . $g0w . '" height="10" />';
					}
					echo '<tr>';
					echo '<td class="list_post_votes" align="right" title="' . $vote['ID'] . '">' . abbc_proc($vote['Title']) . '</td>';
					echo '<td class="list_post_votes"><nobr>' . $graph . '</nobr></td>';
					echo '<td class="list_post_votes"><nobr><b>' . $vote['Votes'] . '</b> &nbsp;&middot;&nbsp; ' . $percent . '%</nobr></td>';
					echo '</tr>';
				}

				echo '<tr>';
				echo '<td class="list_post_votes" colspan="2">&nbsp;</td>';
				echo '<td class="list_post_votes"><nobr>';
				if ($sum == 0)
					echo $T['no_vote'];
				else if ($sum == 1)
					echo $sum . ' ' . $T['vote'];
				else
					echo $sum . ' ' . $T['votes'];
				echo '</nobr></td>';
				echo '</tr>';
			}

			$x = false;
			if ($thread->IsPollEnded())
			{
				if ($x) $x .= ' &nbsp; ';
				$x .= str_replace('%s', FriendlyDate($thread->GetPollTimeout(), 3, 1, true, 4), $T['poll_ended_x']);
			}
			elseif (!$thread->GetPollTimeout())
			{
				if ($x) $x .= ' &nbsp; ';
				$x .= $T['poll_no_timelimit'];
			}
			else
			{
				if ($x) $x .= ' &nbsp; ';
				$a = str_replace('%i', FriendlyDate($thread->GetPollTimeout(), 3, 1, true, 3), $T['poll_ends_x']);
				$a = str_replace('%a', FormatTime($thread->GetPollTimeout(), 3), $a);
				$x .= $a;
			}
			if ($LoginUserID && !$thread->IsPollEnded())
			{
				if ($x) $x .= ' &nbsp; ';
				if ($UserVoteNum > 0)
					$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . $page . '&amp;unvote=1&amp;nocount=1' . sid() . '"><img ' . $IMG['task2'] . ' /> ' . $T['unvote'] . '</a>';
				else
					$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . $page . '&amp;nocount=1' . sid() . '"><img ' . $IMG['task'] . ' /> ' . $T['back_to_voting'] . '</a>';
			}
			if (CheckRights('viewpollusers', $thread->GetForum(), $thread->GetID()) && $sum > 0 && $_GET['pollusers'] != 1)
			{
				if ($x) $x .= ' &nbsp; ';
				$x .= '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . $page . '&amp;showvotes=1&amp;pollusers=1&amp;nocount=1' . sid() . '"><img ' . $IMG['users'] . ' /> ' . $T['show_users'] . '</a>';
			}

			if ($x)
			{
				echo '<tr><td class="list_post_votes" colspan="3">' . $x . '</td></tr>';
			}

			// ----- Display usernames -----
			#echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';
			if ($_GET['pollusers'] == 1 && CheckRights('viewpollusers', $thread->GetForum(), $thread->GetID()))
			{
				echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';

				echo '<tr><td class="list_post_votes" colspan="3" align="left">' . $T['users_have_voted'] . ': ';
				$arr = $thread->GetUsersVoted();
				$c = 0;
				foreach ($arr as $a)
				{
					if ($c++) echo ', ';
					echo '<a href="' . $MODULE['cp'] . PD . 'id=' . $a[0] . sid() . '" title="' . $a[2] . '">' . t2h($a[1]) . '</a>';
				}
				echo '</td></tr>';

				echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';
			}

			echo '</table>';
		}
		else
		{
			# SHOW VOTE OPTIONS
			echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%">';
			echo '<tr>';
			echo '<td class="list_post_question_left"><b>' . t2h($thread->GetQuestion()) . '</b></td>';
			echo '<td class="list_post_question_right">&nbsp;</td>';
			echo '</tr>';
			echo '</table>';

			echo '<form action="' . $ME . PD . 'id=' . $threadid . sid() . '" method="post">';
			echo '<input type="hidden" name="action" value="vote" />';
			echo '<input type="hidden" name="page" value="' . $_GET['page'] . '" />';
			echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%" class="poll_table"' . $TABLE['post']['fixed'] . '>';

			echo '<colgroup>';
			echo '<col width="40" align="right" />';
			echo '<col />';
			echo '</colgroup>';

			echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';

			$votes = $thread->GetVotes();
			if ($votes)
			{
				foreach ($votes as $vote)
				{
					echo '<tr>';
					echo '<td class="list_post_votes" width="40"><input type="radio" class="radio" name="Vote" id="Vote' . $vote['ID'] . '" value="' . $vote['ID'] . '" /></td>';
					echo '<td class="list_post_votes" colspan="2"><label for="Vote' . $vote['ID'] . '">' . abbc_proc($vote['Title']) . '</label></td>';
					echo '</tr>';
				}
			}

			echo '<tr>';
			echo '<td class="list_post_votes">&nbsp;</td>';
			echo '<td class="list_post_votes" colspan="2"><input type="submit" class="button" value="' . $T['do_vote'] . '" /> &nbsp; ';
			echo '<a href="' . $ME . PD . 'id=' . $threadid . '&amp;page=' . $page . '&amp;showvotes=1&amp;nocount=1' . sid() . '"><img ' . $IMG['votes'] . ' /> ' . $T['show_results'] . '</a>';
			if (!$thread->GetPollTimeout())
				echo '&nbsp; <small>' . $T['poll_no_timelimit'] . '</small>';
			else
			{
				$a = str_replace('%i', FriendlyDate($thread->GetPollTimeout(), 3, 1, true, 3), $T['poll_ends_x']);
				$a = str_replace('%a', FormatTime($thread->GetPollTimeout(), 3), $a);
				echo '&nbsp; <small>' . $a . '</small>';
			}
			echo '</td>';
			echo '</tr>';

			echo '<tr><td class="list_post_votes" colspan="3"><img ' . $IMG['empty'] . ' width="1" height="6" /></td></tr>';

			echo '</table></form>';
		}

		if ($TABLE['post']['line'] == 1) echo '<hr />';
		else echo '<br />';
	}

	# ---------- POSTS ----------
	$n = 0;
	$lastdate = 0;
	$lastread = 0;
	$show_readline_option = $LoginUser->GetFlags() & 2;
	if ($posts_per_page)
		$limit = (($page - 1) * $posts_per_page) . ',' . ($posts_per_page + 1);
	else
		$limit = '';
	if ($post->Find($where, $order /*"Date " . ($show_thread ? 'desc' : 'asc')*/, $limit))
	{
		$first_postid = $post->GetID();
		$show_readline = false;

		if ($LoginUserID > 0 && $threadid > 0)
		{
			// what have I read last?
			$lastread = $db->FastQuery1st('ThreadWatch', 'LastRead', 'Thread=' . $post->GetThread() . ' AND User=' . $LoginUserID);
		}

		$count = 0;
		do
		{
			if ($show_thread)
			{
				$thread->Load($post->GetThread());
			}

			if (!CheckRights('viewforum', $thread->GetForum())) continue;

			if (!$n) BeginPostTable(0, $post->GetID());
			#if (!$n && $NEWLAYOUT) PostDelimiter(-2);   // make the first post have a complete border, also at the top side
			if ($n) PostDelimiter($post->GetID(), $show_readline && $show_readline_option);
			$show_readline = false;

			$lastdate = $post->GetDate();
			ShowPost($post, $thread, $n % 2, $page, $writeaccess && $write, false, $show_thread,
				$threadid > 0 &&                            // we are in a thread at all
				$post->GetDate() <= $lastread &&            // post if from before last read
				$post->GetEditDate() > $lastread,           // post was edited after last read
				$highlight);                                // words to highlight

			$thispostdate = $post->GetDate();
			$found_next = $post->FindNext();
			if ($found_next) $nextpostdate = $post->GetDate();
			else             $nextpostdate = 0;

			if ($threadid > 0)
			{
				// The following information is thread related, so don't show it for search results...

				if (CheckRights('is_admin') && $LoginUser->GetFlags() & 8)
				{
					// who has read until here?
					$pre = $db->tblprefix;
					$result = $db->Exec("SELECT {$pre}Users.Name, {$pre}ThreadWatch.LastRead FROM {$pre}ThreadWatch, {$pre}Users " .
						"WHERE {$pre}ThreadWatch.Thread = " . $threadid . ' ' .
						"AND {$pre}ThreadWatch.LastRead >= " . $thispostdate . ' ' .
						($nextpostdate ? "AND {$pre}ThreadWatch.LastRead < " . $nextpostdate . ' ' : '') .
						"AND {$pre}ThreadWatch.User = {$pre}Users.ID " .
						"ORDER BY Name");

					$i = 0;
					while ($record = $db->GetRecord())
					{
						if (!$i)
						{
							if ($NEWLAYOUT)
								echo '<tr><td class="list_post_new_users" colspan="2"><small>';
							else
								echo '<tr><td class="list_post_old_users" colspan="3"><small>';
						}
						if ($i) echo ', ';
						echo '<span title="' . t2h(strip_tags(FriendlyDate($record[1], 1, 3))) . '">' . t2h($record[0]) . '</span>';
						$i++;
					}
					if ($i) echo ' (' . $i . ')</small></td></tr>';
				}

				if ($lastread)
				{
					// have I read until here?
					$show_readline = ($lastread >= $thispostdate) && ($lastread < $nextpostdate);
				}
			}

			$n++;
		}
		while ($found_next && ++$count < $posts_per_page);
	}

	if ($n)
	{
		if ($NEWLAYOUT) PostDelimiter(-1);
		EndPostTable();
	}
	echo '<a name="bottom"></a>';

	echo $actionbar;

	if ($more_actions != '')
	{
		echo '<div class="actionbar_r" style="text-align: right; margin-bottom: 8px;">' . $more_actions . '</div>';
	}

	# SET LAST READ TIME
	if ($threadid > 0 && $page < $page_count)
	{
		// Not yet read all pages of this thread -> only mark as read what was actually displayed
		$thread->SetLastRead($lastdate);
	}
	elseif ($threadid > 0 && $page == $page_count)
	{
		// Thread entirely read, set current date
		$thread->SetLastRead(time());
	}
}


// Display a dropdown box to jump to another forum
//
function JumpForumBox($forumid)
{
	global $MODULE, $T, $output;

	$output = '';
	echo '<form name="jumpforum" action="' . $MODULE['main'] . '" method="get">';
	echo sid(4);
	echo $T['goto_forum'] . ' ';
	echo '<input type="hidden" name="req" value="main" />';
	echo '<select class="text" name="id" onchange="document.jumpforum.submit();">';
	echo '<option value="0"' . ($forumid == 0 ? ' selected="selected"' : '') . '>' . $T['overview'] . '</option>';
	ListForumsRec($forumid);
	echo $output;
	echo '</select>';
	echo ' <input type="submit" class="button" value="' . $T['go'] . '" /><br />';
	echo '</form>';
}

// Show a search textbox
//
// negative forumid will search in Thread ID = -forumid (as implemented in search.inc.php)
//
function SearchTextbox($forumid)
{
	global $MODULE, $IMG, $T;

	echo '<form action="' . $MODULE['search'] . '" method="get" style="display:inline;">';
	echo sid(4);
	echo '<input type="hidden" name="req" value="search" />';
	echo '<img ' . $IMG['search'] . ' /> ';
	echo '<input type="text" class="text" name="Query" size="20" />';
	echo '<input type="hidden" name="InSubject" value="1" />';
	echo '<input type="hidden" name="InMessage" value="1" />';
	echo '<input type="hidden" name="ResultView" value="2" />';
	echo '<input type="hidden" name="Forum" value="' . $forumid . '" />';
	echo ' <input type="submit" class="button" value="' . $T['search'] . '" /><br />';
	echo '</form>';
}


// ------------------------------ GLOBAL DATABASE COUNTING/STATISTIC FUNCTIONS ------------------------------

// Read all User's LastRead values for a set of threads in one step
// Input is an array(UserID)
// Returns an array(UserID => LastRead)
//
function ReadUserReads($threadids = false)
{
	global $LoginUserID, $db, $MODULE;
	if (!$LoginUserID) return 1;

	// do not read the entire table!
	if (is_array($threadids) && sizeof($threadids))
		$where = 'Thread IN (' . join(',', $threadids) . ') AND ';
	else
		return false;

	$a = array();
	if ($record = $db->FastQuery('ThreadWatch', 'Thread, LastRead', $where . 'User=' . $LoginUserID)) do
	{
		$a[$record['Thread']] = $record;
	}
	while ($record = @mysql_fetch_array($db->result));
	#while ($record = $db->GetRecord());

	return $a;
}

// Read ALL Users/Forums into the global array (saves many DB queries at large amount of data in the board)
//
function build_entire_user_cache($users = false)
{
	global $db, $user_cache;

	if (!$users) return false;
	if ($users) $where = 'ID IN (' . join(',', $users) . ')'; else $where = '';

	$user_cache = $db->FastQueryArray('Users', '*', $where, 'ID', '', '', 'ID');
}

function build_entire_forum_cache()
{
	global $db, $forum_cache, $CFG;

	$CFG['entire_forum_cache'] = 1;

	$forum_cache = $db->FastQueryArray('Forums', '*', '', 'Parent, Sort', '', '', 'ID');
}


// ------------------------------ SECURITY FUNCTIONS ------------------------------

function GetAllForumLevels()
{
	global $db, $subforums;
	$forums_tbl = $db->FastQueryArray('Forums', 'ID, Parent', '', 'Parent, ID');
	if ($forums_tbl === false) return array();

	$levels = array();
	$levels[-1] = array(0);
	$curr_level = -1;
	do
	{
		$curr_level++;
		$levels[$curr_level] = array();
		// scan all forums
		foreach ($forums_tbl as $forum)
		{
			// if this forum's parent is in direct superior level, add this forum to current level
			if (in_array($forum['Parent'], $levels[$curr_level - 1]))
				array_push($levels[$curr_level], $forum['ID']);
		}
	} while (sizeof($levels[$curr_level]) > 0);

	$out = array();
	foreach ($levels as $level => $forums)
	{
		foreach ($forums as $forum)
		{
			$out[$forum] = $level;
		}
	}

	// build forum ID resolve table
	// this array contains all subforums (in all levels) for any forum

	// for all forums:
	//   add id to subforums[parent]
	//   for all n in subforums:
	//     if parent is in subforums[n]:
	//       add id to subforums[n]

	// sorting of all forums from the database by 'Parent, ID' is necessary!

	$subforums = array();
	foreach ($forums_tbl as $forum)
	{
		if (!is_array($subforums[$forum['Parent']])) $subforums[$forum['Parent']] = array();

		array_push($subforums[$forum['Parent']], $forum['ID']);

		foreach ($subforums as $id => $sf)
		{
			if (in_array($forum['Parent'], $sf))
			{
				array_push($subforums[$id], $forum['ID']);
			}
		}
	}

	return $out;
}

function ReadACL()
{
	global $LoginUserID, $LoginUserGroups, $ACL, $db, $subforums, $forum_order;

	$forum = new IForum;
	$ACL = array();
	// structure of the ACL array:
	// array:ACL[Action]->array[Forum/-Thread]->Grant

	$flevels = GetAllForumLevels();
	if (sizeof($flevels))
	{
		$forum_order = ', CASE Forum ';
		$n = 0;
		foreach ($flevels as $fid => $level)
		{
			$forum_order .= 'WHEN ' . $fid . ' THEN ' . $level . ' ';
		}
		$forum_order .= 'END DESC';
	}
	else
	{
		$forum_order = '';
	}

	// Find all groups which current user is member of
	if ($LoginUserID > 0)
	{
		if (sizeof($LoginUserGroups) > 0)
			$groups_str = '`Group` IN (0,' . join(',', $LoginUserGroups) . ') OR ';
		else
			$groups_str = '(`Group`=0 AND User=0) OR ';
	}
	else
	{
		$groups_str = '`Group`=0 AND ';
	}

	// Find all relevant ACL rules
	$record = $db->FastQuery('ACL', 'User, `Group`, Action, Forum, Thread, `Grant`',
		'(' . $groups_str . 'User=' . $LoginUserID . ') ' .
		'AND Enabled',
		'User DESC, `Group` DESC, Thread DESC' . $forum_order . ', Action');

	if ($record !== false) do
	{
		// Read all values from the record
		$r_action = intval($record['Action']);
		$r_user = intval($record['User']);
		$r_group = intval($record['Group']);
		$r_forum = intval($record['Forum']);
		$r_thread = intval($record['Thread']);
		$r_grant = intval($record['Grant']);

		#echo '(raw) action $r_action user $r_user group $r_group forum $r_forum thread $r_thread grant $r_grant<br />';

		// Resolve grouped access rights immediately
		$r = $r_grant;

		if ($r_action == 81)   // guest posters
			$actions = array(33=>$r, 34=>$r, 35=>$r);
		elseif ($r_action == 82)   // users
			$actions = array(6=>$r, 7=>$r, 8=>$r, 9=>$r, 10=>$r, 12=>$r, 33=>$r, 34=>$r, 35=>$r, 36=>$r, 47=>$r, 48=>$r);
		elseif ($r_action == 83)   // moderators (incremental)
			$actions = array(38=>$r, 39=>$r, 40=>$r, 42=>$r, 43=>$r, 44=>$r, 46=>$r);
		elseif ($r_action == 84)   // administrators
			$actions = array(1=>$r, 2=>$r, 3=>$r, 4=>$r, 5=>$r, 6=>$r, 7=>$r, 8=>$r, 9=>$r, 10=>$r, 11=>$r, 12=>$r,
				21=>102400, 22=>512000, 23=>600, 24=>400,
				31=>$r, 32=>$r, 33=>$r, 34=>$r, 35=>$r, 36=>$r, 37=>$r, 38=>$r, 39=>$r, 40=>$r, 41=>$r, 42=>$r, 43=>$r, 44=>$r, 45=>$r, 46=>$r, 47=>$r, 48=>$r, 49=>$r,
				61=>10240, 62=>-1);
		else
			$actions = array($r_action=>$r);

		// Resolve forums to all successors
		if ($r_thread != 0)
			$forums = array(-$r_thread);
		else
			$forums = array_merge($subforums[$r_forum], array($r_forum));

		// Build ACL array
		foreach ($actions as $action => $grant)
		{
			// enforce the multilevel array structure of $ACL
			if (!array_key_exists($action, $ACL)) $ACL[$action] = array();

			if ($action < 30)
			{
				// store actions for forum=0 only (they're globally defined)
				if (!$r_forum && !$r_thread)
					if (!array_key_exists(0, $ACL[$action]))
						$ACL[$action][0] = $grant;
			}
			else
			{
				// store per-forum rules for per-forum/thread actions
				foreach ($forums as $forum)
				{
					if (!array_key_exists($forum, $ACL[$action])) $ACL[$action][$forum] = $grant;
				}
			}
		}

	} while ($record = @mysql_fetch_array($db->result));

	// debug: display granted access rights
	#echo '<pre>ACL:<br />'; print_r($ACL); echo '</pre>';

	return true;
}


// Check if current User has the right to perform a particular action
//
// action = Right's name
//
function CheckRights($action, $forum = 0, $thread = 0, $user = 0, $date = 0, $isLastPost = false)
{
	global $LoginUserID, $ACL;
	$read_only = rc('read_only');

	switch ($action)
	{
		case 'is_admin':
			return $ACL[1][0];

		case 'adduser':
			return $ACL[2][0] && !$read_only;

		case 'removeuser':
			return $ACL[3][0] && !$read_only;

		case 'renameuser':
			return $ACL[4][0] && !$read_only;

		case 'editprofile':
			return ($ACL[5][0] || ($LoginUserID == $user && $LoginUserID > 0)) && !$read_only;

		case 'changeavatar':
			return $ACL[6][0];

		case 'sendemail':
			return $ACL[7][0];

		case 'showuserlist':
			return $ACL[8][0];

		case 'showonlineusers':
			return $ACL[9][0];

		case 'showprofile':
			if ($user == $LoginUserID) return true;
			return $ACL[10][0];

		case 'setusergroups':
			return $ACL[11][0];

		case 'showstat':
			return $ACL[12][0];

		case 'maxavatarsize':
			if (isset($ACL[21][0])) return $ACL[21][0];
			return rc('avatar_bytes');

		case 'maxavatarwidth':
			return rc('avatar_x');

		case 'maxavatarheight':
			return rc('avatar_y');

		case 'maxphotosize':
			if (isset($ACL[22][0])) return $ACL[22][0];
			return rc('photo_bytes');

		case 'maxphotowidth':
			if (isset($ACL[23][0])) return $ACL[23][0];
			return rc('photo_x');

		case 'maxphotoheight':
			if (isset($ACL[24][0])) return $ACL[24][0];
			return rc('photo_y');

		case 'addforum':
			return $ACL[31][$forum] && !$read_only;

		case 'editforum':
			return $ACL[32][$forum] && !$read_only;

		case 'viewforum':
			if ($thread > 0 && isset($ACL[33][-$thread])) return $ACL[33][-$thread];
			return $ACL[33][$forum];

		case 'writeforum':
			if ($thread > 0 && isset($ACL[34][-$thread])) return $ACL[34][-$thread];
			return $ACL[34][$forum] && !$read_only;

		case 'addthread':
			return $ACL[35][$forum] && !$read_only;

		case 'createpoll':
			if ($read_only) return false;
			if (isset($ACL[36][-$thread])) return $ACL[36][-$thread];
			return $ACL[36][$forum];

		case 'editmessage':
			return $ACL[37][$forum] && !$read_only;

		case 'closethread':
			if ($read_only) return false;
			if (isset($ACL[38][-$thread])) return $ACL[38][-$thread];
			return $ACL[38][$forum];

		case 'importantthread':
			if ($read_only) return false;
			if (isset($ACL[39][-$thread])) return $ACL[39][-$thread];
			return $ACL[39][$forum];

		case 'editpoll':
			// needs 'editpost' right, too
			// user: sum of votes already given
			if ($read_only) return false;
			if (!$user) return true;
			if ($ACL[1][0]) return true;   // Admins may change everything...
			if (isset($ACL[40][-$thread])) return $ACL[40][-$thread];
			return $ACL[40][$forum];

		case 'viewpollusers':
			if ($read_only) return false;
			if (isset($ACL[41][-$thread])) return $ACL[41][-$thread];
			return $ACL[41][$forum];

		case 'editpost':
			if ($read_only) return false;

			// user's own posts
			if ($user === $LoginUserID && $LoginUserID > 0)
			{
				$maxtime = CheckRights('postedittime', $forum, $thread);
				if ($maxtime == -1) return true;                   // user can ALWAYS edit his own posts
				if (time() - $date < $maxtime * 60) return true;   // user can edit his own posts within maximum time
			}

			if (isset($ACL[42][-$thread])) return $ACL[42][-$thread];
			return $ACL[42][$forum];

		case 'removepost':
			// needs 'editpost' right, too
			// $user is already set properly...
			if ($read_only) return false;

			// user's own posts (only last post in thread and you have editpost right, too)
			if ($user === $LoginUserID &&
			    $LoginUserID > 0 &&
			    $isLastPost &&
			    CheckRights('editpost', $forum, $thread, $user, $date))
			{
				if ($ACL[49][-$thread]) return true;
				if ($ACL[49][$forum]) return true;
			}

			if (isset($ACL[43][-$thread])) return $ACL[43][-$thread];
			return $ACL[43][$forum];

		case 'noeditnote':
			if (isset($ACL[44][-$thread])) return $ACL[44][-$thread];
			return $ACL[44][$forum];

		case 'removeeditnote':
			if (isset($ACL[45][-$thread])) return $ACL[45][-$thread];
			return $ACL[45][$forum];

		case 'showip':
			if (isset($ACL[46][-$thread])) return $ACL[46][-$thread];
			return $ACL[46][$forum];

		case 'downloadattach':
			if (isset($ACL[47][-$thread])) return $ACL[47][-$thread];
			return $ACL[47][$forum];

		case 'allowvoting':
			if (isset($ACL[48][-$thread])) return $ACL[48][-$thread];
			return $ACL[48][$forum];

		case 'removeownpost':
			if (isset($ACL[49][-$thread])) return $ACL[49][-$thread];
			return $ACL[49][$forum];

		case 'maxattachsize':
			if (isset($ACL[61][-$thread])) return $ACL[61][-$thread] * 1024;
			if (isset($ACL[61][$forum])) return $ACL[61][$forum] * 1024;
			return rc('attach_bytes');

		case 'postedittime':
			if (isset($ACL[62][-$thread])) return $ACL[62][-$thread];
			if (isset($ACL[62][$forum])) return $ACL[62][$forum];
			return -1;   // no time restriction
	}

	return false;
}


// Is the given password considered to be secure?
//
// return = 0: ok | 1: too short | 2: equal username | 3: need number | 4: need special character
//
function IsSecurePassword($pass, $username)
{
	if (rc('pass_minlength') > 0 && strlen($pass) < rc('pass_minlength')) return 1;
	if (rc('pass_notusername') && !strcasecmp($pass, $username)) return 2;
	if (rc('pass_neednumber') && !preg_match('/[0-9]/', $pass)) return 3;
	if (rc('pass_needspecial') && !preg_match('/[!-\/:-@[-`{-\xFF]/', $pass)) return 4;
	return 0;
}


// ------------------------------ MISC. FUNCTIONS ------------------------------

// Append an error message to the error logfile
//
function errorlog($error)
{
	global $LoginUserID, $LoginUserName, $ME, $logpath, $CONFFILE;
	if ($CONFFILE['nowrite']) return false;

	$error = trim($error);
	if ($error == '') return;
	$error = str_replace('<br />', "\r\n\t", $error);
	$error = str_replace('<br />', "\r\n\t", $error);
	$error = "\t" . trim($error) . "\r\n\r\n";
	$error = date('d.m.Y H:i:s') . " - User $LoginUserID $LoginUserName - $ME - " . $_SERVER['REMOTE_ADDR'] . "\r\n" . $error;

	$cnt = 10;
	do
	{
		if ($cnt < 10) if (function_exists('usleep')) usleep(50000);   // 50ms delay on collision
		$fp = fopen($logpath . 'error-' . date('Y-m') . '.log', 'a');
	}
	while ($fp === false && --$cnt);
	if ($fp === false && !$cnt) return false;

	#$fp = fopen($logpath . 'error.log', 'a');
	fwrite($fp, $error);
	fclose($fp);
}


// Write a User's action to the board log
//
// email = t: this is an e-mail, f: it's not
//
function addlog($action, $email = false)
{
	global $LoginUserID, $LoginUserName, $host_browser, $host_bver, $host_lang, $host_os, $db, $logpath, $CONFFILE;
	if ($CONFFILE['nowrite']) return false;

	if ($email)
	{
		$fp = fopen($logpath . 'email-' . date('Y-m-d-H-i-s') . '.log', 'w');
		fwrite($fp, $action);
		fclose($fp);
	}
	else
	{
		$name = $LoginUserName;
		if ($name == '') $name = '-';

		$bver = $host_bver;
		if ($bver == '') $bver = '-';

		$sid = session_id();

		$action = str_replace('"', '\\"', trim($action));
		if ($action == '') return;

		/*$db->AddRecord(array(
			'Date' => time(),
			'UserID' => $LoginUserID,
			'UserName' => $name,
			'IP' => $_SERVER['REMOTE_ADDR'],
			'Action' => $action,
			'Browser' => $host_browser,
			'BVer' => $bver,
			'OS' => $host_os,
			'Lang' => $host_lang,
			'Session' => $sid
			), 'Log');*/

		$action = date('Y-m-d H:i:s') . " $LoginUserID \"$name\" " . $_SERVER['REMOTE_ADDR'] . " \"$action\" $host_browser $bver $host_os $host_lang $sid\r\n";

		$cnt = 10;
		do
		{
			if ($cnt < 10) if (function_exists('usleep')) usleep(50000);   // 50ms delay on collision
			$fp = fopen($logpath . 'board-' . date('Y-m-d') . '.log', 'a');
		}
		while ($fp === false && --$cnt);
		if ($fp === false && !$cnt) return false;

		#$fp = fopen($logpath . 'board-' . date('Y-m-d') . '.log', 'a');
		if ($fp === false) die('Unable to open log file.');
		fwrite($fp, $action);
		fclose($fp);
	}
}

// make a string safe for using it in regular expressions
//
function regsafe($str)
{
	return preg_replace('/([\/\\\\\\^\\$.\\[\\]|()?*+{}])/', '\\$1', $str);
}

// Same as PHP function explode(), but also splits by '...'
//
// mask_bs = t|f: take case of \ cahracters, and ignore \" f.ex. [currently ignored to true]
//
function explode_quoted($sep, $str, $mask_bs = true)
{
	$out = array();
	$len = strlen($str);
	$instr = false;   // are we currently inside a string?
	$pos = 0;         // current starting position
	$startpos = 0;    // remember last part beginning
	$pos_s = -1;      // position of ' ' (space) / separator
	$pos_q = -1;      // position of '"' (quote)

	while ($pos < $len)
	{
		if ($pos_s < $pos)   // only search if last result is before current position
			if (($pos_s = strpos($str, $sep, $pos)) === false) $pos_s = $len;
				// set pointer to end of string, if symbol was not found
		if ($pos_q < $pos)
			if (($pos_q = strpos($str, '"', $pos)) === false) $pos_q = $len;

		$minpos = min($pos_s, $pos_q);   // find the nearest interesting symbol

		if ($instr === false && $minpos === $pos_s)   // found a space/separator first (not inside a string)
		{
			$out[] = str_replace(
				'\\"',
				'"',
				substr($str, $startpos, $minpos - $startpos));
			$startpos = $pos = $minpos + 1;
		}
		elseif ($minpos === $pos_q)   // found a quote first
		{
			if ($minpos === $startpos && $instr === false)   // first symbol of this part AND not inside a string (must be so)
			{
				$instr = true;
				$startpos = $pos = $minpos + 1;   // jump over beginning "
			}
			elseif ($instr === true && $str{$minpos - 1} !== '\\' && ($str{$minpos + 1} === $sep
			                                                          || $minpos === $len - 1))
				// inside a string AND previous symbol is not \ AND next symbol is space/separator
				//                                                  OR no more characters (last symbol)
			{
				$instr = false;
				$out[] = str_replace(
					'\\"',
					'"',
					substr($str, $startpos, $minpos - $startpos));
				$startpos = $pos = $minpos + 2;   // jump over quote + separator
			}
			/*
			elseif ($instr === false && $str{$minpos - 1} === '\\')
				// not inside a string AND previous symbol is \
			*/
			else   // found something of no interest
			{
				$pos = $minpos + 1;
			}
		}
		else   // found something of no interest
		{
			$pos = $minpos + 1;
		}
	}

	// Something left?
	if ($startpos < $pos)
	{
		$out[] = str_replace(
			'\\"',
			'"',
			substr($str, $startpos, $pos - $startpos));
	}

	return $out;
}

// mask mail addresses so that spam harvesters won't find it (so easily)
function maskmail($str)
{
	$str = str_replace('@', '&#x0040;', $str);
	$str = str_replace('.', '&#x002E;', $str);
	$str = str_replace('_', '&#x005F;', $str);
	return $str;
}

// convert "10M" in 10.485.760 (10 MB)...
function unverb_size($str)
{
	$len = strlen($str);
	if (!strcasecmp(substr($str, $len - 1, 1), 'k')) return intval($str) * 1024;
	if (!strcasecmp(substr($str, $len - 1, 1), 'm')) return intval($str) * 1024 * 1024;
	if (!strcasecmp(substr($str, $len - 1, 1), 'g')) return intval($str) * 1024 * 1024 * 1024;
	return intval($str);
}

?>
