<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// common_post.inc.php
// Common Library: HTML output functions for Posts and Messages


// Begin / End <table> framework to display Posts using ShowPost
//
function BeginPostTable($mode = 0, $postid = 0)
{
	global $TABLE, $T, $NEWLAYOUT;

	echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%"' . $TABLE['post']['fixed'] . '>';
	if (!$NEWLAYOUT)
	{
		echo '<colgroup>';
		echo '<col width="' . $TABLE['post']['col_info'] . '" />';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';

		if ($mode == 0)
		{
			echo '<tr valign="top">';
			echo '<td class="list_post_head_left" width="' . $TABLE['post']['col_info'] . '">';
			if ($postid) echo '<a name="' . $postid . '"></a>';   // optionally show post-id link
			echo $T['author'] . '</td>';
			echo '<td class="list_post_head_right" colspan="2">' . $T['post'] . '</td>';
			echo '</tr>';
		}
		elseif ($mode == 1)
		{
			echo '<tr valign="top">';
			echo '<td class="list_post_head_left" colspan="2">' . $T['topic'] . ', ' . $T['author'] . ', ' . $T['date'] . '</td>';
			echo '<td class="list_post_head_right">&nbsp;</td>';
			echo '</tr>';
		}
	}
	else   // NEWLAYOUT
	{
		echo '<colgroup>';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr valign="top">';
		echo '<td class="list_post_head_left">';
		if ($postid) echo '<a name="' . $postid . '"></a>';   // optionally show post-id link
		#echo $T['topic'] . ', ' . $T['author'] . ', ' . $T['date'] . '</td>';
		echo $T['posts'] . '</td>';
		echo '<td class="list_post_head_right">&nbsp;</td>';
		echo '</tr>';
	}
}

function PostDelimiter($postid = 0, $readline = false)
{
	global $TABLE, $IMG, $NEWLAYOUT;

	if ($readline && $NEWLAYOUT)
		$class = 'list_post_new_readline';
	elseif ($readline)
		$class = 'list_post_old_readline';
	elseif (!$NEWLAYOUT)
		$class = 'list_post_old_delimiter';
	elseif ($postid == -1)
		$class = 'list_post_new_endline';
	elseif ($postid == -2)
		$class = 'list_post_new_topline';
	else
		$class = 'list_post_new_delimiter';

	echo '<tr><td class="' . $class . '" colspan="' . ($NEWLAYOUT ? 2 : 3) . '">';
	if ($postid > 0) echo '<a name="' . $postid . '"></a>';   // optionally show post-id link
	if ($TABLE['post']['line'] == 1) echo '<hr />';
	else echo '<img ' . $IMG['empty'] . ' width="1" height="1" />';
	echo '</td></tr>';
}

function EndPostTable()
{
	echo '</table>';
}


// Check if a post attachment file may be displayed inline as an image
//
function display_post_image($filename)
{
	$x = strrpos($filename, '.');
	if ($x === false) return false;   // filename needs to contain a . for analysis and renaming
	$ext = substr($filename, $x);
	switch (strtolower($ext))
	{
		case '.jpg':
		case '.jpeg':
		case '.gif':
		case '.png':
			// get the size in pixels
			if (@filesize($filename) > 100000) return false;   // file too long
			$a = @GetImageSize($filename);
			if ($a === false) return false;   // not a valid image file
			if ($a[0] > 750) return false;    // too wide
			if ($a[1] > 600) return false;    // too high

			// image has passed all tests, and may now be displayed
			return true;
	}
	return false;
}

// Show a post, for thread.php (Thread view), post.php (Post preview and Latest post review) and search.php (Search results)
//
// Expects the <table> framework already to be started and to be closed after last post
// Expects CountUserPosts() to be called before showing posts
//
// post = IPost object set to the post to be shown
// style = 0|1: css class for background
// page = currently displayed page
// writeaccess = t|f: show quote/reply controls
// preview = t|f: preview mode? -> no edit/reply... controls
// show_thread = t|f: search mode (display thread link)
// update_mark = t|f: display 'updated' marker: this post has beed edited after read by the user
// hightlight = array of words to highlight, from search i.e.
//
function ShowPost(&$post, &$thread, $style = 0, $page = 1, $writeaccess = true, $preview = false, $show_thread = false, $update_mark = false, $highlight = false)
{
	global $ME, $LoginUserID, $LoginUser, $IMG, $ICON, $TEXT, $MODULE, $abbc_cfg, $avatar_path, $attach_path, $T, $NEWLAYOUT, $TEXTONLY, $CFG;

	$user = new IUser;
	$threadid = $thread->GetID();

	// ----- USER NAME -----
	$out_username = '';
	$user->Reset();
	if ($post->GetUser() > 0)
	{
		if ($user->Load($post->GetUser()))
		{
			$out_username .= GetUserOnlineImg($user->GetOnline()) . ' ';
			$out_username .= MakeUserLink($user->GetID(), $user->GetName(), true) . '&nbsp;';
			if ($user->GetGender() != '')
				$out_username .= ' ' . GetGenderImage($user->GetGender());
			if ($post->GetUser() == $thread->GetUser())
				$out_username .= ' <img ' . $IMG['starter'] . ' title="' . $T['thread_starter'] . '" />';
		}
		else
			$out_username .= '(' . $post->GetUser() . ')';
	}
	else
		$out_username .= '<b>' . t2h($post->GetUserName()) . '</b>';

	// ----- SUBJECT -----
	$out_subject = '';
	if ($post->GetSubject() != '')
	{
		$html = t2h($post->GetSubject());
		if (is_array($highlight))
			$html = preg_replace('/(' . join('|', array_map('regsafe', $highlight)) . ')/i', '<span class="highlight">$1</span>', $html);
		$out_subject .= $html;
	}

	// ----- DATE/TIME -----
	$out_date = '<span class="date">';
	if ($update_mark) $out_date .= '<img ' . $IMG['warning'] . ' title="' . $T['post_changed_since_last_read'] . '" />&nbsp; ';
	$out_date .= FriendlyDate($post->GetDate(), 1, 3);
	if (!$NEWLAYOUT && $out_subject != '')
		$out_date .= ' | <b>' . $out_subject . '</b>';
	$out_date .= '</span>';

	// ----- CONTACT ICONS -----
	$out_contact = '';
	$n = 0;
	if ($user->GetEMail() != '' && CheckRights('sendemail'))
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['email'] . ' title="' . $T['e_mail'] . '" />';
		if ($TEXT) $a .= ' ' . $T['e_mail'];
		$out_contact .= MakeUserLink($user->GetID(), $a, false, true, false);
		$n++;
	}
	/*
	// Disabled for readablility reasons. These icons are not important for daily use.
	if ($user->GetRealJabber() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['jabber'] . ' title="' . $T['jabber'] . ': ' . maskmail(htmlquote($user->GetRealJabber())) . '" />';
		if ($TEXT) $a .= ' ' . $T['jabber'];
		$out_contact .= $a;
		$n++;
	}
	if ($user->GetICQ() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['icq'] . ' title="' . $T['icq'] . ': ' . htmlquote($user->GetICQ()) . '" />';
		if ($TEXT) $a .= ' ' . $T['icq'];
		$out_contact .= '<a href="http://web.icq.com/wwp/' . urlencode($user->GetICQ()) . '" target="_blank">' . $a . '</a>';
		$n++;
	}
	if ($user->GetAIM() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['aim'] . ' title="' . $T['aim'] . ': ' . htmlquote($user->GetAIM()) . '" />';
		if ($TEXT) $a .= ' ' . $T['aim'];
		$out_contact .= $a;
		$n++;
	}
	if ($user->GetYIM() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['yim'] . ' title="' . $T['yim'] . ': ' . htmlquote($user->GetYIM()) . '" />';
		if ($TEXT) $a .= ' ' . $T['yim'];
		$out_contact .= $a;
		$n++;
	}
	if ($user->GetMSN() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['msn'] . ' title="' . $T['msn'] . ': ' . maskmail(htmlquote($user->GetMSN())) . '" />';
		if ($TEXT) $a .= ' ' . $T['msn'];
		$out_contact .= $a;
		$n++;
	}
	*/
	if ($user->GetHomepage() != '')
	{
		if ($n) $out_contact .= '&nbsp; ';
		$a = '';
		if ($ICON) $a .= '<img ' . $IMG['homepage'] . ' title="' . $T['homepage'] . ': ' . htmlquote($user->GetHomepage()) . '" />';
		if ($TEXT) $a .= ' ' . $T['homepage'];
		$out_contact .= '<a href="' . $MODULE['derefer'] . PD . 'url=' . urlencode($user->GetHomepage()) . '" target="_blank">' . $a . '</a>';
		$n++;
	}

	// ----- PREVIEW | POST ACTIONS -----
	$out_action = '';
	if (!$NEWLAYOUT)
		$between = '&nbsp; &nbsp; &nbsp; ';
	else
		$between = '';
	$n2 = 1;
	if ($preview)
	{
		if ($n) $out_action .= '&nbsp; ';
		if ($n2) $out_action .= $between;
		$out_action .= '(' . $T['preview'] . ')';
		$n++; $n2 = 0;
	}
	else
	{
		// Copy link to this post into clipboard. Only available in a thread view
		if ($ME == $MODULE['thread'])
		{
			$newDate = $post->GetDate() - 1;
			// Special timestamp for the first post of a thread
			if ($page == 1 && $post->GetDate() == $thread->GetDate() && $post->GetUser() == $thread->GetUser())
				$newDate = 0;

			if ($LoginUserID)
			{
				if ($n) $out_action .= '&nbsp; ';
				if ($n2) $out_action .= $between;
				$out_action .= '<nobr><a href="' . $MODULE['main'] . PD . 'id=' . $thread->GetForum() . '&amp;threadunread=' . $thread->GetID() . '&amp;threadunread_time=' . $newDate . sid() . '">';
				if ($ICON) $out_action .= '<img ' . $IMG['unread'] . ' title="' . $T['unread_from_here'] . '" />';
				if ($TEXT) $out_action .= ' ' . $T['unread_from_here_short'];
				$out_action .= '</a></nobr>';
				$n++; $n2 = 0;
			}

			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr>' . MakePostLink($post->GetID(), 0, $page, true);
			if ($ICON) $out_action .= '<img ' . $IMG['goto_post'] . ' title="' . $T['link_to_here'] . '" />';
			if ($TEXT) $out_action .= ' ' . $T['link_to_here_short'];
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}

		if (CheckRights('showip', $thread->GetForum(), $thread->GetID()) && $writeaccess)
		{
			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr><a href="' . $MODULE['showip'] . PD . 'id=' . $post->GetID() . sid() . '">';
			if ($ICON) $out_action .= '<img ' . $IMG['host'] . ' title="' . $T['show_ip'] . '" />';
			if ($TEXT) $out_action .= ' IP';
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}
		$isLastPost = $thread->GetLastPostDate() == $post->GetDate();
		if (CheckRights('removepost', $thread->GetForum(), $thread->GetID(), $user->GetID(), $post->GetDate(), $isLastPost) && $writeaccess)
		{
			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr><a href="' . htmlquote('javascript:goDelete("' . $MODULE[post] . h2t(PD) . 'id=' . $post->GetID() . '&page=' . $page . '&delete=yes' . sid() . '")') . '">';
			if ($ICON) $out_action .= '<img ' . $IMG['delete'] . ' title="' . $T['delete'] . '" />';
			if ($TEXT) $out_action .= ' ' . $T['delete'];
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}
		if (CheckRights('editpost', $thread->GetForum(), $thread->GetID(), $user->GetID(), $post->GetDate()) && $writeaccess)
		{
			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr><a href="' . $MODULE['post'] . PD . 'id=' . $post->GetID() . '&amp;page=' . $page . sid() . '">';
			if ($ICON) $out_action .= '<img ' . $IMG['edit'] . ' title="' . $T['change'] . '" />';
			if ($TEXT) $out_action .= ' ' . $T['change'];
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}
		if (CheckRights('writeforum', $thread->GetForum()) && $writeaccess && (!$thread->IsClosed() || CheckRights('closethread')))
		{
			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr><a href="' . $MODULE['post'] . PD . 'thread=' . $threadid . '&amp;quote=' . $post->GetID() . sid() . '">';
			if ($ICON) $out_action .= '<img ' . $IMG['quote'] . ' title="' . $T['quote'] . '" />';
			if ($TEXT) $out_action .= ' ' . $T['quote'];
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}
		if (CheckRights('writeforum', $thread->GetForum()) && $writeaccess && (!$thread->IsClosed() || CheckRights('closethread')))
		{
			if ($n) $out_action .= '&nbsp; ';
			if ($n2) $out_action .= $between;
			$out_action .= '<nobr><a href="' . $MODULE['post'] . PD . 'thread=' . $threadid . '&amp;replyto=' . $post->GetID() . sid() . '">';
			if ($ICON) $out_action .= '<img ' . $IMG['write'] . ' title="' . $T['reply'] . '" />';
			if ($TEXT) $out_action .= ' ' . $T['reply'];
			$out_action .= '</a></nobr>';
			$n++; $n2 = 0;
		}
	}

	// ----- AVATAR -----
	$out_avatar = '';
	if ($post->GetUser() > 0 && rc('avatars_enabled') && !$TEXTONLY && (!$LoginUserID || !($LoginUser->GetFlags() & 1024)))
	{
		if ($user->Load($post->GetUser()))
		{
			// show avatar if set
			if ($user->GetAvatar() != '')
			{
				if (!$NEWLAYOUT)
					$out_avatar .= '<img src="' . avatar_url($user) . '"' . avatar_size($user) . ' alt="' . $T['avatar'] . '" />';
				else
					$out_avatar .= '<img src="' . avatar_url($user) . '"' . avatar_size($user) . ' style="margin-right:15px;" alt="' . $T['avatar'] . '" />';
				#$out_avatar .= '<img src="' . ($user->AvatarFromURL() ? '' : $avatar_path) . $user->GetAvatar() . '" />';
			}
		}
	}

	// ----- USER INFO -----
	$out_ustatus = '';
	$out_utitle = '';
	$out_uinfo = '';
	if ($post->GetUser() > 0)
	{
		if ($user->Load($post->GetUser()))
		{
			$out_ustatus = GetUserStatusText($user->GetID(), '<i>(%s)</i>', true, false);
			if ($user->GetTitle() != '' && rc('show_utitle'))
			{
				$out_utitle .= t2h($user->GetTitle());
			}

			if (!$NEWLAYOUT && $out_avatar) $out_uinfo .= '<br />' . $out_avatar . '<br /><br />';

			if (rc('show_uinfo_regdate'))
				$out_uinfo .= $T['member'] . ' ' . FriendlyDate($user->GetRegDate(), 2, 1, true, 1);

				if (!rc('show_uinfo_posts') || !$NEWLAYOUT)
					$out_uinfo .= '<br />';
				else
					$out_uinfo .= ' &nbsp; &nbsp; ';

			if (rc('show_uinfo_posts'))
			{
				$posts = GetPostsByUser($user->GetID());
				if ($posts > 0)
					$out_uinfo .= '<a href="' . $MODULE['search'] . PD . 'nodef=1&amp;Query=!' . $user->GetID() . '&amp;ResultView=2&amp;InUser=1&amp;Sort=1' . sid() . '">';
				$out_uinfo .= $posts;
				if ($posts > 0)
					$out_uinfo .= '</a>';
				if ($posts == 1)
					$out_uinfo .= ' ' . $T['post'] . '<br />';
				else
					$out_uinfo .= ' ' . $T['posts'] . '<br />';
			}

			if ($user->GetLocation() != '' && rc('show_uinfo_location'))
			{
				$out_uinfo .= $T['location'] . ': ';
				if (rc('location_link'))
					$out_uinfo .= '<a href="' . $MODULE['derefer'] . PD . 'url=' . urlencode(str_replace('%s', urlencode($user->GetLocation()), rc('location_link'))) . '" target="_blank">';
				$out_uinfo .= t2h($user->GetLocation());
				if (rc('location_link'))
					$out_uinfo .= '</a>';
				$out_uinfo .= '<br />';
			}
		}
		else
		{
			$out_ustatus = GetUserStatusText(-1, '<i>(%s)</i>', true, true);
		}
	}
	elseif ($post->GetUser() == 0)
	{
		$out_ustatus = GetUserStatusText(0, '<i>(%s)</i>', true, true);
	}
	elseif ($post->GetUser() == -1)
	{
		$out_ustatus = GetUserStatusText(-1, '<i>(%s)</i>', true, true);
	}

	if ($show_thread)
	{
		// this is a search result list, display forum/thread information with the post
		$forum = new IForum;
		$forumid = $thread->GetForum();
		$forum->Load($forumid);

		if (!$NEWLAYOUT) $classname = 'old';
		else             $classname = 'new';
		echo '<tr valign="top"><td class="list_post_' . $classname . '_thread_' . $style . '" colspan="' . ($NEWLAYOUT ? 2 : 3) . '">';
		echo $T['topic'] . ': <a href="' .
			($CFG['mod_rewrite'] ?
				$threadid . sid(3) :
				$MODULE['thread'] . PD . 'id=' . $threadid . sid()) .
			'"><b>' . t2h($thread->GetSubject()) . '</b></a> &nbsp; ';
		echo '<small>' . $T['forum'] . ': ';
		$f = '';
		do
		{
			if ($f != '') $f = $CFG['forum_separator'] . $f;
			$f = '<a href="' .
				($CFG['mod_rewrite'] ?
					'f.' . $forum->GetID() . sid(3) :
					$MODULE['main'] . PD . 'id=' . $forum->GetID() . sid()) .
				'">' . t2h($forum->GetName()) . '</a>' . $f;
		} while ($forum->Load($forum->GetParent()));
		echo $f . ' &nbsp; &nbsp; ';
		echo MakePostLink($post) . '&nbsp;' . $T['goto_post'];
		echo '</small></td></tr>';
	}

	$TINYLAYOUT = false;

	if (!$NEWLAYOUT)
	{
		echo '<tr valign="top">';
		echo '<td class="list_post_old_uname_' . $style . '">' . $out_username . '</td>';
		echo '<td class="list_post_old_datesubject_' . $style . '">' . $out_date . '</td>';
		echo '<td class="list_post_old_contactaction_' . $style . '" align="right">' . $out_contact . $out_action . '</td>';
		echo '</tr>';

		echo '<tr valign="top">';
		if ($post->GetEditCount() || $post->GetSpamRating() || $post->GetAttachFile() != '')
			echo '<td class="list_post_old_avataruinfo_' . $style . '" rowspan="2">';
		else
			echo '<td class="list_post_old_avataruinfo_' . $style . '">';
		if ($out_utitle) echo $out_utitle . '<br />';
		if ($out_ustatus) echo $out_ustatus . '<br />';
		echo $out_uinfo;
		echo '</td>';
		echo '<td class="list_post_old_message_' . $style . '" colspan="2">';
	}
	elseif (!$TINYLAYOUT)
	{
		echo '<tr valign="middle">';
		echo '<td class="list_post_new_unameinfo_' . $style . '">';
		if ($out_avatar != '')
		{
			echo '<table cellspacing="0" cellpadding="0"><tr valign="middle"><td>' . $out_avatar . '</td><td>';
		}
		echo $out_username . ' &nbsp; ' . $out_contact . ' &nbsp; <small>';
		if ($out_utitle) echo $out_utitle . ' &nbsp; ';
		if ($out_ustatus) echo $out_ustatus;
		echo '<br />';
		echo '<img ' . $IMG['empty'] . ' width="1" height="8" alt="" /><br />';
		echo $out_date . '</small>';
		if ($out_avatar != '')
		{
			echo '</td></tr></table>';
		}
		echo '</td>';

		echo '<td class="list_post_new_date_' . $style . '" align="right">';
		if ($out_uinfo)
		{
			echo $out_uinfo;
			if ($out_action) echo '<img ' . $IMG['empty'] . ' width="1" height="8" alt="" /><br />';
		}
		echo $out_action . '</td>';
		echo '</tr>';

		#if ($TABLE['post']['line']) echo '<tr><td colspan="2"><hr /></td></tr>';   // global $TABLE
		if ($out_subject != '')
		{
			echo '<tr valign="top">';
			echo '<td class="list_post_new_subject_' . $style . '" colspan="2"><b>' . $T['subject'] . ':</b> ' . $out_subject . '</td>';
			echo '</tr>';
		}

		echo '<tr valign="top">';
		echo '<td class="list_post_new_message_' . $style . '" colspan="2">';
	}
	else /* $TINYLAYOUT */
	{
		echo '<tr valign="middle">';
		echo '<td class="list_post_new_unameinfo_' . $style . '">';
		echo $out_username . ' &nbsp; <small>';
		if ($out_ustatus) echo $out_ustatus . ' &nbsp; ';
		echo $out_date . '</small>';
		echo '</td>';

		echo '<td class="list_post_new_date_' . $style . '" align="right">';
		echo $out_action . '</td>';
		echo '</tr>';

		#if ($TABLE['post']['line']) echo '<tr><td colspan="2"><hr /></td></tr>';   // global $TABLE
		if ($out_subject != '')
		{
			echo '<tr valign="top">';
			echo '<td class="list_post_new_subject_' . $style . '" colspan="2"><b>' . $T['subject'] . ':</b> ' . $out_subject . '</td>';
			echo '</tr>';
		}

		echo '<tr valign="top">';
		echo '<td class="list_post_new_message_' . $style . '" colspan="2">';
	}

	// POST MESSAGE, SIGNATURE

	$subsets0 = $abbc_cfg['subsets'];
	if ($post->GetOptions() & 1 || $TEXTONLY) $abbc_cfg['subsets'] &= ~ABBC_SMILEYS;
	if ($post->GetOptions() & 2) $abbc_cfg['subsets'] &= ~ABBC_SPECIAL;
	$html = abbc_proc($post->GetMsg());
	if (is_array($highlight))
		$html = preg_replace('/(' . join('|', array_map('regsafe', $highlight)) . ')/i', '<span class="highlight">$1</span>', $html);
	echo $html;
	$abbc_cfg['subsets'] = $subsets0;

	if ($post->GetUser() > 0 && $user->GetSignature() != '' && (!$LoginUserID || !($LoginUser->GetFlags() & 2048)))
	{
		$subsets0 = $abbc_cfg['subsets'];
		if ($TEXTONLY) $abbc_cfg['subsets'] &= ~ABBC_SMILEYS;
		echo '<br /><div class="signature">' . abbc_proc($user->GetSignature()) . '</div>';
		$abbc_cfg['subsets'] = $subsets0;
	}
#	else
#	{
		echo '&nbsp;';
#	}
#	NOTE: uncommenting this will only output a spare line after posts, not after signatures

	echo '</td>';
	echo '</tr>';

	// EDIT NOTE, ATTACHMENTS

	if ($post->GetEditCount() || $post->GetSpamRating() || $post->GetAttachFile() != '')
	{
		echo '<tr valign="top">';
		if (!$NEWLAYOUT)
			echo '<td class="list_post_old_notes_' . $style . '" colspan="2">';
		else
			echo '<td class="list_post_new_notes_' . $style . '" colspan="2">';

		if ($post->GetEditCount())
		{
			echo str_replace('%s', $post->GetEditCount(), $T['post_changed_n_times']) . ' ' . FriendlyDate($post->GetEditDate(), 0, 3) . ' ' . $T['by'] . ' ';
			if ($user->Load($post->GetEditUser()))
				echo MakeUserLink($user->GetID(), $user->GetName());
			else
				echo $T['unknown'];
			echo '<br />';
		}

		if ($post->GetSpamRating())
		{
			echo $T['post_was_changed_automatically'] . ' <a href="' . htmlquote('javascript:popup("' . $MODULE['textfilter'] . '", 430, 280);') . '">' . $T['why'] . '?</a><br />';
		}

		if ($post->GetAttachFile() != '')
		{
			if (file_exists($attach_path . $post->GetAttachFile()))
			{
				// get type of attachment
				if (display_post_image($attach_path . $post->GetAttachFile()))
				{
					// display the image
					echo '<img ' . $IMG['attach'] . ' />&nbsp; ' . $T['image_attached'] . ': ' .
						t2h($post->GetAttachFileName()) . ' (' .
						@format_number(filesize($attach_path . $post->GetAttachFile())) . ' ' . $T['bytes'] . ')<br /><br />' .
						'<img src="' . $attach_path . $post->GetAttachFile() . '" /><br />&nbsp;';
				}
				else
				{
					if (CheckRights('downloadattach', $thread->GetForum(), $thread->GetID()))
					{
						// show link for download
						echo '<img ' . $IMG['attach'] . ' />&nbsp; ' . $T['file_attached'] . ': ' .
							'<a href="' . $MODULE['thread'] . PD . 'download=' . $post->GetID() . '&amp;inline=1' . sid() . '" target="_blank">' .
							t2h($post->GetAttachFileName()) . '</a> (' .
							'<a href="' . $MODULE['thread'] . PD . 'download=' . $post->GetID() . sid() . '" target="_blank">' .
							$T['save'] . '</a>, ' .
							@format_number(filesize($attach_path . $post->GetAttachFile())) . ' ' . $T['bytes'] . ', ' .
							($post->GetAttachDLCount() ?
								str_replace('%s', $post->GetAttachDLCount(), $T['downloaded_n_times']) :
								$T['never_downloaded']) .
							')<br />';
					}
					else
					{
						echo '<img ' . $IMG['attach'] . ' />&nbsp; ' . $T['file_attached'] . ': ' .
							t2h($post->GetAttachFileName()) . ' (' .
							@format_number(filesize($attach_path . $post->GetAttachFile())) . ' ' . $T['bytes'] . ', ' .
							($post->GetAttachDLCount() ?
								str_replace('%s', $post->GetAttachDLCount(), $T['downloaded_n_times']) :
								$T['never_downloaded']) .
							')<br />';

						echo '<b>' . $T['err_attach_no_permission'] . '</b><br />';
					}
				}
			}
			else
			{
				echo str_replace('%s', t2h($post->GetAttachFileName()), $T['err_attach_not_found']) . '<br />';
			}
		}

		echo '</td>';
		echo '</tr>';
	}
}


// Begin / End <table> framework to display Messages using ShowMessage
//
// part = 1: <table> framework, 2: header line (ORable)
//
function BeginMessageTable($part = 3)
{
	global $TABLE, $T;

	if ($part & 1)
	{
		echo '<div class="messagelist">';
		echo '<table cellspacing="' . $TABLE['msg']['space'] . '" cellpadding="' . $TABLE['msg']['pad'] . '" border="' . $TABLE['msg']['border'] . '" width="100%"' . $TABLE['msg']['fixed'] . '>';
		echo '<colgroup>';
		echo '<col width="' . $TABLE['msg']['col_icon'] . '" />';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';
	}

	if ($part & 2)
	{
		echo '<tr valign="top">';
		echo '<td class="list_message_warn_head_left">&nbsp;</td>';
		echo '<td class="list_message_warn_head_right" colspan="2">' . $T['message'] . '</td>';
		echo '</tr>';
	}
}

function MessageDelimiter()
{
	global $TABLE;

	echo '<tr><td class="list_post_old_delimiter" colspan="3">';
	if ($TABLE['post']['line'] == 1) echo '<hr />';
	else echo '';
	echo '</td></tr>';
}

function EndMessageTable()
{
	echo '</table>';
	echo '</div>';
}


// Show a message, for main.php (Forum list) and post.php (Message preview)
//
// Expects the <table> framework already to be started and to be closed after last message
//
// message = IMessage object set to the message to be shown
// preview = t|f: preview mode? -> no read/unread... controls
// id = >0 if the message is displayed in a forum | <0 is a thread
// page = page in thread
//
function ShowMessage(&$message, $preview = false, $id = 0, $page = 0)
{
	global $IMG, $ME, $MODULE, $LoginUserID, $T;

	$warn = $message->GetOptions() & 1;
	$warnstr = $warn ? '_warn' : '';

	echo '<tr valign="top">';
	echo '<td class="list_message' . $warnstr . '" rowspan="2">';
	if ($warn) echo '<img ' . $IMG['warning'] . ' />'; else echo '<img ' . $IMG['info'] . ' />';
	echo '</td>';

	$forumid = $message->GetForum();
	$user = new IUser;
	$user->Load($message->GetUser());
	echo '<td class="list_message' . $warnstr . '"><span class="date">' . FriendlyDate($message->GetDate(), 1, 3) . ' ' . $T['by'] . ' ' .
		MakeUserLink($user->GetID(), $user->GetName());
	if ($message->GetSubject() != '')
		echo ' | <b>' . t2h($message->GetSubject()) . '</b>';
	echo '</span></td>';

	echo '<td class="list_message' . $warnstr . '" align="right">';
	$x = false;

	switch ($message->GetOptAccess())
	{
		case 1:
			if ($x) $x .= ' | ';
			$x .= '<small>(' . $T['only_for'] . ' ' . $T['guests'] . ')</small>';
			break;
		case 2:
			if ($x) $x .= ' | ';
			$x .= '<small>(' . $T['only_for'] . ' ' . $T['users'] . ')</small>';
			break;
		case 3:
			if ($x) $x .= ' | ';
			$x .= '<small>(' . $T['only_for'] . ' ' . $T['moderators'] . ')</small>';
			break;
	}

	if ($preview)
	{
		if ($x) $x .= ' | ';
		$x .= '(' . $T['preview'] . ')';
	}
	else
	{
		if (CheckRights('editmessage', $forumid))
		{
			if ($x) $x .= ' | ';
			$x .= '<a href="' . $MODULE['post'] . PD . 'message=' . $message->GetID() . sid() . '"><img ' . $IMG['edit'] . ' /> ' . $T['change'] . '</a>';
		}
		if ($LoginUserID)
		{
			if ($x) $x .= ' | ';
			if (!$message->IsRead())
				if ($id > 0)
					$x .= '<a href="' . $ME . PD . 'id=' . $id . '&amp;messageread=' . $message->GetID() . sid() . '">' . $T['mark_read'] . '</a>';
				else
					$x .= '<a href="' . $ME . PD . 'id=' . -$id . '&amp;page=' . $page . '&amp;nocount=1&amp;messageread=' . $message->GetID() . sid() . '">' . $T['mark_read'] . '</a>';
			else
				if ($id > 0)
					$x .= '<a href="' . $ME . PD . 'id=' . $id . '&amp;messageunread=' . $message->GetID() . sid() . '">' . $T['mark_unread'] . '</a>';
				else
					$x .= '<a href="' . $ME . PD . 'id=' . -$id . '&amp;page=' . $page . '&amp;nocount=1&amp;messageunread=' . $message->GetID() . sid() . '">' . $T['mark_unread'] . '</a>';
		}
		if (CheckRights('editmessage', $message->GetForum()))
		{
			if ($x) $x .= ' | ';
			if ($id > 0)
				$x .= '<a href="' . $ME . PD . 'id=' . $id . '&amp;messageallunread=' . $message->GetID() . sid() . '">' . $T['unread_for_all'] . '</a>';
			else
				$x .= '<a href="' . $ME . PD . 'id=' . -$id . '&amp;page=' . $page . '&amp;nocount=1&amp;messageallunread=' . $message->GetID() . sid() . '">' . $T['unread_for_all'] . '</a>';

			$cnt = $message->ReadCount();
			$x .= ' <small>(' . $cnt . ' ' . $T['users'] . ')</small>';
		}
	}

	if ($x) echo $x;
	else echo '&nbsp;';
	echo '</td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_message' . $warnstr . '" colspan="2">' . abbc_proc($message->GetMsg());
	#echo '&nbsp;';
	echo '</td>';
	echo '</tr>';
}

?>