<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// cp.inc.php
// Control Panel and User Profile

require($libpath . 'lang/' . $LANG . '_more.php');
#if ($LoginUserID == 58) echo '<b>cp begin ------ ' . perform_msec() . 'ms</b><br />';

// Import request variables
$userid = $_REQUEST['id'];
if (!$userid) $userid = $LoginUserID;
if (!is_numeric($userid) || $userid < 0) $userid = 0;

$item = $_REQUEST['item'];
$maxitem = 7;
if (CheckRights('is_admin')) $maxitem = 11;
if ($item < 1 || $item > $maxitem) $item = 0;


// See if a date (day, month, year) is valid, taking care of leap years
// Valid years are from 1000 to 9999
//
function MyCheckDate($day, $month, $year)
{
	if ($year < 1000 || $year > 9999) return false;
	if ($month < 1 || $month > 12) return false;
	if ($day < 1 || $day > 31) return false;
	switch ($month)
	{
		case 2:
			if ($day > 28 + ((!($year % 4) && $year % 100) || !($year % 400) ? 1 : 0)) return false;
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			if ($day == 31) return false;
	}
	return true;
}

// Check if an avatar file has correct filetype/filesize
//
// returns:
//   1: no '.' found
//   2: invalid filename extension
//
function is_valid_avatar_name($filename)
{
	$x = strrpos($filename, '.');
	if ($x === false) return 1;
	$ext = substr($filename, $x);
	switch (strtolower($ext))
	{
		// update profile.php, EditProfile, Save, Avatar setting, on change
		case '.jpg':
		case '.jpeg':
		case '.gif':
		case '.png':
			return true;
	}
	return 2;
}

// returns:
//   3: file too big
//   4: not a valid image file
//   5: too wide
//   6: too high
//
function is_valid_avatar_size($filename, $remote = false)
{
	if (!$remote && @filesize($filename) > CheckRights('maxavatarsize')) return 3;   // file too long
	$a = @GetImageSize($filename);
	if ($a === false) return 4;   // not a valid image file
	if ($a[0] > CheckRights('maxavatarwidth')) return 5;    // too wide
	if ($a[1] > CheckRights('maxavatarheight')) return 6;    // too high
	return true;
}

// same for users' photos
//
function is_valid_photo_name($filename)
{
	return is_valid_avatar_name($filename);
}

function is_valid_photo_size($filename, $remote = false)
{
	if (!$remote && @filesize($filename) > CheckRights('maxphotosize')) return 3;   // file too long
	$a = @GetImageSize($filename);
	if ($a === false) return 4;   // not a valid image file
	if ($a[0] > CheckRights('maxphotowidth')) return 5;    // too wide
	if ($a[1] > CheckRights('maxphotoheight')) return 6;    // too high
	return true;
}

// return name of Extra user field.
// $n starts with 1, maximum is $extra_count
//
function GetExtraName($n)
{
	global $extra_names;
	return $extra_names[$n - 1];
}

// How many Extra? columns are present in the Users table
function GetExtraDBCols()
{
	global $db;

	$auser = $db->FastQuery('Users', '*', '', '', 1);
	$n = 1;
	while (isset($auser['Extra' . $n])) $n++;
	$n--;
	return $n;
}

// Set count of Extra? columns in the Users table
// This may cause data loss!
function SetExtraDBCols($newcols)
{
	global $db;

	if (!is_numeric($newcols)) return false;
	if ($newcols < 0) return false;
	if ($newcols > 10) return false;

	$curr = GetExtraDBCols();
	if ($newcols == $curr) return true;
	elseif ($newcols < $curr)
	{
		while ($curr > $newcols) if (!$db->RemoveField('Users', 'Extra' . $curr--)) return false;
		return true;
	}
	else
	{
		while ($curr < $newcols) if (!$db->AddField('Users', 'Extra' . ++$curr, 'VARCHAR(100) NOT NULL')) return false;
		return true;
	}
}

// Unwatch many threads (mode=0) [for current user]
// where = SQL WHERE definition
//
function UnwatchThreads($where, $userid = false)
{
	global $LoginUserID, $db;
	if (!$userid) $userid = $LoginUserID;
	if (!$userid) return false;

	return $db->ChangeRecord(array('Mode' => 0), $where . ' AND User=' . $userid, 'ThreadWatch');
}


$error = false;

// Edit User Profile
//
if ($_POST['action'] == 'edit' && !$_POST['Preview'] && $item <= 7)
{
	$user = new IUser;

	if (!CheckRights('editprofile', 0, 0, $userid))
	{
		addlog('edit_profile ' . $userid . ' no_access');
		$error .= $T['err_access_denied'] . '<br />';
	}

	if ($_POST['RemoveSubscriptions'] == 1 && $_REQUEST['NumOfDays'] != '' && is_numeric($_REQUEST['NumOfDays']))
	{
		$days = intval($_REQUEST['NumOfDays']);
		$ids = $db->FastQuery1stArray('Threads', 'ID', 'LastPostDate < ' . (time() - $days * 3600 * 24));
		if (is_array($ids) && sizeof($ids) > 0) UnwatchThreads('Thread in (' . join(',', $ids) . ')', $userid);
		addlog('remove_subscriptions for ' . $userid . ' before ' . $days . ' days ok');
		ForwardHTML($ME . PD . 'id=' . $userid . '&amp;item=' . $item . sid());
		exit();
	}

	if ($_POST['Remove'] == 1)
	{
		if (CheckRights('removeuser'))
		{
			if ($user->Remove($userid))
			{
				addlog('remove_user ' . $userid . ' ok');
				ForwardHTML($MODULE['users'] . sid(2));
				exit();
			}
			else
			{
				addlog('remove_user ' . $userid . ' error');
				$error .= $T['err_user_not_deleted'] . '<br />';
			}
		}
		else
		{
			addlog('remove_user ' . $userid . ' no_access');
			$error .= $T['err_access_denied'] . '<br />';
		}
	}

	$user->Load($userid);

	if (($_POST['Password'] != '' || $_POST['Password2'] != '') && $_POST['Password'] != $_POST['Password2'])
	{
		$error .= $T['err_passwords_different'] . '<br />';
	}

	if ($_POST['Password'] != '')
	{
		$secure = IsSecurePassword($_POST['Password'], $user->GetName());
		switch ($secure)
		{
			case 0: break;
			case 1: $error .= str_replace('%n', rc('pass_minlength'), $T['err_password_too_short']) . '<br />'; break;
			case 2: $error .= $T['err_password_is_username'] . '<br />'; break;
			case 3: $error .= $T['err_password_need_number'] . '<br />'; break;
			case 4: $error .= $T['err_password_need_special'] . '<br />'; break;
			default: $error .= $T['err_password_generic'] . '<br />'; break;
		}
	}

	if (strpos($_POST['Homepage'], '://') === false && $_POST['Homepage'] != '')
	{
		$_POST['Homepage'] = 'http://' . $_POST['Homepage'];
	}

	if ($_POST['BirthYear'] > 0 && $_POST['BirthYear'] < 100) $_POST['BirthYear'] += 1900;

	if (!MyCheckDate($_POST['BirthDay'], $_POST['BirthMonth'], $_POST['BirthYear']) &&
	    ($_POST['BirthDay'] != '' || $_POST['BirthMonth'] != '' || $_POST['BirthYear'] != ''))
	{
		$error .= $T['err_invalid_birthdate'] . '<br />';
	}

	if (strlen($_POST['Signature']) > rc('max_sig_len'))
	{
		$error .= str_replace('%s', rc('max_sig_len'), $T['err_signature_too_long']) . '<br />';
	}

	if (!$error)
	{
		$ok = false;
		do
		{
			if (isset($_POST['Name']) && $_POST['Name'] != $user->GetName() && CheckRights('renameuser'))
			{
				if (!$user->SetName($_POST['Name'])) break;
				addlog('change_name for ' . $userid . ' to ' . $_POST['Name']);
			}
			if ($_POST['SetUserGroups'] == 1 && $_POST['UserGroups'] != GetUserGroups($userid) && CheckRights('setusergroups'))
			{
				// an admin can't kick himself out of the admin group!
				if ($userid == $LoginUserID && in_array(3, $LoginUserGroups) && !in_array(3, $_POST['UserGroups']))
				{
					array_push($_POST['UserGroups'], 3);
				}

				$groups = $_POST['UserGroups'];
				if (!isset($_POST['UserGroups'])) $groups = array();
				if (!SetUserGroups($userid, $groups)) break;
				addlog('change_groups for ' . $userid . ' to ' . join(', ', $groups));
			}
			if (isset($_POST['Password']) && $_POST['Password'] != '' && $_POST['Password'] == $_POST['Password2'])
			{
				if ($user->GetPassword() != md5(trim($_POST['CurrentPassword'])) && !CheckRights('is_admin'))
				{
					$error = $T['err_invalid_password_cp'] . '<br />';
					break;
				}

				if (!$user->SetPassword(md5($_POST['Password']))) break;
				addlog('change_password for ' . $userid);
			}

			if (isset($_POST['EMail']) && $_POST['EMail'] != $user->GetEMail())
			{
				if ($user->GetPassword() != md5(trim($_POST['CurrentPassword'])) && !CheckRights('is_admin'))
				{
					$error = $T['err_invalid_password_cp'] . '<br />';
					break;
				}

				if (!$user->SetEMail($_POST['EMail'])) break;
			}
			if (isset($_POST['ICQ']) && !$user->SetIM($_POST['ICQ'], $_POST['AIM'], $_POST['YIM'], $_POST['MSN'], $_POST['Jabber'])) break;
			if (isset($_POST['Homepage']) && $_POST['Homepage'] != $user->GetHomepage()) if (!$user->SetHomepage($_POST['Homepage'])) break;

			if (isset($_POST['Gender']) && $_POST['Gender'] != $user->GetGender()) if (!$user->SetGender($_POST['Gender'])) break;
			if (isset($_POST['BirthDay']) && !$user->SetBirthDate($_POST['BirthDay'], $_POST['BirthMonth'], $_POST['BirthYear'])) break;
			if (isset($_POST['Location']) && $_POST['Location'] != $user->GetLocation()) if (!$user->SetLocation($_POST['Location'])) break;
			if (isset($_POST['About']) && $_POST['About'] != $user->GetAbout()) if (!$user->SetAbout($_POST['About'])) break;
			if (isset($_POST['Title']) && $_POST['Title'] != $user->GetTitle()) if (!$user->SetTitle($_POST['Title'])) break;

			for ($n = 1; $n <= $extra_count; $n++)
				if (isset($_POST['Extra' . $n]) && $_POST['Extra' . $n] != $user->GetExtra($n)) if (!$user->SetExtra($n, $_POST['Extra' . $n])) break;

			if (isset($_POST['Signature']) && $_POST['Signature'] != $user->GetSignature()) if (!$user->SetSignature($_POST['Signature'])) break;
			if (isset($_POST['SetNotify']))
			{
				$DefaultNotify = ($_POST['NotifyEMail']                           ? 1 : 0) * 1 +
				                 ($_POST['NotifyICQ']    && $CFG['enable_icq']    ? 1 : 0) * 2 +
				                 ($_POST['NotifyJabber'] && $CFG['enable_jabber'] ? 1 : 0) * 4;
				if ($DefaultNotify != $user->GetDefaultNotify()) if (!$user->SetDefaultNotify($DefaultNotify)) break;
			}
			if (isset($_POST['Design']) && $_POST['Design'] != $user->GetDesign())
			{
				if (!$user->SetDesign($_POST['Design'])) break;
				addlog('change_design for ' . $userid . ' to ' . $_POST['Design'] . ' (' . $DESIGN['name'][$_POST['Design']] . ')');
			}
			if (isset($_POST['SetThreadIcons']))
			{
				$ThreadIcons = ($_POST['ThreadIconsI'] ? 1 : 0) * 1 +
							   ($_POST['ThreadIconsT'] ? 1 : 0) * 2
							   - 1;
				if ($ThreadIcons < 0) $ThreadIcons = 0;
				if ($ThreadIcons != $user->GetThreadIcons())
				{
					if (!$user->SetThreadIcons($ThreadIcons)) break;
					addlog('change_threadicons for ' . $userid . ' to ' . $ThreadIcons);
				}
			}
			if (isset($_POST['SetFlags']))
			{
				$Flags = $user->GetFlags();
				$Flags = ($Flags & ~2) | ($_POST['LastReadLine'] ? 2 : 0);
				$Flags = ($Flags & ~4) | ($_POST['ShowHideCodeBlock'] ? 4 : 0);
				if (CheckRights('is_admin')) $Flags = ($Flags & ~8) | ($_POST['ShowUserReads'] ? 8 : 0);
				$Flags = ($Flags & ~16) | ($_POST['FastReply'] ? 16 : 0);
				$Flags = ($Flags & ~32) | ($_POST['AutoReLogin'] ? 32 : 0);
				$Flags = ($Flags & ~64) | ($_POST['PostCopy'] ? 64 : 0);
				$Flags = ($Flags & ~128) | ($_POST['LinksNewWindow'] ? 128 : 0);
				$Flags = ($Flags & ~256) | ($_POST['NewLayout'] ? 256 : 0);
				$Flags = ($Flags & ~512) | ($_POST['HalfSizeAvatar'] ? 512 : 0);
				$Flags = ($Flags & ~1024) | ($_POST['HideAvatars'] ? 1024 : 0);
				$Flags = ($Flags & ~2048) | ($_POST['HideSigs'] ? 2048 : 0);
				if ($Flags != $user->GetFlags())
				{
					if (!$user->SetFlags($Flags)) break;
					addlog('change_flags for ' . $userid . ' to ' . $Flags);
				}
			}

			if (isset($_POST['Language']) && $_POST['Language'] != $user->GetLanguage())
			{
				if (!$user->SetLanguage($_POST['Language'])) break;
				addlog('change_language for ' . $userid . ' to '. $_POST['Language']);
			}

			if (isset($_POST['Timezone']))
			{
				if (!$user->SetTimezone($_POST['Timezone'], $_POST['TimezoneDS'])) break;
				addlog('change_timezone for ' . $userid . ' to ' . $_POST['Timezone'] . ' (DS ' . $_POST['TimezoneDS'] . ')');
			}

			if (isset($_POST['DateFormat']) && $_POST['DateFormat'] != $user->GetDateFormat())
			{
				if (!$user->SetDateFormat($_POST['DateFormat'])) break;
			}

			$ok = true;
		}
		while (false);

		if (ok)
		{
			if (CheckRights('changeavatar'))
			{
				if (isset($_POST['avatar']) && $_POST['avatar'] == 0) if (!$user->SetAvatar(''))
				{
					$ok = false;
					$error .= $T['err_avatar_not_deleted'] . '<br />';
				}
				if ($_POST['avatar'] == 2)
				{
					if (($err = is_valid_avatar_size($_POST['avatarurl'], true)) !== true)
					{
						$ok = false;
						switch ($err)
						{
							case 1: $a = $T['err_no_dot_filename']; break;
							case 2: $a = $T['err_invalid_file_ext']; break;
								// update com_dataproc.inc.php, is_valid_avatar_name(), on change
							case 3: $a = str_replace('%s', CheckRights('maxavatarsize'), $T['err_file_too_big']); break;
							case 4: $a = $T['err_unknown_file_format']; break;
							case 5: $a = str_replace('%s', rc('avatar_x'), $T['err_image_too_wide']); break;
							case 6: $a = str_replace('%s', rc('avatar_y'), $T['err_image_too_high']); break;
							default: $a = $T['err_unknown_error'] . ': ' . $err;
						}
						$error .= $T['err_invalid_avatar_file'] . ' (' . $a . ')<br />';
					}
					else
					{
						if (!$user->SetAvatar(''))
						{
							$ok = false;
							$error .= $T['err_avatar_not_deleted'] . '<br />';
						}
						else
						{
							$user->SetAvatar($_POST['avatarurl']);
						}
					}
				}
				if ($_POST['avatar'] == 1)
				{
					$name = basename($_FILES['avatarfile']['name']);
					$x = strrpos($name, '.');
					$err = 0;   // avatar error
					if (($err = is_valid_avatar_name($name)) !== true ||
						($err = is_valid_avatar_size($_FILES['avatarfile']['tmp_name'])) !== true)
					{
						$ok = false;
						switch ($err)
						{
							case 1: $a = $T['err_no_dot_filename']; break;
							case 2: $a = $T['err_invalid_file_ext']; break;
								// update com_dataproc.inc.php, is_valid_avatar_name(), on change
							case 3: $a = str_replace('%s', CheckRights('maxavatarsize'), $T['err_file_too_big']); break;
							case 4: $a = $T['err_unknown_file_format']; break;
							case 5: $a = str_replace('%s', rc('avatar_x'), $T['err_image_too_wide']); break;
							case 6: $a = str_replace('%s', rc('avatar_y'), $T['err_image_too_high']); break;
							default: $a = $T['err_unknown_error'] . ': ' . $err;
						}
						$error .= $T['err_invalid_avatar_file'] . ' (' . $a . ')<br />';
					}
					else
					{
						$ext = substr($name, $x);
						$name = 'avatar_' . $userid . $ext;

						if (!file_exists($avatar_path)) mkdir($avatar_path);
						if (!$user->SetAvatar(''))
						{
							$ok = false;
							$error .= $T['err_avatar_not_deleted'] . '<br />';
						}
						else
						{
							if (!move_uploaded_file($_FILES['avatarfile']['tmp_name'], $avatar_path . $name))
							{
								$ok = false;
								$error .= $T['err_avatar_not_saved'] . ' (1)<br />';
							}
							else
							{
								@chmod($avatar_path . $name, 0644);   // prevent access problems by the webserver
								if (!$user->SetAvatar($name))
								{
									$ok = false;
									$error .= $T['err_avatar_not_saved'] . ' (2)<br />';
								}
							}
						}
					}
				}
			}  // CheckRights changeavatar

			if (isset($_POST['photo']) && $_POST['photo'] == 0) if (!$user->SetPhoto(''))
			{
				$ok = false;
				$error .= $T['err_photo_not_deleted'] . '<br />';
			}
			if ($_POST['photo'] == 2)
			{
				if (($err = is_valid_photo_size($_POST['photourl'], true)) !== true)
				{
					$ok = false;
					switch ($err)
					{
						case 1: $a = $T['err_no_dot_filename']; break;
						case 2: $a = $T['err_invalid_file_ext']; break;
							// update com_dataproc.inc.php, is_valid_avatar_name(), on change
						case 3: $a = str_replace('%s', CheckRights('maxphotosize'), $T['err_file_too_big']); break;
						case 4: $a = $T['err_unknown_file_format']; break;
						case 5: $a = str_replace('%s', CheckRights('maxphotowidth'), $T['err_image_too_wide']); break;
						case 6: $a = str_replace('%s', CheckRights('maxphotoheight'), $T['err_image_too_high']); break;
						default: $a = $T['err_unknown_error'] . ': ' . $err;
					}
					$error .= $T['err_invalid_photo_file'] . ' (' . $a . ')<br />';
				}
				else
				{
					if (!$user->SetPhoto(''))
					{
						$ok = false;
						$error .= $T['err_photo_not_deleted'] . '<br />';
					}
					else
					{
						$user->SetPhoto($_POST['photourl']);
					}
				}
			}
			if ($_POST['photo'] == 1)
			{
				$name = basename($_FILES['photofile']['name']);
				$x = strrpos($name, '.');
				$err = 0;   // avatar error
				if (($err = is_valid_photo_name($name)) !== true ||
				    ($err = is_valid_photo_size($_FILES['photofile']['tmp_name'])) !== true)
				{
					$ok = false;
					switch ($err)
					{
						case 1: $a = $T['err_no_dot_filename']; break;
						case 2: $a = $T['err_invalid_file_ext']; break;
							// update com_dataproc.inc.php, is_valid_avatar_name(), on change
						case 3: $a = str_replace('%s', CheckRights('maxphotosize'), $T['err_file_too_big']); break;
						case 4: $a = $T['err_unknown_file_format']; break;
						case 5: $a = str_replace('%s', CheckRights('maxphotowidth'), $T['err_image_too_wide']); break;
						case 6: $a = str_replace('%s', CheckRights('maxphotoheight'), $T['err_image_too_high']); break;
						default: $a = $T['err_unknown_error'] . ': ' . $err;
					}
					$error .= $T['err_invalid_photo_file'] . ' (' . $a . ')<br />';
				}
				else
				{
					$ext = substr($name, $x);
					$name = 'photo_' . $userid . $ext;

					if (!file_exists($photo_path)) mkdir($photo_path);
					if (!$user->SetPhoto(''))
					{
						$ok = false;
						$error .= $T['err_avatar_not_deleted'] . '<br />';
					}
					else
					{
						if (!move_uploaded_file($_FILES['photofile']['tmp_name'], $photo_path . $name))
						{
							$ok = false;
							$error .= $T['err_photo_not_saved'] . ' (1)<br />';
						}
						else
						{
							@chmod($photo_path . $name, 0644);   // prevent access problems by the webserver
							if (!$user->SetPhoto($name))
							{
								$ok = false;
								$error .= $T['err_photo_not_saved'] . ' (2)<br />';
							}
						}
					}
				}
			}
		}

		if ($ok)
		{
			addlog('edit_profile ' . $userid . ' item ' . $item . ' ok');
			ForwardHTML($ME . PD . 'id=' . $userid . '&amp;item=' . $item . sid());
			exit();
		}
		else
		{
			addlog('edit_profile ' . $userid . 'item ' . $item . ' error');
			$error .= $T['err_one_item_not_saved'];
			if ($user->db->LastError() != '') $error .= ' (' . t2h($user->db->LastError()) . ')';
		}

		unset($user);
	}
}

// Change system configuration
//
if ($_POST['action'] == 'edit' && !$_POST['Preview'] && $item > 7 && CheckRights('is_admin'))
{
	if ($item == 8)
	{
		$CONFFILE['forum_title'] = $_POST['ForumTitle'];
		$CONFFILE['home_url'] = TrailingSlash($_POST['HomeURL'], false);
		$CONFFILE['parent_url'] = TrailingSlash($_POST['ParentURL'], false);
		$CONFFILE['db_server'] = $_POST['DbServer'];
		$CONFFILE['db_user'] = $_POST['DbUser'];
		if (($_POST['DbPass'] == '' || trim($_POST['DbPass']) != '')) $CONFFILE['db_pass'] = base64_encode($_POST['DbPass']);
		$CONFFILE['db_name'] = $_POST['DbName'];
		$CONFFILE['db_prefix'] = $_POST['DbPrefix'];
		$CONFFILE['smtp_server'] = $_POST['SmtpServer'];
		$CONFFILE['smtp_sender'] = $_POST['SmtpSender'];
		$CONFFILE['smtp_user'] = $_POST['SmtpUser'];
		if (($_POST['SmtpPass'] == '' || trim($_POST['SmtpPass']) != '')) $CONFFILE['smtp_pass'] = base64_encode($_POST['SmtpPass']);
		$CONFFILE['jabber_server'] = $_POST['JabberServer'];
		$CONFFILE['jabber_user'] = $_POST['JabberUser'];
		if (($_POST['JabberPass'] == '' || trim($_POST['JabberPass']) != '')) $CONFFILE['jabber_pass'] = base64_encode($_POST['JabberPass']);

		// [Disabled ICQ transport]
		//$CONFFILE['jabber_icq'] = $_POST['JabberICQ'];
		$CONFFILE['def_lang'] = $_POST['DefLanguage'];
		$CONFFILE['tz_offset'] = $_POST['DefTimezone'];
		$CONFFILE['tz_dst'] = $_POST['DefTimezoneDS'];
	}

	if ($item == 9)
	{
		$CONFFILE['threads_per_page'] = intval($_POST['ThreadsPerPage']);
		$CONFFILE['posts_per_page'] = intval($_POST['PostsPerPage']);
		$CONFFILE['users_per_page'] = intval($_POST['UsersPerPage']);
		$CONFFILE['hot_thread_posts'] = intval($_POST['HotThreadPosts']);
		$CONFFILE['hot_thread_views'] = intval($_POST['HotThreadViews']);

		$CONFFILE['design'] = $_POST['DefDesign'];

		$CONFFILE['nav_line'] = $_POST['NavLine'] ? 1 : 0;
		$CONFFILE['login_top'] = $_POST['LoginTop'] ? 1 : 0;
		$CONFFILE['show_online_users'] = $_POST['ShowOnlineUsers'] ? 1 : 0;
		$CONFFILE['foot_line'] = $_POST['ShowFootLine'] ? 1 : 0;
		$CONFFILE['foot_db_time'] = $_POST['FootLineDbTime'] ? 1 : 0;

		$CONFFILE['show_goto_forum'] = $_POST['ShowGotoForum'] ? 1 : 0;
		$CONFFILE['show_search_forum'] = $_POST['ShowSearchForum'] ? 1 : 0;

		$CONFFILE['post_show_textlength'] = $_POST['PostShowTextlength'] ? 1 : 0;
		$CONFFILE['small_layout'] = $_POST['SmallLayout'] ? 1 : 0;
		$CONFFILE['post_use_rcol'] = $_POST['PostUseRcol'] ? 1 : 0;

		$CONFFILE['own_posts_in_threadlist'] = $_POST['OwnPostsInThreadlist'] ? 1 : 0;
		$CONFFILE['display_thread_startdate'] = $_POST['DisplayThreadStartdate'] ? 1 : 0;
		$CONFFILE['advanced_thread_counter'] = $_POST['AdvancedThreadCounter'] ? 1 : 0;
		$CONFFILE['count_thread_views'] = $_POST['CountThreadViews'] ? 1 : 0;
		$CONFFILE['display_thread_lastposter'] = $_POST['DisplayThreadLastposter'] ? 1 : 0;
		$CONFFILE['count_forum_threads_posts'] = $_POST['CountForumThreadsPosts'] ? 1 : 0;
		$CONFFILE['display_forum_lastpost'] = $_POST['DisplayForumLastpost'] ? 1 : 0;
		$CONFFILE['new_layout'] = $_POST['NewLayout'] ? 1 : 0;
		$CONFFILE['enable_trace_users'] = $_POST['EnableTraceUsers'] ? 1 : 0;

		$CONFFILE['show_utitle'] = $_POST['ShowUTitle'] ? 1 : 0;
		$CONFFILE['show_uinfo_regdate'] = $_POST['ShowUInfoRegdate'] ? 1 : 0;
		$CONFFILE['show_uinfo_posts'] = $_POST['ShowUInfoPosts'] ? 1 : 0;
		$CONFFILE['show_uinfo_location'] = $_POST['ShowUInfoLocation'] ? 1 : 0;

		$CONFFILE['smilies'] = $_POST['ShowSmilies'] ? 1 : 0;
		$CONFFILE['smilie_cols'] = intval($_POST['SmilieCols']);
		$CONFFILE['smilie_rows'] = intval($_POST['SmilieRows']);
		$CONFFILE['special_chars'] = $_POST['ShowSpecialChars'] ? 1 : 0;
		$CONFFILE['special_cols'] = intval($_POST['SpecialCols']);
		$CONFFILE['special_rows'] = intval($_POST['SpecialRows']);
		$CONFFILE['location_link'] = $_POST['LocationLink'];
		$CONFFILE['poll_current_days'] = intval($_POST['PollCurrentDays']);
		$CONFFILE['quote_with_date'] = intval($_POST['QuoteWithDate']);
	}

	if ($item == 10)
	{
		$CONFFILE['new_user_validation'] = $_POST['NewUserValidation'];
		$CONFFILE['avatars_enabled'] = $_POST['AvatarsEnabled'] ? 1 : 0;
		$CONFFILE['avatar_x'] = intval($_POST['AvatarX']);
		$CONFFILE['avatar_y'] = intval($_POST['AvatarY']);
		$CONFFILE['photos_enabled'] = $_POST['PhotosEnabled'] ? 1 : 0;
		$CONFFILE['photo_x'] = intval($_POST['PhotoX']);
		$CONFFILE['photo_y'] = intval($_POST['PhotoY']);
		$CONFFILE['max_post_len'] = intval($_POST['MaxPostLen']);
		$CONFFILE['max_sig_len'] = intval($_POST['MaxSigLen']);
		$CONFFILE['attach_bytes'] = intval($_POST['AttachBytes']);
		$CONFFILE['prog_id'] = $_POST['ProgID'];
		$CONFFILE['disallowed_usernames'] = $_POST['DisallowedUsernames'];
		$CONFFILE['disallowed_emails'] = $_POST['DisallowedEmails'];
		$CONFFILE['allowed_email_domains'] = $_POST['AllowedEmailDomains'];

		$CONFFILE['pass_minlength'] = intval($_POST['PassMinlength']);
		$CONFFILE['pass_notusername'] = $_POST['PassNotusername'] ? 1 : 0;
		$CONFFILE['pass_neednumber'] = $_POST['PassNeednumber'] ? 1 : 0;
		$CONFFILE['pass_needspecial'] = $_POST['PassNeedspecial'] ? 1 : 0;
	}

	if ($item == 11)
	{
		$n = sizeof(explode('|', $_POST['ExtraNames']));
		if ($n > 10)
		{
			$error .= $T['err_too_many_extra_fields'] . '<br />';
		}
		elseif (!SetExtraDBCols($n))
		{
			$error .= $T['err_db_setextracols'] . t2h($db->LastError()) . '<br />';
		}
		else
		{
			$CONFFILE['extra_names'] = $_POST['ExtraNames'];
			$CONFFILE['lib_path'] = TrailingSlash($_POST['LibPath']);
			$CONFFILE['design_path'] = TrailingSlash($_POST['DesignPath']);
			$CONFFILE['avatar_path'] = TrailingSlash($_POST['AvatarPath']);
			$CONFFILE['photo_path'] = TrailingSlash($_POST['PhotoPath']);
			$CONFFILE['attach_path'] = TrailingSlash($_POST['AttachPath']);
			$CONFFILE['log_path'] = TrailingSlash($_POST['LogPath']);
			$CONFFILE['show_sql'] = $_POST['ShowSQL'];
			$CONFFILE['admin_lock'] = $_POST['AdminLock'] ? 1 : 0;
			$CONFFILE['read_only'] = $_POST['ReadOnly'] ? 1 : 0;
			$CONFFILE['enable_icq'] = $_POST['EnableICQ'] ? 1 : 0;
			$CONFFILE['enable_jabber'] = $_POST['EnableJabber'] ? 1 : 0;

			$message = new IMessage;
			if ($message->Find(-1)) $message->Remove();
			if (trim($_POST['MaintenanceMsg']) != '')
			{
				$message->Add(-1, 0, trim($_POST['MaintenanceMsg']), 0);
			}
		}
	}

	if (!$error && !rebuild_conffile()) $error .= $T['err_write_conffile'] . '<br />';

	if (!$error)
	{
		addlog('change_config item ' . $item);
		ForwardHTML($ME . PD . 'id=' . $userid . '&amp;item=' . $item . sid());
		exit();
	}
}


// Send E-Mail to User
//
if ($_POST['action'] == 'sendemail')
{
	if (!CheckRights('sendemail'))
	{
		$error .= $T['err_access_denied'] . '<br />';
	}

	if (($_POST['id'] == '' || trim($_POST['Msg']) == ''))
	{
		$error .= $T['err_form_not_complete'] . '<br />';
	}

	$user = new IUser;
	if (!$user->Load($userid))
	{
		$error .= $T['err_invalid_user'] . '<br />';
	}

	if (!$error)
	{
		$subject_key = 'usermail_subject';
		$subject_data = array('%s' => $LoginUserName);

		$msg_data = array(
			'%name%' => $user->GetName(),
			'%poster%' => $LoginUserName,
			'%msg%' => trim($_POST['Msg']),
			'%email%' => $LoginUser->GetEMail(),
			'%url%' => rc('home_url') . '/' . $MODULE['cp'] . h2t(PD) . 'id=' . $LoginUserID . '&action=email'
			);

		if ($_POST['reply_email'] == 1)
		{
			$msg_key = 'usermail_1';
			$from = $LoginUser->GetEMail();
		}
		else
		{
			$msg_key = 'usermail_2';
			$from = false;
		}

		if (NotifyUser($userid, 1, $subject_key, $subject_data, $msg_key, $msg_data, $from))
		{
			addlog("From: $LoginUserName ($LoginUserID)\r\n" .
				"To: " . $user->GetName() . " ($userid)\r\n" .
				"Date: " . date("D, d.m.Y, H:i:s") . "\r\n" .
				($_POST['reply_email'] == 1 ? "Reply-to: $from\r\n" : "") .
				"\r\n" .
				trim($_POST['Msg']),
				true);
			addlog('email_user ' . $userid . ' ok');
			ForwardHTML($ME . PD . 'id=' . $userid . '&amp;action=emailsuccess' . sid());
			exit();
		}
		else
		{
			addlog('email_user ' . $userid . ' error');
			$error .= $T['err_message_not_sent'] . '<br />';
			if ($mail_error != '') $error .= t2h(rtrim($mail_error), 1, 1, 1) . '<br />';
		}

		unset($user);
	}
}


// Show User Profile for userid
//
function ShowProfile($userid)
{
	// Calculate a User's age from his birthday
	//
	function age_from_birthday($day, $month, $year)
	{
		$age = date('Y') - $year;

		if ($month < date('m')) return $age;
		if ($month > date('m')) return $age - 1;
		if ($day <= date('d')) return $age;
		return $age - 1;
	}

	global $MODULE, $ME, $IMG, $TABLE, $DESIGN, $extra_count, $avatar_path, $T;

	CountUserPosts();

	$thread = new IThread;
	$forum = new IForum;

	$user = new IUser;
	if (!$user->Load($userid)) return false;

	// ---------- 1ST BLOCK ----------
	echo '<table width="100%" cellspacing="0" cellpadding="0"><tr valign="bottom">';
	if ($user->GetPhoto() != '' && rc('photos_enabled'))
	{
		echo '<td><img src="' . photo_url($user) . '" align="left" style="margin:4px 15px 4px 0px;" alt="' . $T['user_photo'] . '" /></td>';
	}
	echo '<td width="90%">' . GetUserOnlineImg($user->GetOnline()) . '&nbsp; ';
	echo '<span class="username">' . t2h($user->GetName()) . '</span></td>';
	echo '<td align="right">';
	if (CheckRights('editprofile', 0, 0, $userid))
		echo '<nobr><a href="' . $ME . PD . 'id=' . $userid . '&amp;item=1' . sid() . '"><img ' . $IMG['edit'] . ' /> ' . $T['edit_profile'] . '</a></nobr>';
	else
		echo '&nbsp;';
	echo '</td>';
	echo '</tr></table><br />';

	echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad'] . '" border="' . $TABLE['cp']['border'] . '" width="100%"' . $TABLE['cp']['fixed'] . '>';
	echo '<colgroup>';
	echo '<col width="' . $TABLE['cp']['col_profile1'] . '" />';   // update at extra fields display, too!
	echo '<col width="' . $TABLE['cp']['col_profile2'] . '" />';
	echo '<col width="' . $TABLE['cp']['col_profile1'] . '" />';
	echo '<col width="' . $TABLE['cp']['col_profile2'] . '" />';
	echo '</colgroup>';

	// ---------- 2ND BLOCK ----------
	echo '<tr valign="top">';
	echo '<td class="list_cp_head_left" colspan="2">' . $T['posts_and_stats'] . '</td>';
	echo '<td class="list_cp_head_right" colspan="2">' . $T['contacting'] . '</td>';
	echo '</tr>';

	// REG DATE, E-MAIL/MESSENGER
	echo '<tr valign="top">';
	echo '<td class="list_cp_n" width="' . $TABLE['cp']['col_profile1'] . '">' . $T['registered_on'] . '</td>';
	echo '<td class="list_cp_n" width="' . $TABLE['cp']['col_profile2'] . '">' . FriendlyDate($user->GetRegDate(), 1) . '</td>';

	echo '<td class="list_cp_x" colspan="2" rowspan="4">';
	echo '<table cellspacing="1" cellpadding="0">';
	if ($user->GetEMail() != '')
	{
		echo '<tr>';
		echo '<td>' . $T['e_mail'] . ' &nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['email'] . ' title="' . $T['e_mail'] . '" /></td>';
		echo '<td>&nbsp; <a href="' . $ME . PD . 'id=' . $userid . '&amp;action=email' . sid() . '">' .
			str_replace('%s', t2h($user->GetName()), $T['send_mail_to_x']) . '</a></td>';
		echo '</tr>';
	}
	$im_cnt = 0;
	if ($user->GetRealJabber() != '')
	{
		echo '<tr>';
		if (!$im_cnt++) echo '<td>' . $T['instant_messenger'] . '</td>'; else echo '<td>&nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['jabber'] . ' title="' . $T['jabber'] . '" /></td>';
		echo '<td>&nbsp; <a href="mailto:' . maskmail(t2h($user->GetRealJabber())) . '">' .
			maskmail(t2h($user->GetRealJabber())) . '</a></td>';
		echo '</tr>';
	}
	if ($user->GetICQ() != '')
	{
		echo '<tr>';
		if (!$im_cnt++) echo '<td>' . $T['instant_messenger'] . ' &nbsp;</td>'; else echo '<td>&nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['icq'] . ' title="' . $T['icq'] . '" /></td>';
		echo '<td>&nbsp; <a href="http://web.icq.com/wwp/' . urlencode($user->GetICQ()) . '" target="_blank">' .
			t2h($user->GetICQ()) . '</a></td>';
		echo '</tr>';
	}
	if ($user->GetAIM() != '')
	{
		echo '<tr>';
		if (!$im_cnt++) echo '<td>' . $T['instant_messenger'] . '</td>'; else echo '<td>&nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['aim'] . ' title="' . $T['aim'] . '" /></td>';
		echo '<td>&nbsp; ' . t2h($user->GetAIM()) . '</td>';
		echo '</tr>';
	}
	if ($user->GetYIM() != '')
	{
		echo '<tr>';
		if (!$im_cnt++) echo '<td>' . $T['instant_messenger'] . '</td>'; else echo '<td>&nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['yim'] . ' title="' . $T['yim'] . '" /></td>';
		echo '<td>&nbsp; ' . t2h($user->GetYIM()) . '</td>';
		echo '</tr>';
	}
	if ($user->GetMSN() != '')
	{
		echo '<tr>';
		if (!$im_cnt++) echo '<td>' . $T['instant_messenger'] . '</td>'; else echo '<td>&nbsp;</td>';
		echo '<td align="center"><img ' . $IMG['msn'] . ' title="' . $T['msn'] . '" /></td>';
		echo '<td>&nbsp; <a href="mailto:' . maskmail(t2h($user->GetMSN())) . '">' .
			maskmail(t2h($user->GetMSN())) . '</a></td>';
		echo '</tr>';
	}
	echo '</table>';
	echo '</td>';
	echo '</tr>';

	// POSTS
	$post_count = GetPostsByUser($userid);
	$day_count = (time() - $user->GetRegDate()) / 3600 / 24;

	$post = new IPost;
	if ($post->Find('User=' . $userid, 'Date ASC', 1))
	{
		$date0 = $post->GetDate();
		$post->Find('User=' . $userid, 'Date DESC', 1);
		$date1 = $post->GetDate();
		$day_count_eff = ($date1 - $date0) / 3600 / 24;
	}
	else
		$day_count_eff = 0;

	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['posts'] . '</td>';
	echo '<td class="list_cp_n">';
	if ($post_count)
	{
		echo $post_count . ' &nbsp; <small>(';

		if ($post_count / $day_count >= 1)
			echo round($post_count / $day_count, 1) . ' ' . $T['per_day'];
		else
			echo round($post_count / $day_count * 7, 1) . ' ' . $T['per_week'];

		if ($day_count_eff)
		{
			echo ', ' . $T['effective'] . ' ';
			if ($post_count / $day_count_eff >= 1)
				echo round($post_count / $day_count_eff, 1) . ' ' . $T['per_day'];
			else
				echo round($post_count / $day_count_eff * 7, 1) . ' ' . $T['per_week'];
		}
		echo ')</small>';

		echo ' &nbsp; <a href="' . $MODULE['search'] . PD . 'nodef=1&amp;Query=!' . $userid . '&amp;ResultView=2&amp;InUser=1&amp;Sort=1' . sid() . '">' . $T['find_posts'] . '</a>';
	}
	else
	{
		echo $T['none'];
	}
	echo '</td>';
	echo '</tr>';

	// LAST ACTIVITY
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['last_activity'] . '</td>';


	echo '<td class="list_cp_n">';
	if ($user->GetLastActivity())
	{
		echo FormatTime($user->GetLastActivity(), 3);

		if ($user->GetLastLogin())
		{
			if (rc('small_layout')) echo '<br />';
			else                    echo ' &nbsp; ';
			echo '<small>(' . $T['online'] . ' ' . FriendlyDate($user->GetLastActivity() - $user->GetLastLogin(), 3, 6, false, 2) . ')</small>';
		}

		if ($user->GetLastForum() == 0)
			echo '<br />[' . $T['overview'] . ']';
		elseif ($user->GetLastForum() < 0)
		{
			if ($user->GetLastForum() == ULF_HELP) echo '<br />[' . $T['help'] . ']';
			elseif ($user->GetLastForum() == ULF_CONFIG) echo '<br />[' . $T['config'] . ']';
			elseif ($user->GetLastForum() == ULF_PROFILE) echo '<br />[' . $T['user_profile'] . ']';
			elseif ($user->GetLastForum() == ULF_SEARCH) echo '<br />[' . $T['search'] . ']';
			elseif ($user->GetLastForum() == ULF_STAT) echo '<br />[' . $T['statistics'] . ']';
			elseif ($user->GetLastForum() == ULF_USERS) echo '<br />[' . $T['userlist'] . ']';
		}
		elseif ($forum->Load($user->GetLastForum()))
			echo '<br />' . $T['in_forum'] . ' <a href="' . $MODULE['main'] . PD . 'id=' . $forum->GetID() . sid() . '">' . t2h($forum->GetName()) . '</a>';
	}
	else
	{
		echo $T[never];
	}
	echo '</td>';

	echo '</tr>';

	// LAST POST
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['last_post'] . '</td>';
	echo '<td class="list_cp_n">';
	$record = GetLastPost('User=' . $userid);
	if ($record)
	{
		$thread->Load($record['Thread']);
		$forum->Load($thread->GetForum());

		echo FriendlyDate($record['Date'], 1, 3) . ' &nbsp; ' . MakePostLink($record) . '<br />';
		echo $T['in_thread'] . ' <a href="' . $MODULE['thread'] . PD . 'id=' . $record['Thread'] . sid() . '">' . t2h($thread->GetSubject()) . '</a><br />';
		echo $T['in_forum'] . ' <a href="' . $MODULE['main'] . PD . 'id=' . $forum->GetID() . sid() . '">' . t2h($forum->GetName()) . '</a>';
	}
	else
	{
		echo '';
	}
	echo '</td>';

	echo '</tr>';

	echo '<tr><td colspan="4">&nbsp;</td></tr>';

	// ---------- 3RD BLOCK ----------
	echo '<tr valign="top">';
	echo '<td class="list_cp_head_left" colspan="2">' . $T['more_information'] . '</td>';
	echo '<td class="list_cp_head_right" colspan="2">&nbsp;</td>';
	echo '</tr>';

	// HOMEPAGE, GROUPS
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['homepage'] . '</td>';
	echo '<td class="list_cp_n">';
	if ($user->GetHomepage() != '')
	{
		echo '<a href="' . $MODULE['derefer'] . PD . 'url=' . urlencode($user->GetHomepage()) . '" target="_blank">' . str_replace('/', '/<wbr>', t2h($user->GetHomepage())) . '</a>';
	}
	else
		echo '&nbsp;';
	echo '</td>';

	echo '<td class="list_cp_x" rowspan="2">' . $T['group_membership'] . '</td>';
	echo '<td class="list_cp_n" rowspan="2">';
	$groups = GetUserGroupNames($userid);
	if ($groups) foreach ($groups as $g) echo t2h($g) . '<br />';
	else echo '(' . $T['none'] . ')';
	echo '</td>';
	echo '</tr>';

	// LOCATION
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['location'] . '</td>';
	echo '<td class="list_cp_n">';
	if (rc('location_link'))
		echo '<a href="' . $MODULE['derefer'] . PD . 'url=' . urlencode(str_replace('%s', urlencode($user->GetLocation()), rc('location_link'))) . '" target="_blank">';
	echo t2h($user->GetLocation());
	if (rc('location_link'))
		echo '</a>';
	echo '&nbsp;</td></tr>';

	// BIRTHDAY, TITLE
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['birthdate'] . '</td>';
	echo '<td class="list_cp_n">';
	if ($user->GetBirthDay() > 0)
	{
		$btime = array('year' => $user->GetBirthYear(),
			'month' => $user->GetBirthMonth(),
			'day' => $user->GetBirthDay());
		echo bb_date($T['dateformat_short'], $btime);
		echo ' &nbsp; <small>(' . age_from_birthday($user->GetBirthDay(), $user->GetBirthMonth(), $user->GetBirthYear()) . ' ' . $T['years'] . ')</small>';
	}
	else
		echo '&nbsp;';
	echo '</td>';

	echo '<td class="list_cp_x">' . $T['user_title'] . '</td>';
	echo '<td class="list_cp_n">' . t2h($user->GetTitle()) . '&nbsp;</td>';
	echo '</tr>';

	// GENDER
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['gender'] . '</td>';
	echo '<td class="list_cp_n">';
	$im = GetGenderImage($user->GetGender());
	if ($im != '') echo $im . '&nbsp; ';
	echo t2h($user->GetGenderVerbose()) . '&nbsp;</td>';
	$x = ($p ? $_POST['Gender'] : $user->GetGender());

	echo '<td class="list_cp_x" colspan="2">&nbsp;</td>';
	echo '</tr>';

	// ABOUT, AVATAR
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['description'] . '</td>';
	echo '<td class="list_cp_n">' . t2h($user->GetAbout(), true, true, true) . '&nbsp;</td>';

	if ($user->GetAvatar() != '' && rc('avatars_enabled'))
	{
		echo '<td class="list_cp_x">' . $T['avatar'] . '</td>';
		echo '<td class="list_cp_n"><img src="' . avatar_url($user) . '"' . avatar_size($user, false) . ' vspace="4" alt="' . $T['avatar'] . '" /></td>';
	}
	else
	{
		echo '<td class="list_cp_x" colspan="2">&nbsp;</td>';
	}
	echo '</tr>';

	// EXTRA, SIGNATURE
	if ($extra_count || $user->GetSignature())
	{
		echo '<tr valign="top">';
		echo '<td class="list_cp_n" colspan="2">';

		if ($extra_count > 0)
		{
			echo '<table cellspacing="0" cellpadding="0" width="100%"' . $TABLE['cp']['fixed'] . '>';
			echo '<colgroup>';
			echo '<col width="' . $TABLE['cp']['col_profile1_2x'] . '" />';
			echo '<col width="' . $TABLE['cp']['col_profile2_2x'] . '" />';
			echo '</colgroup>';
			for ($n = 1; $n <= $extra_count; $n++)
			{
				echo '<tr valign="top">';
				echo '<td class="list_cp_e">' . GetExtraName($n) . '</td>';
				echo '<td class="list_cp_e">' . t2h($user->GetExtra($n)) . '&nbsp;</td>';
				echo '</tr>';
			}
			echo '</table>';
		}
		else
		{
			echo '&nbsp;';
		}

		echo '</td>';
		echo '<td class="list_cp_x" colspan="2">';
		if ($user->GetSignature()) echo $T['signature'] . '<br />' . abbc_proc($user->GetSignature());
		echo '&nbsp;</td>';
		echo '</tr>';
	}

	echo '</table>';

	return true;
}


// Show form to send an e-mail to a User
//
function EMailForm($userid)
{
	global $ME, $TABLE, $T;

	if (!CheckRights('sendemail'))
	{
		echo '<p class="error">' . $T['err_access_denied'] . '</p>';
		return false;
	}

	$user = new IUser;
	if (!$user->Load($userid)) return false;

	if ($user->GetEMail() == '')
	{
		echo '<p class="error">' . $T['err_user_has_no_email'] . '</p>';
		return false;
	}

	echo '<form name="form" action="' . $ME . PD . 'id=' . $userid . sid() . '" method="post">';
	echo '<input type="hidden" name="action" value="sendemail" />';
	echo '<input type="hidden" name="id" value="' . $userid . '" />';

	echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad'] . '" border="' . $TABLE['cp']['border'] . '" width="100%">';
	echo '<colgroup>';
	echo '<col width="150" />';
	echo '<col />';
	echo '<col width="10" />';
	echo '</colgroup>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_head_left" colspan="2">' . $T['send_mail_to_user'] . '</td>';
	echo '<td class="list_cp_head_right" width="10">&nbsp;</td>';
	echo '</tr>';

	# NAME
	echo '<tr>';
	echo '<td class="list_cp_n">' . $T['receipient'] . '</td>';
	echo '<td class="list_cp_n">' . t2h($user->GetName()) . '</td>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '</tr>';

	# MESSAGE
	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['text'] . '</td>';
	echo '<td class="list_cp_n"><textarea class="text" name="Msg" cols="70" rows="18">' . t2h($_POST['Msg']) . '</textarea></td>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '</tr>';

	# REPLY BY E-MAIL
	echo '<tr>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '<td class="list_cp_n"><input type="checkbox" class="radio" name="reply_email" id="reply_email" value="1"' . ($_POST['reply_email'] ? ' checked="checked"' : '') . ' /><label for="reply_email">' . $T['allow_direct_reply'] . '</label><br /><small>' . $T['direct_reply_desc'] . '</small></td>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '</tr>';

	echo '<tr>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '<td class="list_cp_n"><input type="submit" name="send" class="button" value="' . $T['send'] . '" accesskey="' . $T['send_key'] . '" onclick="disableObj(\'document.form.send\');" /> <small>' . $T['send_key_name'] . '</small></td>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '</tr>';

	echo '</table>';

	echo '</form>';

	return true;
}


// Display User control panel
//
function CPForm($userid, $item = 1)
{
	global $LoginUserName, $LoginUserGroups, $LoginUserID, $MODULE, $ME, $IMG, $TABLE, $DESIGN, $extra_count, $avatar_path, $CFG, $T, $TIMEZONE, $error, $ICON, $TEXT, $db;

	$titles = array($T['cp_item_acc_settings'], $T['cp_item_pers_profile'], $T['cp_item_subscriptions'], $T['cp_item_bookmarks'], $T['cp_item_sig_avatar'], $T['cp_item_appearance'], $T['cp_item_localisation'],
		$T['cfg_general'], $T['cfg_appearance'], $T['cfg_security'], $T['cfg_advanced']);

	$width = $TABLE['cp']['col_optname'];

	$user = new IUser;
	$user->Load($userid);

	CountUserPosts();

	$p = $error || $_POST['Preview'];

	echo '<table cellspacing="0" cellpadding="0" width="100%">';
	echo '<colgroup>';
	echo '<col width="' . $TABLE['cp']['col_menu'] . '" />';
	echo '<col width="' . $TABLE['cp']['col_menu2'] . '" />';
	echo '<col />';
	echo '</colgroup>';

	echo '<tr valign="top">';
	echo '<td width="' . $TABLE['cp']['col_menu'] . '">';

	// BEGIN control panel menu
	echo '<table class="cp_pages" cellspacing="0" cellpadding="0" width="100%">';
	echo '<tr valign="top">';
	echo '<td class="list_cp_head_left">' . $T['cp_menu'] . '</td>';
	echo '<td class="list_cp_head_right">&nbsp;</td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_l" colspan="2">';
	echo '<table cellspacing="0" cellpadding="0" width="100%">';
	for ($n = 0; $n < 7; $n++)
	{
		echo '<tr><td' . ($item - 1 == $n ? ' class="cp_selpage"' : ' class="cp_nonselpage"') . '>';
		echo '<a href="' . $ME . PD . 'id=' . $userid . '&amp;item=' . ($n + 1) . sid() . '">' . $titles[$n] . '</a>';
		echo '</td></tr>';
	}
	echo '</table>';
	echo '</td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_l" colspan="2">';
	echo '<table cellspacing="0" cellpadding="0" width="100%"><tr><td class="cp_nonselpage">';
	echo '<a href="' .
		($CFG['mod_rewrite'] ?
			'u.' . $userid . sid(3) :
			$ME . PD . 'id=' . $userid . sid()) .
		'">' . $T['show_profile'] . '</a>';
	echo '</td></tr></table>';
	echo '</td>';
	echo '</tr>';
	echo '</table>';

	if (CheckRights('is_admin'))
	{
		echo '<br /><table class="cp_pages" cellspacing="0" cellpadding="0" width="100%">';
		echo '<tr valign="top">';
		echo '<td class="list_cp_head_left">' . $T['config'] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr valign="top">';
		echo '<td class="list_cp_l" colspan="2">';
		echo '<table cellspacing="0" cellpadding="0" width="100%">';

		echo '<tr><td' . ($item == 8 ? ' class="cp_selpage"' : ' class="cp_nonselpage"') . '>';
		echo '<a href="' . $ME . PD . 'id=' . $userid . '&amp;item=8' . sid() . '">' . $titles[7] . '</a>';
		echo '</td></tr>';

		echo '<tr><td' . ($item == 9 ? ' class="cp_selpage"' : ' class="cp_nonselpage"') . '>';
		echo '<a href="' . $ME . PD . 'id=' . $userid . '&amp;item=9' . sid() . '">' . $titles[8] . '</a>';
		echo '</td></tr>';

		echo '<tr><td' . ($item == 10 ? ' class="cp_selpage"' : ' class="cp_nonselpage"') . '>';
		echo '<a href="' . $ME . PD . 'id=' . $userid . '&amp;item=10' . sid() . '">' . $titles[9] . '</a>';
		echo '</td></tr>';

		echo '<tr><td' . ($item == 11 ? ' class="cp_selpage"' : ' class="cp_nonselpage"') . '>';
		echo '<a href="' . $ME . PD . 'id=' . $userid . '&amp;item=11' . sid() . '">' . $titles[10] . '</a>';
		echo '</td></tr>';

		echo '</table>';
		echo '</td>';
		echo '</tr>';
		echo '</table>';

		echo '<br /><table cellspacing="0" cellpadding="0" width="100%">';
		echo '<tr valign="top">';
		echo '<td class="list_cp_head_left">' . $T['admin_tools'] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr valign="top">';
		echo '<td class="list_cp_l" colspan="2">';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=forums' . sid() . '">' . $T['forum_overview'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=acl' . sid() . '">' . $T['acl_editor'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=groups' . sid() . '">' . $T['groups_editor'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=log&amp;showtime=1&amp;showuserid=1&amp;showusername=1&amp;showip=1&amp;showaction=1&amp;showbrowser=0&amp;showbver=0&amp;showos=0&amp;time=' . urlencode('>' . date('H:i', time() - 2 * 3600)) . sid() . '">' . $T['show_log'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=avatars' . sid() . '">' . $T['avatar_overview'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=users' . sid() . '">' . $T['inactive_users'] . '</a><br />';
		echo '<a href="' . $MODULE['admin'] . PD . 'tool=setuid' . sid() . '">' . $T['set_user'] . '</a><br />';
		echo '</td>';
		echo '</tr>';
		echo '</table>';
	}

	// END control panel menu

	echo '</td>';
	echo '<td width="10">&nbsp;</td>';
	echo '<td>';

	if ($item == 1)   // Account settings
	{
		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col width="' . $width . '" />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left" width="' . $width . '">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">' . $T['account_summary'] . '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['name_and_id'] . '</td>';
		echo '<td class="list_cp_n">';
		if (CheckRights('renameuser'))
			echo '<input type="text" class="text" name="Name" value="' . ($p ? $_POST['Name'] : t2h($user->GetName())) . '" size="20" maxlength="40" />';
		else
			echo t2h($user->GetName());
		echo ' (' . $userid . ')</td>';
		echo '</tr>';

		echo '<tr valign="top">';
		echo '<td class="list_cp_n">' . $T['group_membership'] . '</td>';
		echo '<td class="list_cp_n">';
		if (CheckRights('setusergroups'))
		{
			echo '<input type="hidden" name="SetUserGroups" value="1" />';
			echo '<select name="UserGroups[]" class="text" size="6" multiple="multiple">';
			$groups = GetGroupNames();
			$usergroups = GetUserGroups($userid);
			foreach ($groups as $id => $name)
				echo '<option value="' . $id . '"' . (in_array($id, $usergroups) ? ' selected="selected"' : '') . '>' . t2h($name) . '</option>';
			echo '</select>';
		}
		else
		{
			$groups = GetUserGroupNames($userid);
			if ($groups) foreach ($groups as $g) echo t2h($g) . '<br />';
			else echo '(' . $T['none'] . ')';
		}
		echo '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['registered_on'] . '</td>';
		echo '<td class="list_cp_n">' . FriendlyDate($user->GetRegDate(), 1, 3) . '</td>';
		echo '</tr>';

		$post_count = GetPostsByUser($userid);
		echo '<tr>';
		echo '<td class="list_cp_l">' . $T['number_of_posts'] . '</td>';
		echo '<td class="list_cp_l">';
		if ($post_count)
		{
			echo $post_count . ' &nbsp; ';

			echo ' &nbsp; <a href="' . $MODULE['search'] . PD . 'nodef=1&amp;Query=!' . $userid . '&amp;ResultView=2&amp;InUser=1&amp;Sort=1' . sid() . '">' . $T['find_posts'] . '</a>';
		}
		else
		{
			echo $T['none'];
		}
		echo '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">' . $T['email_password'] . '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['current_password'] . '</td>';
		echo '<td class="list_cp_n"><input type="password" class="text" name="CurrentPassword" size="20" maxlength="40" /> <small>(' . $T['pwd_necessary'] . ')</small></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['current_email_address'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="EMail" value="' . ($p ? t2h($_POST['EMail']) : t2h($user->GetEMail())) . '" size="40" maxlength="100" /></td>';
		echo '</tr>';

		if (CheckRights('setusergroups'))
		{
			echo '<tr>';
			echo '<td class="list_cp_n">' . $T['registration_email_address'] . '</td>';
			echo '<td class="list_cp_n"><a href="mailto:' . $user->GetRegEMail() . '">' . t2h($user->GetRegEMail()) . '</a></td>';
			echo '</tr>';
		}

		echo '<tr valign="top">';
		echo '<td class="list_cp_l">' . $T['change_password'] . '</td>';
		echo '<td class="list_cp_l"><input type="password" class="text" name="Password" size="20" maxlength="40" /><br />';
		echo '<input type="password" class="text" name="Password2" size="20" maxlength="40" /> <small>(' . $T['repeat_new_password'] . ')</small></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="2"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" />';
		if (CheckRights('removeuser'))
			echo '&nbsp; &nbsp; <input type="checkbox" class="radio" name="Remove" id="Remove" value="1" ' . suredelete() . ' /><label for="Remove">' . $T['remove_user'] . '?</label>';
		echo '</td></tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item == 2)   // Personal profile
	{
		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col width="' . $width . '" />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left" width="' . $width . '">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">' . $T['contacting_you'] . '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['jabber'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="Jabber" value="' . ($p ? t2h($_POST['Jabber']) : t2h($user->GetJabber())) . '" size="35" maxlength="50" /></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['icq'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="ICQ" value="' . ($p ? t2h($_POST['ICQ']) : t2h($user->GetICQ())) . '" size="20" maxlength="15" /></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['aim'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="AIM" value="' . ($p ? t2h($_POST['AIM']) : t2h($user->GetAIM())) . '" size="20" maxlength="30" /></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['yim'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="YIM" value="' . ($p ? t2h($_POST['YIM']) : t2h($user->GetYIM())) . '" size="20" maxlength="30" /></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['msn'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="MSN" value="' . ($p ? t2h($_POST['MSN']) : t2h($user->GetMSN())) . '" size="35" maxlength="50" /></td>';
		echo '</tr>';

		$cols = rc('small_layout') ? 40 : 50;
		echo '<tr>';
		echo '<td class="list_cp_l">' . $T['homepage'] . '</td>';
		echo '<td class="list_cp_l"><input type="text" class="text" name="Homepage" value="' . ($p ? t2h($_POST['Homepage']) : t2h($user->GetHomepage())) . '" size="' . $cols . '" maxlength="150" /></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">' . $T['personal_info'] . '</td>';
		echo '</tr>';

		$x = ($p ? $_POST['Gender'] : $user->GetGender());
		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['gender'] . '</td>';
		echo '<td class="list_cp_n"><input type="radio" class="radio" name="Gender" id="Genderm" value="m"' . ($x == 'm' ? ' checked="checked"' : '') . ' /><label for="Genderm">' . $T['male'] . '</label> <input type="radio" class="radio" name="Gender" id="Genderf" value="f"' . ($x == 'f' ? ' checked="checked"' : '') . ' /><label for="Genderf">' . $T['female'] . '</label> <input type="radio" class="radio" name="Gender" id="Gender0" value=""' . ($x == '' ? ' checked="checked"' : '') . ' /><label for="Gender0">' . $T['no_idea'] . '</label></td>';
		echo '</tr>';

		$x = ($p ? $_POST['BirthDay'] : str_pad($user->GetBirthDay(), 2, '0', STR_PAD_LEFT));
		$y = ($p ? $_POST['BirthMonth'] : str_pad($user->GetBirthMonth(), 2, '0', STR_PAD_LEFT));
		$z = ($p ? $_POST['BirthYear'] : str_pad($user->GetBirthYear(), 4, '0', STR_PAD_LEFT));
		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['birthdate'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="BirthDay" value="' . ($x > 0 ? $x : '') . '" size="2" maxlength="2" />.<input type="text" class="text" name="BirthMonth" value="' . ($y > 0 ? $y : '') . '" size="2" maxlength="2" />.<input type="text" class="text" name="BirthYear" value="' . ($z > 0 ? $z : '') . '" size="4" maxlength="4" /> &nbsp; <small>(Format: T.M.J)</small></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['location'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="Location" value="' . ($p ? t2h($_POST['Location']) : t2h($user->GetLocation())) . '" size="40" maxlength="50" /></td>';
		echo '</tr>';

		echo '<tr valign="top">';
		echo '<td class="list_cp_n">' . $T['user_title'] . '</td>';
		echo '<td class="list_cp_n"><input type="text" class="text" name="Title" value="' . ($p ? t2h($_POST['Title']) : t2h($user->GetTitle())) . '" size="40" maxlength="40" /><br /><small>' . $T['user_title_note'] . '</small></td>';
		echo '</tr>';

		$cols = rc('small_layout') ? 40 : 60;
		echo '<tr valign="top">';
		echo '<td class="list_cp_l">' . $T['description'] . '</td>';
		echo '<td class="list_cp_l"><textarea class="text" name="About" cols="' . $cols . '" rows="6">' . ($p ? t2h($_POST['About']) : t2h($user->GetAbout())) . '</textarea></td>';
		echo '</tr>';

		if ($extra_count > 0)
		{
			$cols = rc('small_layout') ? 40 : 50;
			echo '<tr>';
			echo '<td class="list_cp_d" colspan="2">' . $T['additional_info'] . '</td>';
			echo '</tr>';

			for ($i = 1; $i <= $extra_count; $i++)
			{
				$l = ($i == $extra_count) ? '_l' : '_n';
				echo '<tr>';
				echo '<td class="list_cp' . $l . '">' . GetExtraName($i) . '</td>';
				echo '<td class="list_cp' . $l . '"><input type="text" class="text" name="Extra' . $i . '" value="' . ($p ? t2h($_POST['Extra' . $i]) : t2h($user->GetExtra($i))) . '" size="' . $cols . '" maxlength="100" /></td>';
				echo '</tr>';
			}
		}

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="2"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" /></td>';
		echo '</tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item == 3)   // Subscriptions
	{
		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';
		echo '<input type="hidden" name="SetNotify" value="1" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_l" colspan="2">' . $T['default_notification'] . ' ';
		$x = ($p ? $_POST['NotifyEMail'] : $user->GetDefaultNotify() & 1);
		$y = ($p ? $_POST['NotifyICQ'] : $user->GetDefaultNotify() & 2);
		$z = ($p ? $_POST['NotifyJabber'] : $user->GetDefaultNotify() & 4);
		echo '<input type="checkbox" class="radio" name="NotifyEMail" id="NotifyEMail" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="NotifyEMail">' . $T['e_mail'] . '</label>';
		if ($CFG['enable_icq'])
			echo ' <input type="checkbox" class="radio" name="NotifyICQ" id="NotifyICQ" value="1"' . ($y ? ' checked="checked"' : '') . ' /><label for="NotifyICQ">' . $T['icq'] . '</label>';
		if ($CFG['enable_jabber'])
			echo ' <input type="checkbox" class="radio" name="NotifyJabber" id="NotifyJabber" value="1"' . ($z ? ' checked="checked"' : '') . ' /><label for="NotifyJabber">' . $T['jabber'] . '</label>';
		echo '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_l" colspan="2"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" /></td>';
		echo '</tr>';

		echo '</table>';
		echo '</form>';

		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';
		echo '<input type="hidden" name="RemoveSubscriptions" value="1" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col />';
		echo '<col />';
		echo '<col />';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="5"><br />' . $T['subscription_note'];
		echo '<br />&nbsp;</td>';
		echo '</tr>';

		$tw = $user->GetThreadWatchs(0, '&127');
		$thread = new IThread;
		$forum = new IForum;
		$c = 1;
		if ($tw)
		{
			foreach($tw as $key => $a)
			{
				#if ($thread->Load($a['Thread']))
				{
					if ($c == 1)
					{
						echo '<tr valign="bottom">';
						echo '<td class="list_cp_l">' . $T['forum'] . '</td>';
						echo '<td class="list_cp_l">' . $T['topic'] . '</td>';
						echo '<td class="list_cp_l" align="center">' . $T['last_post'] . '</td>';
						echo '<td class="list_cp_l" align="center">' . $T['notify_via'] . '</td>';
						echo '<td class="list_cp_l" align="center">' . $T['unsubscribe'] . '</td>';
						echo '</tr>';
					}

					#$l = ($c % 4) ? '_n' : '_l';
					$l = ($a['Forum'] != $tw[$key + 1]['Forum']) ? '_l' : '_n';

					echo '<tr>';

					$forumid = $a['Forum'];   /* $thread->GetForum() */
					$forum->Load($forumid);
					echo '<td class="list_cp' . $l . '"><a href="' . $MODULE['main'] . PD . 'id=' . $forumid . sid() . '">' . t2h($forum->GetName()) . '</a></td>';

					echo '<td class="list_cp' . $l . '"><a href="' . $MODULE['thread'] . PD . 'id=' . $a['Thread'] . sid() . '">' . t2h($a['Subject']) . '</a></td>';

					echo '<td class="list_cp' . $l . '" align="center"><nobr>' . FriendlyDate($a['LastPostDate'], 1, 3) . '</nobr></td>';

					echo '<td class="list_cp' . $l . '" align="center">';
					if ($a['Mode'] & 1)
					{
						if ($ICON) echo '<img ' . $IMG['email'] . ' vspace="3" title="' . $T['e_mail'] . '" />';
						else echo $T['e_mail'];
					}
					if ($a['Mode'] & 2 && $CFG['enable_icq'])
					{
						if ($ICON) echo ' <img ' . $IMG['icq'] . ' title="' . $T['icq'] . '" />';
						else echo ' ' . $T['icq'];
					}
					if ($a['Mode'] & 4 && $CFG['enable_jabber'])
					{
						if ($ICON) echo ' <img ' . $IMG['jabber'] . ' title="' . $T['jabber'] . '" />';
						else echo ' ' . $T['jabber'];
					}
					echo '</td>';

					echo '<td class="list_cp' . $l . '" align="center">';
					echo '<a href="' . $MODULE['thread'] . PD . 'id=' . $a['Thread'] . '&amp;unwatch=7&amp;b2p=' . $userid . sid() . '"><img ' . $IMG['delete'] . ' title="' . $T['watch_no_longer'] . '" /></a></td>';

					echo '</tr>';

					$c++;
				}
			}

			$c--;
			echo '<tr>';
			echo '<td class="list_cp_l" colspan="5">(' . $c . ' ' . $T['topics'] . ')</td>';
			echo '</tr>';
		}
		else
		{
			echo '<tr>';
			echo '<td class="list_cp_l" colspan="5">(' . $T['none'] . ')</td>';
			echo '</tr>';
		}

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="5">' . $T['remove_subs_older_than'] . ' <input type="text" class="text" name="NumOfDays" size="4" maxlength="4" /> ' . $T['days'] . '. <input type="submit" class="button" name="Save" value="' . $T['delete'] . '" /></td>';
		echo '</tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item == 4)   // Bookmarks
	{
		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';
		echo '<input type="hidden" name="RemoveSubscriptions" value="1" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col />';
		echo '<col />';
		echo '<col />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left" colspan="3">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		$tw = $user->GetThreadWatchs(0, '&128');
		$thread = new IThread;
		$forum = new IForum;
		$c = 1;
		if ($tw)
		{
			foreach($tw as $key => $a)
			{
				if (!($a['Mode'] & 128)) continue;   // only show favorites here [double check]

				if ($c == 1)
				{
					echo '<tr valign="bottom">';
					echo '<td class="list_cp_l">' . $T['forum'] . '</td>';
					echo '<td class="list_cp_l">' . $T['topic'] . '</td>';
					echo '<td class="list_cp_l" align="center">' . $T['last_post'] . '</td>';
					echo '<td class="list_cp_l" align="center">' . $T['remove_bookmark'] . '</td>';
					echo '</tr>';
				}

				#$l = ($c % 4) ? '_n' : '_l';
				$l = ($a['Forum'] != $tw[$key + 1]['Forum']) ? '_l' : '_n';

				echo '<tr>';

				$forumid = $a['Forum'];   /* $thread->GetForum() */
				$forum->Load($forumid);
				echo '<td class="list_cp' . $l . '"><a href="' . $MODULE['main'] . PD . 'id=' . $forumid . sid() . '">' . t2h($forum->GetName()) . '</a></td>';

				echo '<td class="list_cp' . $l . '"><a href="' . $MODULE['thread'] . PD . 'id=' . $a['Thread'] . sid() . '">' . t2h($a['Subject']) . '</a></td>';

				echo '<td class="list_cp' . $l . '" align="center"><nobr>' . FriendlyDate($a['LastPostDate'], 1, 3) . '</nobr></td>';

				echo '<td class="list_cp' . $l . '" align="center">';
				echo '<a href="' . $MODULE['thread'] . PD . 'id=' . $a['Thread'] . '&amp;unwatch=128&amp;b2b=' . $userid . sid() . '"><img ' . $IMG['delete'] . ' title="' . $T['remove_bookmark'] . '" /></a></td>';

				echo '</tr>';

				$c++;
			}

			$c--;
			echo '<tr>';
			echo '<td class="list_cp_l" colspan="5">(' . $c . ' ' . $T['topics'] . ')</td>';
			echo '</tr>';
		}
		else
		{
			echo '<tr>';
			echo '<td class="list_cp_l" colspan="5">(' . $T['none'] . ')</td>';
			echo '</tr>';
		}
		echo '</table>';
		echo '</form>';
	}
	elseif ($item == 5)   // Signature, Avatar, Photo
	{
		echo '<form name="form" action="' . $ME . sid(2) . '" method="post" enctype="multipart/form-data">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col width="' . $width . '" />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left" width="400">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">' . $T['signature'] . '</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="2">';
		echo abbc_proc($p ? t2h($_POST['Signature']) : t2h($user->GetSignature())) . '&nbsp;';
		echo '</td>';
		echo '</tr>';

		$cols = rc('small_layout') ? 50 : 80;
		echo '<tr>';
		echo '<td class="list_cp_l" colspan="2"><textarea name="Signature" class="text" cols="' . $cols . '" rows="5">' . ($p ? t2h($_POST['Signature']) : t2h($user->GetSignature())) . '</textarea> &nbsp; ';
		echo '<a href="javascript:popup(\'' . $MODULE['abbc_help'] . sid() . '\', 430, 550);">' . $T['help'] . '...</a><br />';
		echo '<small>' . $T['signature_note'] . '<br />&nbsp;</small></td>';
		echo '</tr>';

		if (rc('avatars_enabled'))
		{
			echo '<tr>';
			echo '<td class="list_cp_d" colspan="2">' . $T['avatar'] . '</td>';
			echo '</tr>';

			echo '<tr valign="top">';
			if (rc('small_layout')) echo '<td class="list_cp_n" colspan="2">';
			else                    echo '<td class="list_cp_l">';
			echo '<input type="radio" class="radio" name="avatar" id="avatar1" value="1" /><label for="avatar1">' . $T['upload_new_avatar'] . '</label><br />';
			echo '<input type="file" class="text" name="avatarfile" size="40" style="margin-left:21px;" onchange="document.form.avatar1.checked = true;" /><br />';
			echo '<input type="radio" class="radio" name="avatar" id="avatar2" value="2" /><label for="avatar2">' . $T['load_avatar_from_url'] . '</label><br />';
			echo '<input type="text" class="text" name="avatarurl" value="' .
				htmlquote($p ? $_POST['avatarurl'] :
					($user->AvatarFromURL() ? $user->GetAvatar() : '')) .
				'" size="55" maxlength="150" style="margin-left:21px;" onchange="document.form.avatar2.checked = true;" /><br />';
			echo '<input type="radio" class="radio" name="avatar" id="avatar0" value="0" /><label for="avatar0">' . $T['remove_existing_avatar'] . '</label><br />';
			echo '<input type="radio" class="radio" name="avatar" id="avatar_1" value="-1" checked="checked" /><label for="avatar_1">' . $T['no_change'] . '</label><br />';
			echo '<small><br />' . $T['maximum'] . ': ' . rc('avatar_x') . '&times;' . rc('avatar_y') . ' ' . $T['pixels'] . ', ' . format_number(CheckRights('maxavatarsize'), 1, 1024, ' ') . $T['bytes'] . '<br />';
			echo $T['avatar_note'] . '<br />&nbsp;</small>';
			echo '</td>';

			if (rc('small_layout')) echo '</tr><tr><td class="list_cp_l" colspan="2">';
			else                    echo '<td class="list_cp_l">';
			if ($user->GetAvatar() != '')
				echo '<img src="' . avatar_url($user) . '"' . avatar_size($user, false) . ' class="img_trans_back" alt="' . $T['avatar'] . '" /></td>';
			else
				echo '(' . $T['no_avatar_present'] . ')</td>';
			echo '</tr>';
		}

		if (rc('photos_enabled'))
		{
			echo '<tr>';
			echo '<td class="list_cp_d" colspan="2">' . $T['user_photo'] . '</td>';
			echo '</tr>';

			echo '<tr valign="top">';
			if (rc('small_layout')) echo '<td class="list_cp_n" colspan="2">';
			else                    echo '<td class="list_cp_l">';
			echo '<input type="radio" class="radio" name="photo" id="photo1" value="1" /><label for="photo1">' . $T['upload_new_photo'] . '</label><br />';
			echo '<input type="file" class="text" name="photofile" size="40" style="margin-left:21px;" onchange="document.form.photo1.checked = true;" /><br />';
			echo '<input type="radio" class="radio" name="photo" id="photo2" value="2" /><label for="photo2">' . $T['load_photo_from_url'] . '</label><br />';
			echo '<input type="text" class="text" name="photourl" value="' .
				htmlquote($p ? $_POST['photourl'] :
					($user->photoFromURL() ? $user->Getphoto() : '')) .
				'" size="55" maxlength="150" style="margin-left:21px;" onchange="document.form.photo2.checked = true;" /><br />';
			echo '<input type="radio" class="radio" name="photo" id="photo0" value="0" /><label for="photo0">' . $T['remove_existing_photo'] . '</label><br />';
			echo '<input type="radio" class="radio" name="photo" id="photo_1" value="-1" checked="checked" /><label for="photo_1">' . $T['no_change'] . '</label><br />';
			echo '<small><br />' . $T['maximum'] . ': ' . CheckRights('maxphotowidth') . '&times;' . CheckRights('maxphotoheight') . ' ' . $T['pixels'] . ', ' . format_number(CheckRights('maxphotosize'), 1, 1024, ' ') . $T['bytes'] . '<br />';
			echo $T['photo_note'] . '<br />&nbsp;</small>';
			echo '</td>';

			if (rc('small_layout')) echo '</tr><tr><td class="list_cp_l" colspan="2">';
			else                    echo '<td class="list_cp_l">';
			if ($user->GetPhoto() != '')
				echo '<img src="' . photo_url($user) . '" class="img_trans_back" alt="' . $T['user_photo'] . '" /></td>';
			else
				echo '(' . $T['no_photo_present'] . ')</td>';
			echo '</tr>';
		}

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="2"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" /><input type="submit" class="button" name="Preview" value="' . $T['preview'] . '" accesskey="' . $T['preview_key'] . '" /> &nbsp; <small>' . $T['sig_preview_note'] . '</small></td>';
		echo '</tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item == 6)   // Board appearance
	{
		echo '<form name="form" action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';
		echo '<input type="hidden" name="SetFlags" value="1" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col width="' . $width . '" />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right">&nbsp;</td>';
		echo '</tr>';

		$l = rc('small_layout') ? 'n' : 'l';
		$cs = rc('small_layout') ? ' colspan="2"' : '';
		$flags = $user->GetFlags();

		echo '<tr>';
		echo '<td class="list_cp_n"' . $cs . '>' . $T['design'] . '</td>';
		if (rc('small_layout')) echo '</tr><tr>';
		echo '<td class="list_cp_n"' . $cs . '><select class="text" name="Design">';
		for ($n = -1; $n < $DESIGN['count']; $n++)
		{
			echo '<option value="' . $n . '"' . ($p && $_POST['Design'] == $n || !$p && $user->GetDesign() == $n ? ' selected="selected"' : '') . '>';
			if ($n == -1)
				echo $T['default'] . ' (' . t2h($DESIGN['name'][rc('design')]) . ')';
			else
				echo t2h($DESIGN['name'][$n]);
			echo '</option>';
		}
		echo '</select>';
		echo ' &nbsp; <a href="javascript:PreviewDesign();">' . $T['preview'] . '</a>';
		echo '</td></tr>';

		$x = ($flags & 256);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="NewLayout" id="NewLayout" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="NewLayout">' . $T['new_layout_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 512);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_l">&nbsp;</td>';
		echo '<td class="list_cp_l"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="HalfSizeAvatar" id="HalfSizeAvatar" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="HalfSizeAvatar">' . $T['half_size_avatar_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		echo '<input type="hidden" name="SetThreadIcons" value="1" />';
		$x = ($p ? $_POST['ThreadIconsI'] : $user->GetThreadIcons() != 1);
		$y = ($p ? $_POST['ThreadIconsT'] : $user->GetThreadIcons() > 0);
		echo '<tr valign="top">';
		echo '<td class="list_cp_' . $l . '"' . $cs . '>' . $T['thread_icons'] . '</td>';
		if (rc('small_layout')) echo '</tr><tr>';
		echo '<td class="list_cp_l"' . $cs . '>' . $T['thread_icons_note'] . '<br />';
		echo '<input type="checkbox" class="radio" name="ThreadIconsI" id="ThreadIconsI" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="ThreadIconsI">' . $T['with_icons'] . '</label> <input type="checkbox" class="radio" name="ThreadIconsT" id="ThreadIconsT" value="1"' . ($y ? ' checked="checked"' : '') . ' /><label for="ThreadIconsT">' . $T['with_text'] . '</label><br />';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 2);
		echo '<tr>';
		echo '<td class="list_cp_n"' . $cs . '>' . $T['options'] . '</td>';
		if (rc('small_layout')) echo '</tr><tr>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="LastReadLine" id="LastReadLine" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="LastReadLine">' . $T['last_read_line'] . '</label>';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 4);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="ShowHideCodeBlock" id="ShowHideCodeBlock" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="ShowHideCodeBlock">' . $T['show_hide_code_block_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		if (CheckRights('is_admin'))
		{
			$x = ($flags & 8);
			echo '<tr>';
			if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
			echo '<td class="list_cp_n"' . $cs . '>';
			echo '<input type="checkbox" class="radio" name="ShowUserReads" id="ShowUserReads" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="ShowUserReads">' . $T['show_user_reads_option'] . '</label>';
			echo '</td>';
			echo '</tr>';
		}

		$x = ($flags & 16);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="FastReply" id="FastReply" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="FastReply">' . $T['fast_reply_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 32);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="AutoReLogin" id="AutoReLogin" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="AutoReLogin">' . $T['auto_relogin_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		/*
		$x = ($flags & 64);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="PostCopy" id="PostCopy" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="PostCopy">' . $T['include_copy_of_post'] . '</label>';
		echo '</td>';
		echo '</tr>';
		*/

		$x = ($flags & 128);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="LinksNewWindow" id="LinksNewWindow" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="LinksNewWindow">' . $T['links_new_window_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 1024);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_n">&nbsp;</td>';
		echo '<td class="list_cp_n"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="HideAvatars" id="HideAvatars" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="HideAvatars">' . $T['hide_avatars_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		$x = ($flags & 2048);
		echo '<tr>';
		if (!rc('small_layout')) echo '<td class="list_cp_l">&nbsp;</td>';
		echo '<td class="list_cp_l"' . $cs . '>';
		echo '<input type="checkbox" class="radio" name="HideSigs" id="HideSigs" value="1"' . ($x ? ' checked="checked"' : '') . ' /><label for="HideSigs">' . $T['hide_sigs_option'] . '</label>';
		echo '</td>';
		echo '</tr>';

		/*
		echo '<tr>';
		echo '<td class="list_cp_d" colspan="2">Thread lists</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">Threads per page</td>';
		echo '<td class="list_cp_n"></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">Sort by</td>';
		echo '<td class="list_cp_n"></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">Time interval</td>';
		echo '<td class="list_cp_n"></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_l">Posts per page</td>';
		echo '<td class="list_cp_l"></td>';
		echo '</tr>';
		*/

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="2"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" /></td>';
		echo '</tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item == 7)   // Localisation
	{
		echo '<form action="' . $ME . sid(2) . '" method="post">';
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $userid . '" />';
		echo '<input type="hidden" name="item" value="' . $item . '" />';

		echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad']. '" border="' . $TABLE['cp']['border'] . '" width="100%">';
		echo '<colgroup>';
		echo '<col width="' . $width . '" />';
		echo '<col width="160" />';
		echo '<col />';
		echo '</colgroup>';

		echo '<tr>';
		echo '<td class="list_cp_head_left">' . $titles[$item - 1] . '</td>';
		echo '<td class="list_cp_head_right" colspan="2">&nbsp;</td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['language'] . '</td>';
		echo '<td class="list_cp_n" colspan="2"><select name="Language" class="text">';

		$curr = $user->GetLanguage();
		echo '<option value=""' . ($curr == '' ? ' selected="selected"' : '') . '>' . $T['default'] . '</option>';
		foreach ($CFG['languages'] as $lang)
		{
			echo '<option value="' . $lang . '"' . ($curr == $lang ? ' selected="selected"' : '') . '>' .
				t2h($CFG['language_names'][$lang]) . ' (' . $lang . ')' .
				'</option>';
		}

		echo '</select></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n">' . $T['timezone'] . '</td>';
		echo '<td class="list_cp_n" colspan="2"><select name="Timezone" class="text">';

		$curr = $user->GetTimezone();
		echo '<option value="99"' . ($curr == '99' ? ' selected="selected"' : '') . '>' . $T['default'] . '</option>';
		$tz_a = array(-12, -11, -10, -9, -8, -7, -6, -5, -4, -3.5, -3, -2, -1, 0, 1, 2, 3, 3.5, 4, 4.5, 5, 5.5, 5.75, 6, 6.5, 7, 8, 9, 9.5, 10, 11, 12, 13);
		foreach ($tz_a as $tz)
		{
			$tz4 = $tz * 4;

			$tz_hr = str_pad(abs(intval($tz)), 2, '0', STR_PAD_LEFT);
			$tz_min = (abs($tz) * 60) % 60;

			$tzn = ($tz >= 0 ? '+' : '-') . $tz_hr;
			$tzn .= ':' . str_pad($tz_min, 2, '0', STR_PAD_LEFT);
			$tzn .=  ' ' . $T['hours'];

			echo '<option value="' . $tz4 . '"' . ($curr == $tz4 ? ' selected="selected"' : '') . '>UTC ' . $tzn . '</option>';
		}

		echo '</select>';
		if (rc('small_layout')) echo '<br />';
		else                    echo ' &nbsp; ';

		$curr = $user->GetTimezoneDS();
		if ($curr == -1) $curr = $TIMEZONE['withdst'];
		echo '<input type="checkbox" class="radio" name="TimezoneDS" id="TimezoneDS" value="1"' . ($curr == 1 ? ' checked="checked"' : '') . ' /><label for="TimezoneDS">' . $T['adjust_for_ds_time'] . '</label>';
		echo '</td>';
		echo '</tr>';

		echo '<tr valign="top">';
		echo '<td class="list_cp_l">' . $T['date_format'] . '</td>';
		echo '<td class="list_cp_l">';
		$curr = $user->GetDateFormat();
		echo '<input type="text" class="text" name="DateFormat" value="' . htmlquote($curr) . '" />';
		echo '</td>';
		echo '<td class="list_cp_l"><small>' . $T['date_format_note'] . '</small></td>';
		echo '</tr>';

		echo '<tr>';
		echo '<td class="list_cp_n" colspan="3"><input type="submit" class="button" name="Save" value="' . $T['save'] . '" /></td>';
		echo '</tr>';

		echo '</table>';

		echo '</form>';
	}
	elseif ($item >= 8 && CheckRights('is_admin'))
	{
		// Board configuration forms are outsourced into another file
		require($libpath . 'cp_config.inc.php');
	}

	echo '</td></tr>';
	echo '</table>';
}


// Begin page
//
BeginHTML($T['user_profile']);

echo '<script type="text/javascript">' /*. '<![CDATA['*/;
include($libpath . 'editor.js.php');
echo /*']]>' .*/ '</script>';
echo '<script type="text/javascript" src="' . $libpath . 'editor.js"></script>';

if ($LoginUserID) $LoginUser->SetLastActivity(ULF_PROFILE);
else SetGuestLastForum(ULF_PROFILE);

if ($error)
{
	echo '<p class=error>' . $error . '</p>';
	errorlog($error);
}

$user = new IUser;
if (!$user->Load($userid))
{
	echo '<p class="error">' . $T['err_invalid_user'] . '</p>';
}
else
{
	if ($_REQUEST['action'] == 'email' || $_REQUEST['action'] == 'sendemail')
	{
		EMailForm($userid);
	}
	elseif ($_REQUEST['action'] == 'emailsuccess')
	{
		echo '<b>' . str_replace('%s', t2h($user->GetName()), $T['mail_to_x_was_sent']) . '</b>';
	}
	else
	{
		if (!$item)
		{
			addlog('view_profile ' . $userid);
			if (!CheckRights('showprofile', 0, 0, $userid))
			{
				errorlog('Zugriff verweigert');
				echo '<p class="error">' . $T['err_access_denied'] . '</p>';
			}
			else
			{
				ShowProfile($userid);
			}
			// echo '<p class="error">' . $T['err_showing_profile'] . '</p>';
		}
		else
		{
			addlog('user_cp ' . $userid . ' item ' . $item);

			if ($userid != $LoginUserID)
				echo '<p class="info">' . str_replace('%name%', t2h($user->GetName()), $T['currently_editing_profile']) . '</p>';

			CPForm($userid, $item);
		}
	}
}

UpdateStat('PageHits', 1);

EndHTML();
?>
