<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// database.lib.php
// Database abstraction class, provides the IDatabase interface


// Global timing variables
$gDBcount = 0;
$gDBtime = 0;

// Debug only
if (!isset($show_sql)) $show_sql = false;


class IDatabase
{

// ------------------------------------------------------------ PUBLIC VARIABLES
var $server = '';      // port number may be added automatically!
var $user = '';
var $password = '';
var $dbname = '';      // database name
var $tblprefix = '';   // entire string, for tables like "bb1_Users", this must be "bb1_"

// ------------------------------------------------------------ PRIVATE VARIABLES
var $conn = false;
var $result = false;

var $q_table = '';
var $q_where = '';
var $q_order = '';
var $q_limit = '';
var $q_group = '';


// ------------------------------------------------------------ STATISTICS

function ResetStat()
{
	global $gDBcount, $gDBtime;

	$gDBcount = 0;
	$gDBtime = 0;
	return true;
}

function GetCount()
{
	global $gDBcount;

	return $gDBcount;
}

function GetTime($factor = 1000, $decimals = 1)
{
	global $gDBcount, $gDBtime;

	if ($decimals == -1)
		return $gDBtime * $factor;
	else
		return round($gDBtime * $factor, $decimals);
}


// ------------------------------------------------------------ DB SERVER METHODS

// Establish connection to DB server
//
function Open()
{
	if ($this->server == '')
	{
		die('<b>BlackBoard Error:</b> No server set for database connection.<br />');
	}
	if ($this->user == '')
	{
		die('<b>BlackBoard Error:</b> No username set for database connection.<br />');
	}
	if ($this->dbname == '')
	{
		die('<b>BlackBoard Error:</b> No database name set for database connection.<br />');
	}

	if (!function_exists('version_compare') || version_compare(phpversion(), '4.2.0') < 0)
	{
		$this->conn = @mysql_connect($this->server, $this->user, $this->password);   // port 3306
	}
	else
	{
		$this->conn = @mysql_connect($this->server, $this->user, $this->password, true);
	}
	if ($this->conn == false)
	{
		die('<b>BlackBoard Error:</b> Cannot connect to database.<br />' . htmlspecialchars(mysql_error()));
	}

	if (!@mysql_select_db($this->dbname))
	{
		die('<b>BlackBoard Error:</b> Cannot switch to my database.<br />' . htmlspecialchars(mysql_error()));
	}

	return true;
}

// Terminate connection to DB server
//
function Close()
{
	if (!$this->IsConnected()) return false;
	@mysql_close($this->conn);
	return true;
}

// Check if we're connected
//
function IsConnected()
{
	return ($this->conn != false);
}

// Tell the last error
//
function LastError()
{
	return @mysql_error($this->conn);
}

// ------------------------------------------------------------ RAW DB ACCESS VIA SQL

function Exec($sql)
{
	global $gDBcount, $gDBtime, $show_sql;

	if (!$this->conn) return false;

	if (rc('foot_db_time'))
	{
		$start = getmicrotime();
	}

	$this->result = @mysql_query($sql, $this->conn);

	if (rc('foot_db_time'))
	{
		$end = getmicrotime();
		$gDBtime += $end - $start;
		$gDBcount++;
	}

	if ($show_sql)
	{
		echo '<small><b>SQL:</b> ' . htmlspecialchars($sql) . ' <b>(' . round(($end - $start) * 1000) . ')</b>';
		if ($error = @mysql_error($this->conn)) echo ' - <b>Error:</b> ' . htmlspecialchars($error);
		echo '</small><br />';
	}

	if ($this->result === false) return false;

	return true;
}


// ------------------------------------------------------------ TABLE ACCESS METHODS

// Remove a table from our database
//
function RemoveTable($name)
{
	$name = $this->tblprefix . $name;
	return $this->Exec('DROP TABLE IF EXISTS `' . $name . '`');
}

// Create a new table in our database
//
// fields = "<name> <type>[, <name> <type>...]" (see AddField(), MySQL Doc: 6.5.3)
//
function CreateTable($name, $fields, $remove = false)
{
	if ($fields == '') return false;

	if ($remove) $this->RemoveTable($name);

	$name = $this->tblprefix . $name;
	return $this->Exec('CREATE TABLE `' . $name . '` (' . $fields . ')');
}

// Add a field to a table
//
// type = bool, int[(length)] [unsigned], varchar(length) [binary], text (MySQL Doc: 6.2)
// where = 0: before first column | col name: insert new column after $where | unset: add column at the end
//
function AddField($table, $name, $type, $where = '')
{
	if ($where == 0) $where = ' FIRST';

	elseif ($where != '') $where = ' AFTER ' . $where;

	$table = $this->tblprefix . $table;
	return $this->Exec('ALTER TABLE `' . $table . '` ADD `' . $name . '` ' . $type . $where);
}

// Alter a field in a table
//
// type = bool, int[(length)] [unsigned], varchar(length) [binary], text (MySQL Doc: 6.2)
//
function ChangeField($table, $oldname, $name, $type)
{
	$table = $this->tblprefix . $table;
	return $this->Exec('ALTER TABLE `' . $table . '` CHANGE `' . $oldname . '` `' . $name . '` ' . $type);
}

// Remove a field from a table
//
function RemoveField($table, $name)
{
	$table = $this->tblprefix . $table;
	return $this->Exec('ALTER TABLE `' . $table . '` DROP `' . $name . '`');
}

// Add an index to a table
//
// type = index, primary key, unique, fulltext (MySQL Doc: 6.5.4)
// name = name(s) of field(s) to create index for
//
function AddIndex($table, $type, $name)
{
	$table = $this->tblprefix . $table;
	return $this->Exec('ALTER TABLE `' . $table . '` ADD ' . $type . ' `' . $name . '`');
}

// Get a list of all columns in a table
//
// IN:
// table = Table name
//
// RET:
// a 2-dimensional array(Field => array(Field, Type, Null, Key, Default, Extra))
//
function ListTableCols($table)
{
	$table = $this->tblprefix . $table;
	if (!$this->Exec('DESCRIBE `' . $table . '`')) return false;
	$record = $this->GetRecord();
	$arr = array();
	do
	{
		$arr[$record['Field']] = $record;
	}
	while ($record = $this->GetRecord());
	return $arr;
}


// ------------------------------------------------------------ ROW ACCESS METHODS -- READING

// Reset all parameters for a new query
//
function NewQuery()
{
	$this->q_table = '';
	$this->q_where = '';
	$this->q_order = '';
	$this->q_limit = '';
	$this->q_group = '';
	return true;
}

// Set new table for queries
//
function SetTable($name)
{
	$this->q_table = $name;
	return true;
}

// Set new criteria for queries
//
function SetWhere($name)
{
	$this->q_where = $name;
	return true;
}

// Set new order for queries
//
function SetOrder($name)
{
	$this->q_order = $name;
	return true;
}

// Set new limit for queries
//
function SetLimit($name)
{
	$this->q_limit = $name;
	return true;
}

// Set new group for queries
//
function SetGroup($name)
{
	$this->q_group = $name;
	return true;
}

// Do query on given fields
//
function QueryFields($fields = '*')
{
	if ($this->q_table == '') return false;

	// build SQL query
	$table = $this->tblprefix . $this->q_table;
	$q = 'SELECT ' . $fields . ' FROM `' . $table . '`';
	if ($this->q_where != '') $q .= ' WHERE ' . $this->q_where;
	if ($this->q_group != '') $q .= ' GROUP BY ' . $this->q_group;
	if ($this->q_order != '') $q .= ' ORDER BY ' . $this->q_order;
	if ($this->q_limit != '') $q .= ' LIMIT ' . $this->q_limit;

	return $this->Exec($q);
}

// Get next recordset of previous query
//
function GetRecord()
{
	global $gDBtime, $show_sql;

	if ($this->result === false) return false;

	/*if (rc('foot_db_time'))
	{
		$start = getmicrotime();
	}*/

	$record = @mysql_fetch_array($this->result);

	/*if (rc('foot_db_time'))
	{
		$end = getmicrotime();
		$gDBtime += $end - $start;
	}*/
	#if ($show_sql) echo "<small><b>GetRecord:</b> (" . round(($end - $start) * 1000) . ")</small><br />";

	return $record;
}

// Do query and get first record in one call
// Further calls of GetRecord are possible
//
function FastQuery($table, $fields = '*', $where = '', $order = '', $limit = '', $group = '')
{
	$this->NewQuery();
	$this->SetTable($table);
	$this->SetWhere($where);
	$this->SetOrder($order);
	$this->SetLimit($limit);
	$this->SetGroup($group);
	if ($this->QueryFields($fields) === false) return false;
	return @mysql_fetch_array($this->result);
	#return $this->GetRecord();
}

// Same as FastQuery but return the first row
// Further calls of GetRecord are possible
//
function FastQuery1st($table, $fields = '*', $where = '', $order = '', $limit = '', $group = '')
{
	$record = $this->FastQuery($table, $fields, $where, $order, $limit, $group);
	if ($record === false) return false;
	else return $record[0];
}

// Same as FastQuery1st, but copies all records in an array
//
// key = record's field to use for array's key. should be unique
//
function FastQuery1stArray($table, $fields = '*', $where = '', $order = '', $limit = '', $group = '', $key = false)
{
	$record = $this->FastQuery($table, $fields, $where, $order, $limit, $group);
	if ($record === false) return false;
	$arr = array();
	do
	{
		if ($key === false) array_push($arr, $record[0]);
		else                $arr[$record[$key]] = $record[0];
	}
	while ($record = @mysql_fetch_array($this->result));
	#while ($record = $this->GetRecord());

	return $arr;
}

// Same as FastQuery1stArray, but copies records ENTIRELY into the array
//
// key = record's field to use for array's key. should be unique
//
function FastQueryArray($table, $fields = '*', $where = '', $order = '', $limit = '', $group = '', $key = false)
{
	$record = $this->FastQuery($table, $fields, $where, $order, $limit, $group);
	if ($record === false) return false;
	$arr = array();
	do
	{
		if ($key === false) array_push($arr, $record);
		else                $arr[$record[$key]] = $record;
	}
	while ($record = @mysql_fetch_array($this->result));
	#while ($record = $this->GetRecord());

	return $arr;
}


// ------------------------------------------------------------ ROW ACCESS METHODS -- WRITING

// Add a new record
//
// fields =
//   (array) : associative array ("field" => "value"...)
//   (string): string for SET part of SQL query.
//             Callee must pay attention to `-quote field names and db_encode() values as neccessary!
//
function AddRecord($fields, $table = '')
{
	if ($table == '') $table = $this->q_table;
	if ($table == '') return false;

	// build SQL query
	$table = $this->tblprefix . $table;
	$q = 'INSERT INTO `' . $table . '` SET ';

	if (is_array($fields))
	{
		$pos = 0;
		reset($fields);
		while (list ($key, $value) = each ($fields))
		{
			#$num = is_numeric($value);
			$num = false;   // let the DBMS decide whether it is numeric or not...

			if ($pos >= 1) $q .= ', ';

			$q .= '`' . $key . '`=';
			$q .= ($num ? '' : '"');
			$q .= db_encode($value);
			$q .= ($num ? '' : '"');

			$pos++;
		}
	}
	elseif (is_string($fields))
	{
		$q .= $fields;
	}
	else
		return false;   // input type not supported

	return $this->Exec($q);
}


// Change a record
//
// fields =
//   (array) : associative array ("field" => "value"...)
//   (string): string for SET part of SQL query.
//             Callee must pay attention to `-quote field names and db_encode() values as neccessary!
// where = condition of recordsets to update
//
function ChangeRecord($fields, $where = '', $table = '')
{
	if ($table == '') $table = $this->q_table;
	if ($table == '') return false;

	// build SQL query
	$table = $this->tblprefix . $table;
	$q = 'UPDATE `' . $table . '` SET ';

	if (is_array($fields))
	{
		$pos = 0;
		foreach ($fields as $key => $value)
		{
			if ($pos >= 1) $q .= ', ';
			$q .= '`' . $key . '`="' . db_encode($value) . '"';
			$pos++;
		}
	}
	elseif (is_string($fields))
	{
		$q .= $fields;
	}
	else
		return false;   // input type not supported

	if ($where != '') $q .= ' where ' . $where;

	return $this->Exec($q);
}


// Delete a record
//
// where = condition of recordsets to update
//
function RemoveRecord($where = '', $table = '')
{
	if ($table == '') $table = $this->q_table;
	if ($table == '') return false;

	// build SQL query
	$table = $this->tblprefix . $table;
	$q = 'DELETE FROM `' . $table . '`';

	if ($where != '') $q .= ' WHERE ' . $where;

	return $this->Exec($q);
}


}  // class


// Encode values so that they can be stored to the database without breaking out of the SQL query
// Masks the follwing characters: \ ' " \n \r \t
//
function db_encode($str)
{
	$str = str_replace('\\', '\\\\', $str);
	$str = str_replace('\'', '\\\'', $str);
	$str = str_replace('"', '\\"', $str);
	$str = str_replace("\n", '\\n', $str);
	$str = str_replace("\r", '\\r', $str);
	$str = str_replace("\t", '\\t', $str);
	return $str;
}

?>