<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// message.lib.php
// Message Library, provides the IMessage interface


class IMessage
{

// ------------------------------------------------------------ PUBLIC VARIABLES

// ------------------------------------------------------------ PRIVATE VARIABLES
var $db;

var $ID = 0;
var $Forum = 0;
var $Date = 0;
var $User = 0;
var $Subject = '';
var $Msg = '';
var $Options = 0;
	// 1	important message
	// 2	recursive message (also display in sub-forums)
	// 4	also show message in threads
	// 8+16(=24)  display for all (00=0) | guests only (01=8) | users only (10=16) | mods only (11=24)

// ------------------------------------------------------------ CONSTRUCTOR

function IMessage()
{
	global $db;
	if (PHP5)
		$this->db = $db->__clone();
	else
		$this->db = $db;
}

function Reset()
{
	$this->ID = 0;
	$this->Forum = 0;
	$this->Date = 0;
	$this->User = 0;
	$this->Subject = '';
	$this->Msg = '';
	$this->Options = 0;
}


// ------------------------------------------------------------ FIND

// id = Forum ID
//
function Find($id, $newonly = false, $inthread = false)
{
	global $LoginUserID, $LoginUserGroups;

	// Find all parent forums for recursive messages
	$parents = array();
	$forum = new IForum;
	$newid = $id;
	while ($newid)
	{
		$forum->Load($newid);
		$newid = $forum->GetParent();
		array_push($parents, $newid);
	}
	$p_str = '';

	// Get access mask
	if (!in_array(1, $LoginUserGroups))
		$access = 'Options & 24 IN (0,8)';         // guests only
	elseif (in_array(3, $LoginUserGroups))
		$access = 'Options & 24 IN (0,8,16,24)';   // admins must see all messages to be able to edit them!
	elseif (in_array(2, $LoginUserGroups))
		$access = 'Options & 24 IN (0,16,24)';     // mods|users only
	else   // implies in_array(1, $LoginUserGroups)
		$access = 'Options & 24 IN (0,16)';        // users only

	if ($newonly)
	{
		$pre = $this->db->tblprefix;

		if ($parents)
			$p_str = " OR ({$pre}Messages.Forum IN (" . join(',', $parents) . ") AND {$pre}Messages.Options & 2)";

		$this->db->Exec(
			"SELECT {$pre}Messages.* " .
			"FROM {$pre}Messages LEFT JOIN {$pre}MessageRead ON {$pre}Messages.ID={$pre}MessageRead.Msg AND {$pre}MessageRead.User={$LoginUserID} " .
			"WHERE ({$pre}Messages.Forum=$id" . $p_str . ") AND {$pre}MessageRead.Msg IS NULL " .
				($inthread ? "AND {$pre}Messages.Options & 4 " : '') .
				" AND $access " .
			"ORDER BY {$pre}Messages.Options & 1 DESC, {$pre}Messages.Date");
		$record = $this->db->GetRecord();
	}
	else
	{
		if ($parents)
			$p_str = ' OR (Forum IN (' . join(',', $parents) . ') AND Options & 2)';

		$record = $this->db->FastQuery('Messages', '*',
			'(Forum=' . $id . $p_str . ')' .
				($inthread ? ' AND Options & 4' : '') .
				' AND ' . $access,
			'Options & 1 DESC, Date');
	}

	return $this->LoadFromRecord($record);
}

function FindNext()
{
	$record = $this->db->GetRecord();
	return $this->LoadFromRecord($record);
}


// ------------------------------------------------------------ READ ACCESS

function Load($id)
{
	$record = $this->db->FastQuery('Messages', '*', 'ID=' . $id);
	return $this->LoadFromRecord($record);
}

function LoadFromRecord($record)
{
	if ($record)
	{
		$this->ID = intval($record['ID']);
		$this->Forum = intval($record['Forum']);
		$this->Date = intval($record['Date']);
		$this->User = intval($record['User']);
		$this->Subject = $record['Subject'];
		$this->Msg = $record['Msg'];
		$this->Options = $record['Options'];
		return true;
	}
	else
	{
		$this->Reset();
		return false;
	}
}


function GetID()
{
	return $this->ID;
}

function GetForum($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Forum;
}

function GetDate($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Date;
}

function GetUser($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->User;
}

function GetSubject($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Subject;
}

function GetMsg($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Msg;
}

function GetOptions($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Options;
}

function GetOptAccess($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & (8 | 16)) >> 3;   // divide by 8
}


function Count($forumid, $inthread = false)
{
	// Find all parent forums for recursive messages
	$parents = array();
	$forum = new IForum;
	$newid = $forumid;
	while ($newid)
	{
		$forum->Load($newid);
		$newid = $forum->GetParent();
		array_push($parents, $newid);
	}
	if ($parents)
		$p_str = ' OR (Forum IN (' . join(',', $parents) . ') AND Options & 2)';
	else
		$p_str = '';

	return $this->db->FastQuery1st('Messages', 'COUNT(*)',
		'(Forum=' . $forumid . $p_str . ')' .
			($inthread ? ' AND Options & 4' : ''));
}


// ------------------------------------------------------------ WRITE ACCESS

function Add($Forum, $Subject, $Msg, $Options)
{
	global $LoginUserID;

	$max = intval($this->db->FastQuery1st('Messages', 'MAX(ID)'));

	$this->ID = $max + 1;
	$this->Forum = intval($Forum);
	$this->Date = time();
	$this->User = $LoginUserID;
	$this->Subject = trim($Subject);
	$this->Msg = trim($Msg);
	$this->Options = intval($Options);

	return $this->db->AddRecord(array(
			'ID' => $this->ID,
			'Forum' => $this->Forum,
			'Date' => $this->Date,
			'User' => $this->User,
			'Subject' => $this->Subject,
			'Msg' => $this->Msg,
			'Options' => $this->Options
		), 'Messages');
}


function Change($ID, $Forum, $Subject, $Msg, $Options)
{
	global $LoginUserID;

	if (!$this->Load($ID)) return false;

	if ($Forum != -1) $this->Forum = intval($Forum);
	$this->Subject = trim($Subject);
	$this->Msg = trim($Msg);
	$this->Options = trim($Options);

	return $this->db->ChangeRecord(array(
			'Forum' => $this->Forum,
			'Date' => $this->Date,
			'User' => $this->User,
			'Subject' => $this->Subject,
			'Msg' => $this->Msg,
			'Options' => $this->Options
		), 'ID=' . $ID, 'Messages');
}

function SetUser($User = false)
{
	$arr = array();

	if ($User !== false)
	{
		$this->User = $User;
		$arr['User'] = $User;
	}
	if (!$arr) return true;

	return $this->db->ChangeRecord($arr, 'ID=' . $this->ID, 'Messages');
}


function Remove($id = 0)
{
	if (!$id) $id = $this->ID;

	$this->RemoveAllReads($id);

	return $this->db->RemoveRecord('ID=' . $id, 'Messages');
}


// ------------------------------------------------------------ READ/UNREAD FUNCTIONS

function SetRead($read = true)
{
	global $LoginUserID;

	if ($read)
	{
		if ($this->db->FastQuery1st('MessageRead', 'COUNT(*)', 'Msg=' . $this->ID . ' AND User=' . $LoginUserID) == 0)
		{
			return $this->db->AddRecord(array('Msg' => $this->ID, 'User' => $LoginUserID), 'MessageRead');
		}
		else
			return true;
	}
	else
	{
		if ($this->db->FastQuery1st('MessageRead', 'COUNT(*)', 'Msg=' . $this->ID . ' AND User=' . $LoginUserID) > 0)
		{
			return $this->db->RemoveRecord('Msg=' . $this->ID . ' AND User=' . $LoginUserID, 'MessageRead');
		}
		else
			return true;
	}
}

function IsRead($id = 0)
{
	global $LoginUserID;

	if (!$id) $id = $this->ID;
	$db = $this->db;    // copy db-object or we'd collide with other running queries
	return ($db->FastQuery1st('MessageRead', 'COUNT(*)', 'Msg=' . $id . ' AND User=' . $LoginUserID) > 0);
}

function ReadCount($id = 0)
{
	global $LoginUserID;

	if (!$id) $id = $this->ID;
	$db = $this->db;    // copy db-object or we'd collide with other running queries
	return $db->FastQuery1st('MessageRead', 'COUNT(*)', 'Msg=' . $id);
}

// removes any read status relating to (this) message
//
function RemoveAllReads($id = 0)
{
	if (!$id) $id = $this->ID;
	return $this->db->RemoveRecord('Msg=' . $id, 'MessageRead');
}

// removes any read status relating to (this) user
//
function RemoveAllUserReads($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;
	if (!$id) $id = $LoginUserID;
	return $this->db->RemoveRecord('User=' . $id, 'MessageRead');
}

}  // class

?>