<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// post.inc.php
// Post editing form

require_once($libpath . 'common_post.lib.php');

$maxpostlen = rc('max_post_len');

/*
echo '<small><pre>GET=';
print_r($_GET);
echo '<br />POST=';
print_r($_POST);
echo '</pre></small>';
*/

// Import request variables
$forumid = $_REQUEST['forum'];
if (!is_numeric($forumid) || $forumid < 0) $forumid = 0;

$threadid = $_REQUEST['thread'];
if (!is_numeric($threadid) || $threadid < 0) $threadid = 0;

$postid = $_REQUEST['id'];
if (!is_numeric($postid) || $postid < 0) $postid = 0;

$replyto = $_REQUEST['replyto'];
if (!is_numeric($replyto) || $replyto < 0) $replyto = 0;

$messageid = $_REQUEST['message'];
if (!is_numeric($messageid) || $messageid < -1) $messageid = 0;

$page = $_REQUEST['page'];
if (!is_numeric($page) || $page < -1 || $page == 0) $page = 1;


$thread = new IThread;
if ($postid && !$threadid)
{
	$post = new IPost;
	$post->Load($postid);
	$threadid = $post->GetThread();
	$thread->Load($threadid);
	$forumid = $thread->GetForum();
}
else
{
	$thread->Load($threadid);
}
if ($messageid && !$forumid)
{
	$threadid = 0;
	$message = new IMessage;
	$message->Load($messageid);
	$forumid = $message->GetForum();
}


// Check if a post attachment file has correct filetype/filesize
//
function is_valid_attach_name($filename)
{
	$x = strrpos($filename, '.');
	if ($x === false) return false;   // filename needs to contain a . for analysis and renaming
	$ext = substr($filename, $x);
	if (in_array(strtolower($ext), rc('attach_exts', true))) return true;
	return true;   // set this to false to enforce above filetypes!
}

// thread = Thread object to add attachment to (important to evaluate ACL rules correctly)
//
function is_valid_attach_size($filename, &$thread)
{
	if (@filesize($filename) > CheckRights('maxattachsize', $thread->GetForum(), $thread->GetID()))
		return false;   // file too long
	return true;
}

// Send out ForumWatch/ThreadWatch notifications to all users who want one
//
// send thread watch notifications, if forum->ID = 0, send forum watch notifications else.
//
function SendWatchNotification(&$forum, &$thread, &$post, $page, $username)
{
	global $db, $LoginUserID, $LoginUserName, $MODULE, $CFG;

	$threadid = $thread->GetID();
	$t_subject = $thread->GetSubject();
	$t_desc = $thread->GetDesc();
	if ($t_desc != '') $t_desc = ' (' . $t_desc . ')';
	$postid = $post->GetID();

	$forum2 = new IForum;
	$forum2->Load($thread->GetForum());
	$forum_name = $forum2->GetName();

	if ($LoginUserID) $poster = $LoginUserName;
	else              $poster = $username;

	if ($page > 1) $pagelink = '&amp;page=' . $page;
	else           $pagelink = '';

	if ($CFG['mod_rewrite'])
		$url = rc('home_url') . '/' . $threadid . ',' . $page . '#' . $postid;
	else
		$url = rc('home_url') . '/' . $MODULE['thread'] . h2t(PD) . 'id=' . $threadid . $pagelink . '#' . $postid;

	if (!$forum->GetID())
	{
		addlog('debug: swn: ThreadWatch processing');
		// ThreadWatch's
		$mail_users = $db->FastQuery1stArray('ThreadWatch', 'User', 'Thread=' . $threadid . ' AND Mode & 1 AND LastNotify<=LastRead AND User!=' . $LoginUserID);
		$icq_users = $CFG['enable_icq'] ?
			$db->FastQuery1stArray('ThreadWatch', 'User', 'Thread=' . $threadid . ' AND Mode & 2 AND LastNotify<=LastRead AND User!=' . $LoginUserID) :
			false;
		$jabber_users = $CFG['enable_jabber'] ?
			$jabber_users = $db->FastQuery1stArray('ThreadWatch', 'User', 'Thread=' . $threadid . ' AND Mode & 4 AND LastNotify<=LastRead AND User!=' . $LoginUserID) :
			false;
		if (!$mail_users && !$icq_users && !$jabber_users) return true;

		if ($mail_users)
		{
			$msg_key = 'threadwatchnotify_mail';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);
			$subject_key = 'threadwatchnotify_mail_subject';
			$subject_data = array(
				'%subject%' => $t_subject
				);

			$start = getmicrotime();
			$count = NotifyUser($mail_users, 1, $subject_key, $subject_data, $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('email_notify for thread ' . $threadid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($mail_users as $userid)
			{
				$thread->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
		if ($icq_users)
		{
			// WARNING: ICQ doesn't seem to allow URLs or something similar to appear in pager messages :(

			$msg_key = 'threadwatchnotify_icq';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);

			$start = getmicrotime();
			$count = NotifyUser($icq_users, 2, '', '', $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('icq_notify for thread ' . $threadid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($icq_users as $userid)
			{
				$thread->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
		if ($jabber_users)
		{
			$msg_key = 'threadwatchnotify_jabber';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);
			$subject_key = 'threadwatchnotify_jabber_subject';
			$subject_data = array(
				'%subject%' => $t_subject
				);

			$start = getmicrotime();
			$count = NotifyUser($jabber_users, 4, $subject_key, $subject_data, $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('jabber_notify for thread ' . $threadid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($jabber_users as $userid)
			{
				$thread->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
	}
	else
	{
		$forumid = $forum->GetID();
		addlog('debug: swn: ForumWatch processing for forum ' . $forumid);

		// ForumWatch's
		/*$mail_users = $db->FastQuery1stArray('ForumWatch', 'User', "Forum=$forumid and Mode & 1 and LastNotify<=LastRead and User!=$LoginUserID");
		$icq_users = $CFG['enable_icq'] ?
			$db->FastQuery1stArray('ForumWatch', 'User', "Forum=$forumid and Mode & 2 and LastNotify<=LastRead and User!=$LoginUserID") :
			false;
		$jabber_users = $CFG['enable_jabber'] ?
			$jabber_users = $db->FastQuery1stArray('ForumWatch', 'User', "Forum=$forumid and Mode & 4 and LastNotify<=LastRead and User!=$LoginUserID") :
			false;*/

		$mail_users = $db->FastQuery1stArray('ForumWatch', 'User', 'Forum=' . $forumid . ' AND Mode & 1 AND User!=' . $LoginUserID);
		$icq_users = $CFG['enable_icq'] ?
			$db->FastQuery1stArray('ForumWatch', 'User', 'Forum=' . $forumid . ' AND Mode & 2 AND User!=' . $LoginUserID) :
			false;
		$jabber_users = $CFG['enable_jabber'] ?
			$jabber_users = $db->FastQuery1stArray('ForumWatch', 'User', 'Forum=' . $forumid . ' AND Mode & 4 AND User!=' . $LoginUserID) :
			false;
		if (!$mail_users && !$icq_users && !$jabber_users) return true;

		if ($mail_users)
		{
			$msg_key = 'forumwatchnotify_mail';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%desc%' => $t_desc,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);
			$subject_key = 'forumwatchnotify_mail_subject';
			$subject_data = array(
				'%subject%' => $t_subject,
				'%forum%' => $forum_name
				);

			$start = getmicrotime();
			$count = NotifyUser($mail_users, 1, $subject_key, $subject_data, $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('email_notify for forum ' . $forumid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($mail_users as $userid)
			{
				$forum->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
		if ($icq_users)
		{
			// WARNING: ICQ doesn't seem to allow URLs or something similar to appear in pager messages :(

			$msg_key = 'forumwatchnotify_icq';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%desc%' => $t_desc,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);

			$start = getmicrotime();
			$count = NotifyUser($icq_users, 2, '', '', $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('icq_notify for forum ' . $forumid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($icq_users as $userid)
			{
				$forum->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
		if ($jabber_users)
		{
			$msg_key = 'forumwatchnotify_jabber';
			$msg_data = array(
				'%subject%' => $t_subject,
				'%desc%' => $t_desc,
				'%poster%' => $poster,
				'%forum%' => $forum_name,
				'%url%' => $url
				);
			$subject_key = 'forumwatchnotify_jabber_subject';
			$subject_data = array(
				'%subject%' => $t_subject,
				'%forum%' => $forum_name
				);

			$start = getmicrotime();
			$count = NotifyUser($jabber_users, 4, $subject_key, $subject_data, $msg_key, $msg_data);
			$end = getmicrotime();
			addlog('jabber_notify for forum ' . $forumid . ' (' . $count . ' users) in ' . round(($end - $start) * 1000) . ' msec');

			foreach ($jabber_users as $userid)
			{
				$forum->SetLastNotify($post->GetDate(), 0, $userid);
			}
		}
	}
}

// Clean text from too many smilies, repeating characters...
//
function SpamRate(&$text)
{
	global $abbc_scan;

	$spamrating = 0;

	// look for character repetitions
	$text0 = $text;   // make a backup
	$text = preg_replace("/(([!\?\.]){30})\\2*/", "$1", $text);   // more than 30x ! ? .
	$text = preg_replace("/((\r\n){5})\\2*/", "$1", $text);      // more than 5x new-line

	if ($text != $text0) $spamrating = 1;

	/*
	// look for too many smilies
	$abbc_scan = array();
	abbc_proc($text);

	$imgs = $abbc_scan['smile'] + $abbc_scan['img'];
	$len = $abbc_scan['len'];
	if ($imgs > 20) $spamrating = 1;
	if ($imgs > 3 && $len > 50) if ($imgs / $len / 0.04 > 1) $spamrating = 1;
	//echo "imgs=$imgs, len=$len, spamrating=$spamrating, arith=" . ($imgs / $len / 0.04);
	*/

	return $spamrating;
}


// SPAM RATING
//
$_POST['Msg'] = trim($_POST['Msg']);
$_POST['Msg0'] = $_POST['Msg'];   // make a backup

$spamrating = SpamRate($_POST['Msg']);
#$spamrating = 0;

$IsPreview = ($_POST['preview'] != '' || $_POST['Poll'] != '' || $_POST['NoPoll'] != '');

$error = false;

// --------------------------------------- ACTION FOR POST ADDING ----------

if ($_POST['action'] == 'add' && $_POST['preview'] != '')   // don't use $IsPreview to not display the post when adding/removing a poll
{
	if (trim($_POST['Msg']) == '')
	{
		$error .= $T['err_no_text'] . '<br />';
	}

	if (abbc_check(trim($_POST['Msg'])))
	{
		$error .= $T['err_abbc_syntax'] . '<br />';
		$_POST['preview'] = 'xx';   // just to show the preview
	}

	if ($_POST['Poll'] == 1 && $_POST['PollTimeout'] < 0)
	{
		$error .= $T['err_poll_timeout'] . '<br />';
	}
}

if ($_POST['action'] == 'add' && $_POST['save'] != '')
{
	$post = new IPost;
	$thread = new IThread;
	$forum = new IForum;
	$user = new IUser;

	if (!$LoginUserID && trim($_POST['UserName']) == '')
	{
		$error .= $T['err_guest_name'] . '<br />';
	}

	if (!$LoginUserID && $user->FindByName(trim($_POST['UserName'])))
	{
		$error .= $T['err_username_assigned'] . '<br />';
	}

	if (trim($_POST['Msg']) == '')
	{
		$error .= $T['err_no_text'] . '<br />';
	}

	if (abbc_check(trim($_POST['Msg'])))
	{
		$error .= $T['err_abbc_syntax'] . '<br />';
		$_POST['preview'] = 'xx';   // just to show the preview
	}

	if ($_POST['Poll'] == 1 && trim($_POST['Question']) == '')
	{
		$error .= $T['err_poll_no_question'] . '<br />';
	}

	if ($_POST['Poll'] == 1 && $_POST['PollTimeout'] < 0)
	{
		$error .= $T['err_poll_timeout'] . '<br />';
	}

	if ($_FILES['AttachFile']['name'] != '' &&
		filesize($_FILES['AttachFile']['tmp_name']) > CheckRights('maxattachsize', $thread->GetForum(), $threadid))
	{
		$error .= $T['err_invalid_attach'] . '<br />';
	}

	if (!$error && $threadid != 0)
	{
		$newthread = false;
		$thread->Load($threadid);

		if (!CheckRights('writeforum', $thread->GetForum()))
		{
			$error .= $T['err_access_denied'] . '<br />';
		}
		if (!$error && $thread->IsClosed() && !CheckRights('closethread'))
		{
			$error .= $T['err_thread_closed'] . '<br />';
		}
		if ($_POST['Poll'] == 1 && !CheckRights('createpoll', $thread->GetForum(), $thread->GetID()))
		{
			$error .= $T['err_poll_not_allowed'] . '<br />';
		}
	}

	if (!$error && $threadid == 0)
	{
		# we first have to create a new thread

		# see if we have a subject given...
		if (!CheckRights('addthread', $forumid))
		{
			$error .= $T['err_access_denied'] . '<br />';
		}
		elseif (trim($_POST['Subject']) == '')
		{
			$error .= $T['err_no_topic'] . '<br />';
		}
		else
		{
			if (!$thread->Add($forumid, trim($_POST['UserName']), trim($_POST['Subject']), $opt))
			{
				$error .= $T['err_thread_not_created'] . ' (' . t2h($thread->db->LastError()) . ')<br />';
			}
			else
			{
				$thread->SetDesc(trim($_POST['Desc']));
			}
			$threadid = $thread->GetID();
			$newthread = true;
		}
	}

	if (!$error)
	{
		# now thread exists, create post
		$opt0 = $opt = $thread->GetOptions();
		if (CheckRights('closethread', $thread->GetForum(), $thread->GetID())) $opt = ($opt & ~1) | ($_POST['CloseThread'] ? 1 : 0) * 1;
		if (CheckRights('importantthread', $thread->GetForum(), $thread->GetID())) $opt = ($opt & ~2) | ($_POST['ImportantThread'] ? 1 : 0) * 2;
		if (CheckRights('createpoll', $thread->GetForum(), $thread->GetID()) && $newthread) $opt = ($opt & ~4) | ($_POST['Poll'] ? 1 : 0) * 4;
		if ($opt != $opt0) $thread->SetOptions($opt);

		$opt = 0;
		$opt += ($_POST['NoSmileys'] ? 1 : 0) * 1;
		$opt += ($_POST['NoSpecialABBC'] ? 1 : 0) * 2;

		if (!$post->Add($threadid, $replyto, trim($_POST['UserName']), trim($_POST['Subject']), trim($_POST['Msg']), $opt, $spamrating))
		{
			addlog('add_post to ' . $threadid . ' error');
			$error .= $T['err_post_not_created'] . ' (' . t2h($post->db->LastError()) . ')<br />';
		}
		else
		{
			// update LastPostDate for correct display order
			// note: there's no error detection on this operation
			$thread->SetLastPostDate($post->GetDate());

			// copy and link the attachment file
			$name = basename($_FILES['AttachFile']['name']);
			if ($name != '' &&
			    filesize($_FILES['AttachFile']['tmp_name']) <= CheckRights('maxattachsize', $thread->GetForum(), $threadid))
			{
				$x = strrpos($name, '.');
				if (!is_valid_attach_name($name) || !is_valid_attach_size($_FILES['AttachFile']['tmp_name'], $thread))
				{
					$ok = false;
					$error .= $T['err_invalid_attach'] . '<br />';
				}
				else
				{
					$ext = substr($name, $x);
					$name2 = "post_" . $post->GetID() . $ext;

					if (!file_exists($attach_path)) mkdir($attach_path);
					if (!move_uploaded_file($_FILES['AttachFile']['tmp_name'], $attach_path . $name2))
					{
						$ok = false;
						$error .= $T['err_attach_not_saved'] . '<br />';
					}
					else
					{
						@chmod($attach_path . $name2, 0644);   // prevent access problems by the webserver
						if (!$post->SetAttachFile($name2, $name)) $ok = false;
					}
				}
			}

			// as we've come so far ;) we can create the poll definitions
			if ($_POST['Poll'] == 1 && !$thread->SetQuestion($_POST['Question'],
			                          ($_POST['PollTimeout'] == 0 ? 0 : (time() + $_POST['PollTimeout'] * 3600 * 24))))
			{
				// oops, an error? now?!
				$error .= $T['err_poll_not_created'] . ' (0) (' . t2h($thread->db->LastError()) . ')<br />';
			}
			if (!$error && $_POST['Poll'] == 1 && $_POST['PollDef'] == 1)
			{
				for ($n = 1; $n <= 10; $n++)
				{
					if (trim($_POST['Poll' . $n]) != '')
					{
						if (!$thread->CreateVoteOption(0, trim($_POST['Poll' . $n]), $n))
						{
							// oops, an error? now?!
							$error .= $T['err_poll_not_created'] . ' (' . $n . ') (' . t2h($thread->db->LastError()) . ')<br />';
						}
					}
				}
			}

			if ($error)
			{
				# obviously, the poll couldn't be created. try to roll back things...
				if (!$post->Remove())
					$error .= $T['err_post_not_deleted_2'] . ' (' . t2h($post->db->LastError()) . ')<br />';
			}
			else
			{
				// get the page our new post is on inside its thread
				$page = ceil($post->Count('Thread=' . $threadid) / rc('posts_per_page'));

				// now is a good time to send out notifications
				if (!$newthread)
					SendWatchNotification($forum, $thread, $post, $page, trim($_POST['UserName']));
				else
				{
					$forum->Load($forumid);
					SendWatchNotification($forum, $thread, $post, $page, trim($_POST['UserName']));
				}

				// add ThreadWatch if selected
				$mode = $thread->IsWatched();
				$mode = ($mode & ~1) | ($_POST['NotifyEMail'] ? 1 : 0) * 1;
				if ($CFG['enable_icq']) $mode = ($mode & ~2) | ($_POST['NotifyICQ'] ? 1 : 0) * 2;
				if ($CFG['enable_jabber']) $mode = ($mode & ~4) | ($_POST['NotifyJabber'] ? 1 : 0) * 4;
				$thread->SetWatched($mode, $threadid);

				addlog('add_post ' . $post->GetID() . ' to ' . $threadid . ' ok');
				ForwardHTML($MODULE['thread'] . PD . 'id=' . $threadid . '&amp;page=' . $page . '&amp;nocount=1' . sid() . '#' . $post->GetID());
				exit();
			}
		}

		if ($error)
		{
			// something bad happened after (creating) the thread,
			// so if we created a new thread, it's better to remove it again
			if ($newthread)
				if (!$thread->Remove())
					$error .= $T['err_thread_not_deleted_2'] . ' (' . t2h($thread->db->LastError()) . ')<br />';
		}
	}

	unset($post);
}

// --------------------------------------- ACTION FOR POST EDITING ----------

if ($_POST['action'] == 'edit' && $_POST['preview'] != '')   // don't use $IsPreview to not display the post when adding/removing a poll
{
	if (trim($_POST['Msg']) == '')
	{
		$error .= $T['err_no_text'] . '<br />';
	}

	if (abbc_check(trim($_POST['Msg'])))
	{
		$error .= $T['err_abbc_syntax'] . '<br />';
		$_POST['preview'] = 'xx';   // just to show the preview
	}

	if ($_POST['Poll'] == 1 && $_POST['PollTimeout'] < 0)
	{
		$error .= $T['err_poll_timeout'] . '<br />';
	}
}

if ($_POST['action'] == 'edit' && $_POST['save'] != '' || $_GET['delete'] == 'yes')
{
	$post = new IPost;
	$post->Load($postid);
	$thread = new IThread;
	$thread->Load($post->GetThread());

	if ($_POST['Poll'] == 1 && trim($_POST['Question']) == '')
	{
		$error .= $T['err_poll_no_question'] . '<br />';
	}

	if (!$error && !CheckRights('editpost', $thread->GetForum(), $thread->GetID(), $post->GetUser(), $post->GetDate()))
	{
		$error .= $T['err_access_denied'] . '<br />';
	}
	else
	{
		if ($_POST['Remove'] == 1 || $_GET['delete'] == 'yes')
		{
			if (!CheckRights('removepost', $thread->GetForum(), $thread->GetID(), $post->GetUser(), $post->GetDate(), $thread->GetLastPostDate() == $post->GetDate() /* isLastPost */))
			{
				addlog('edit_post ' . $postid . ' no_access');
				$error .= $T['err_access_denied'] . '<br />';
			}
			else
			{
				switch ($post->Remove())
				{
					case 1:  // there are remaining posts in this thread
						// update LastPostDate for correct display order
						// note: there's no error detection on this operation
						$record = GetLastPost($thread->GetID());
						$thread->SetLastPostDate($record['Date']);

						addlog('remove_post ' . $postid . ' ok');
						ForwardHTML($MODULE['thread'] . PD . 'id=' . $post->GetThread() . '&amp;page=' . $page . '&amp;nocount=1' . sid());
						exit();
					case 2:  // this was the last post in this thread and the thread was removed
						addlog('remove_post ' . $postid . ' ok');
						ForwardHTML($MODULE['main'] . PD . 'id=' . $thread->GetForum() . '&amp;nocount=1' . sid());
						exit();
				}

				$error .= $T['err_post_not_deleted'] . ' (' . t2h($post->db->LastError()) . ')<br />';
			}
		}
		else
		{
			if (trim($_POST['Msg']) == '')
			{
				$error .= $T['err_no_text'] . '<br />';
			}

			if (abbc_check(trim($_POST['Msg'])))
			{
				$error .= $T['err_abbc_syntax'] . '<br />';
				$_POST['preview'] = 'xx';
			}

			if ($_FILES['AttachFile']['name'] != '' &&
				filesize($_FILES['AttachFile']['tmp_name']) > CheckRights('maxattachsize', $thread->GetForum(), $threadid))
			{
				$error .= $T['err_invalid_attach'] . '<br />';
			}

			if (!$error)
			{
				$thread = new IThread;
				$thread->Load($post->GetThread());

				$opt = $thread->GetOptions();
				if (CheckRights('closethread', $thread->GetForum(), $thread->GetID())) $opt = ($opt & ~1) | ($_POST['CloseThread'] ? 1 : 0) * 1;
				if (CheckRights('importantthread', $thread->GetForum(), $thread->GetID())) $opt = ($opt & ~2) | ($_POST['ImportantThread'] ? 1 : 0) * 2;
				$thread->SetOptions($opt);

				if ($_POST['NoEditNote'] == 1 && CheckRights('noeditnote', $thread->GetForum(), $thread->GetID()))
					$edituser = 0;
				else
					$edituser = $LoginUserID;

				if ($_POST['RemoveEditNote'] == 1 && CheckRights('removeeditnote', $thread->GetForum(), $thread->GetID()))
					$edituser = -1;

				$opt = ($_POST['NoSmileys'] ? 1 : 0) * 1;
				$opt += ($_POST['NoSpecialABBC'] ? 1 : 0) * 2;

				if (!$post->Change($postid, trim($_POST['UserName']), trim($_POST['Subject']), trim($_POST['Msg']), $opt, $edituser, $spamrating))
				{
					addlog('edit_post ' . $postid . ' error');
					$error .= $T['err_post_not_created'] . ' (' . t2h($post->db->LastError()) . ')<br />';
				}
				else
				{
					$first_post = $db->FastQuery1st('Posts', 'COUNT(*)', 'Date<' . $post->GetDate() . ' AND Thread=' . $post->GetThread()) == 0;
					$only_post = $db->FastQuery1st('Posts', 'COUNT(*)', 'Thread=' . $post->GetThread()) <= 1;

					// Change the post's user ID
					if (CheckRights('is_admin') && isset($_POST['NewUser']) && is_numeric($_POST['NewUser']))
						$post->SetUser(intval($_POST['NewUser']));

					// Move this post to another thread
					$moved_post = false;
					if (!$only_post &&
						CheckRights('closethread', $thread->GetForum(), $thread->GetID()) &&
						CheckRights('importantthread', $thread->GetForum(), $thread->GetID()) &&
					    isset($_POST['NewThread']) && is_numeric($_POST['NewThread']) &&
					    intval($_POST['NewThread']) > 0 &&
					    intval($_POST['NewThread']) != $thread->GetID())
					{
						$post->SetThreadArray(intval($_POST['NewThread']));
						$thread->SetLastPostDate();
						$thread->SetLastPostDate(false, intval($_POST['NewThread']));
						$moved_post = true;
					}

					// rename Thread's Subject if this is the 1st Post
					if ($first_post && !$moved_post)
					{
						$thread->SetSubject(trim($_POST['Subject']));
						if (isset($_POST['Desc'])) $thread->SetDesc(trim($_POST['Desc']));

						if (CheckRights('is_admin') && isset($_POST['NewUser']) && is_numeric($_POST['NewUser']))
							$thread->SetUser(intval($_POST['NewUser']), $_POST['UserName']);

						if ($thread->HasPoll() && CheckRights('editpoll', $thread->GetForum, $thread->GetID(), 0) ||
							!$thread->HasPoll() && CheckRights('createpoll', $thread->GetForum(), $thread->GetID()))
						{
							if ($_POST['Poll'] == 1)
							{
								if (trim($_POST['Question']) == '')
								{
									$error .= $T['err_poll_no_question'] . '<br />';
								}

								if (!$error && !($thread->GetOptions() & 4))
								{
									if (!$thread->SetOptions($thread->GetOptions() | 4))
									{
										$error .= $T['err_poll_not_created'] . ' (' . t2h($thread->db->LastError()) . ')<br />';
									}
								}

								$settitle = CheckRights('editpoll', $thread->GetForum, $thread->GetID(), $thread->CountVotes());

								if (!$error)
								{
									$timeout = $_POST['PollTimeout'];
									if ($timeout == 'x')
										$timeout = time();   // set time to NOW, so the poll is immediately closed
									elseif ($timeout == 'i')
										$timeout = 0;
									else
										$timeout = time() + $timeout * 24 * 3600;   // add some days

									$settimeout = !($thread->IsPollEnded() && $_POST['PollTimeout'] == 'x');

									if (!$thread->SetQuestion(trim($_POST['Question']), $timeout, $settitle, $settimeout))
									{
										$error .= $T['err_poll_details_not_saved'] . ' (' . t2h($thread->db->LastError()) . ')<br />';
									}
								}

								if (!$error)
								{
									for ($n = 1; $n <= 10; $n++)
									{
										$sort = $_POST['Sort' . $n];
										if (!$sort) $sort = $n;   // auto-assign position if none given

										if ($_POST['ID' . $n] != '')
										{
											// this is to be edited or deleted
											if (!$thread->ChangeVoteOption($_POST['ID' . $n], trim($_POST['Poll' . $n]), $sort, $settitle))
											{
												// oops, an error? now?!
												$error .= str_replace('%s', $n, $T['err_poll_change_reply_n']) . '<br />';
											}
										}
										elseif (trim($_POST['Poll' . $n]) != '')
										{
											// no ID given yet, create new option
											if (!$thread->CreateVoteOption(0, trim($_POST['Poll' . $n]), $sort))
											{
												// oops, an error? now?!
												$error .= str_replace('%s', $n, $T['err_poll_create_reply_n']) . '<br />';
											}
										}
									}
								}

								if ($error) addlog('edit_poll ' . $threadid . ' error');
							}
							else   // $_POST[Poll] = 0
							{
								if ($thread->HasPoll())
								{
									if (CheckRights('editpoll', $thread->GetForum, $thread->GetID(), $thread->CountVotes()))
									{
										if (!$thread->SetOptions($thread->GetOptions & ~4))
										{
											$error .= $T['err_poll_not_deleted'] . '<br />';
										}
										if (!$error && !$thread->RemoveAllVotes())
										{
											$error .= $T['err_votes_not_deleted'] . '<br />';
										}
									}
									else
									{
										$error .= $T['err_access_denied_delete_poll'] . '<br />';
									}
								}
							}
						}
					}

					// copy and link the attachment file or remove it
					$name = basename($_FILES['AttachFile']['name']);
					if ($_POST['RemoveAttach'] == 1)
					{
						$post->SetAttachFile('');
					}
					elseif ($name != '' &&
					        filesize($_FILES['AttachFile']['tmp_name']) <= CheckRights('maxattachsize', $thread->GetForum(), $thread->GetID()))
					{
						$x = strrpos($name, '.');
						if (is_valid_attach_name($name) && is_valid_attach_size($_FILES['AttachFile']['tmp_name'], $thread))
						{
							$ext = substr($name, $x);
							$name2 = 'post_' . $post->GetID() . $ext;

							if (!file_exists($attach_path)) mkdir($attach_path);
							if (move_uploaded_file($_FILES['AttachFile']['tmp_name'], $attach_path . $name2))
							{
								@chmod($attach_path . $name2, 0644);   // prevent access problems by the webserver
								if (!$post->SetAttachFile($name2, $name)) $ok = false;
							}
						}
					}

					# display success
					addlog('edit_post ' . $postid . ' ok');
					ForwardHTML($MODULE['thread'] . PD . 'id=' . $post->GetThread() . '&amp;page=' . $page . '&amp;nocount=1' . sid() . '#' . $post->GetID());
					exit();
				}
			}
		}
		unset($post);
	}
}

// --------------------------------------- ACTION FOR MESSAGE ADDING ----------

if ($_POST['action'] == 'addmessage' && $_POST['save'] != '' && CheckRights('editmessage', $forumid))
{
	$message = new IMessage;

	if (trim($_POST['Msg']) == '')
	{
		$error .= $T['err_no_text'] . '<br />';
	}

	if (!$error)
	{
		$opt = 0;
		$opt = ($opt & ~1) | ($_POST['ImportantMessage'] ? 1 : 0) * 1;
		$opt = ($opt & ~2) | ($_POST['RecursiveMessage'] ? 1 : 0) * 2;
		$opt = ($opt & ~4) | ($_POST['ShowInThreads'] ? 1 : 0) * 4;
		$opt = ($opt & ~24) | ($_POST['Access'] & 3) * 8;   // these are 2 bits: 8 and 16
		if (!$message->Add($forumid, $_POST['Subject'], $_POST['Msg'], $opt))
		{
			addlog('add_message to ' . $forumid . ' error');
			$error .= $T['err_message_not_saved'] . ' (' . t2h($message->db->LastError()) . ')<br />';
		}
		else
		{
			addlog('add_message to ' . $forumid . ' ok');
			ForwardHTML($MODULE['main'] . PD . 'id=' . $message->GetForum() . sid());
			exit();
		}
	}

	unset($message);
}

// --------------------------------------- ACTION FOR MESSAGE EDITING ----------

if ($_POST['action'] == 'editmessage' && $_POST['save'] != '')
{
	$message = new IMessage;
	$message->Load($messageid);

	if (CheckRights('editmessage', $message->GetForum()))
	{
		if ($_POST['Remove'] == 1)
		{
			if ($message->Remove($messageid))
			{
				addlog('remove_message ' . $messageid . ' ok');
				ForwardHTML($MODULE['main'] . PD . 'id=' . $message->GetForum() . sid());
				exit();
			}

			addlog('remove_message ' . $messageid . ' error');
			$error .= $T['err_message_not_deleted'] . ' (' . t2h($message->db->LastError()) . ')<br />';
		}
		else
		{
			if (trim($_POST['Msg']) == '')
			{
				$error .= $T['err_no_text'] . '<br />';
			}

			if (!$error)
			{
				$opt = $message->GetOptions();
				$opt = ($opt & ~1) | ($_POST['ImportantMessage'] ? 1 : 0) * 1;
				$opt = ($opt & ~2) | ($_POST['RecursiveMessage'] ? 1 : 0) * 2;
				$opt = ($opt & ~4) | ($_POST['ShowInThreads'] ? 1 : 0) * 4;
				$opt = ($opt & ~24) | ($_POST['Access'] & 3) * 8;   // these are 2 bits: 8 and 16
				if (!$message->Change($messageid, -1, $_POST['Subject'], $_POST['Msg'], $opt))
				{
					addlog('edit_message ' . $messageid . ' error');
					$error .= $T['err_message_not_saved'] . ' (' . t2h($message->db->LastError()) . ')<br />';
				}
				else
				{
					if (CheckRights('is_admin') && isset($_POST['NewUser']) && is_numeric($_POST['NewUser']))
						$message->SetUser(intval($_POST['NewUser']));

					addlog('edit_message ' . $messageid . ' ok');
					ForwardHTML($MODULE['main'] . PD . 'id=' . $message->GetForum() . sid());
					exit();
				}
			}
		}
	}
	unset($message);
}


// =============================================== POST EDIT FORM =========

// Post Edit <form>
//
// forumid = Forum to create new Thread in
// threadid = Thread to create reply in
// postid = Post to edit
// messageid = Message to edit
// thread = Thread object
//
function PostForm($forumid, $threadid, $postid, $messageid, &$thread)
{
	global $MODULE, $ME, $TABLE, $LoginUserID, $LoginUserName, $LoginUser, $error, $abbc_cfg, $abbc_smileys, $abbc_scan, $page, $maxpostlen, $spamrating, $attach_path, $CFG, $db, $T, $special_char_list, $NEWLAYOUT, $IsPreview;

	$p = ($IsPreview || $error);   // take data from (P)revious form posting
	$m = ($messageid != 0);   // we're here to edit a (M)essage, no Post

	if (!$m)
		$post = new IPost;
	else
		$message = new IMessage;

	# BEGIN PREVIEW

	if ($_POST['preview'] != '')   // don't use $IsPreview to not display the post when adding/removing a poll
	{
		if (!$m)
		{
			// this is a POST
			if ($postid)
			{
				$post->Load($postid);
				if ($_POST['RemoveEditNote'] == 1)
				{
					$post->EditCount = 0;
				}
				elseif ($_POST['NoEditNote'] != 1)
				{
					$post->EditCount++;
					$post->EditUser = $LoginUserID;
					$post->EditDate = time();
				}
				if ($post->GetUser() <= 0) $post->UserName = $_POST['UserName'];
				$post->Subject = $_POST['Subject'];
				$post->Msg = $_POST['Msg'];
				$post->Options = ($_POST['NoSmileys'] ? 1 : 0) * 1 + ($_POST['NoSpecialABBC'] ? 1 : 0) * 2;
				if ($_POST['RemoveAttach'] == 1) $post->AttachFile = '';
				$post->SpamRating = $spamrating;
			}
			else
			{
				$post->LoadFromRecord(array(
						'ID' => 0,
						'Date' => time(),
						'User' => $LoginUserID,
						'UserName' => $_POST['UserName'],
						'Subject' => $_POST['Subject'],
						'Msg' => $_POST['Msg'],
						'Options' => ($_POST['NoSmileys'] ? 1 : 0) * 1 + ($_POST['NoSpecialABBC'] ? 1 : 0) * 2,
						'EditCount' => 0,
						'SpamRating' => $spamrating
					));
			}

			CountUserPosts();

			BeginPostTable();
			ShowPost($post, $thread, 0, 0, 0, true);
			if ($NEWLAYOUT) PostDelimiter(-1);
			EndPostTable();
		}
		else
		{
			// this is a MESSAGE
			if ($messageid > 0)
			{
				// Message is already saved, just edit it -> we have a User
				$message->Load($messageid);
				$message->Subject = $_POST['Subject'];
				$message->Msg = $_POST['Msg'];

				$opt = $message->GetOptions();
				$opt = ($opt & ~1) | ($_POST['ImportantMessage'] ? 1 : 0) * 1;
				$message->Options = $opt;

			}
			else
			{
				// Message is about to be created -> fill in the required fields for display
				$opt = 0;
				$opt = ($opt & ~1) | ($_POST['ImportantMessage'] ? 1 : 0) * 1;
				$opt = ($opt & ~2) | ($_POST['RecursiveMessage'] ? 1 : 0) * 2;
				$opt = ($opt & ~4) | ($_POST['ShowInThreads'] ? 1 : 0) * 4;
				$opt = ($opt & ~24) | ($_POST['Access'] & 3) * 8;   // these are 2 bits: 8 and 16

				$message->LoadFromRecord(array(
						'ID' => 0,
						'Forum' => $forumid,
						'Date' => time(),
						'User' => $LoginUserID,
						'Subject' => $_POST['Subject'],
						'Msg' => $_POST['Msg'],
						'Options' => $opt
					));
			}

			BeginMessageTable(1 | ($message->GetOptions() & 1) * 2);
			ShowMessage($message, true);
			EndMessageTable();
		}
		echo '<br />';
	}

	# END PREVIEW

	$user = new IUser;

	echo '<form action="' . $ME . sid(2) . '" method="post" name="form" enctype="multipart/form-data">';
	#echo '<form action='$ME" . sid(2) . "' method=post name=form>';

	if ($postid > 0 && !$m)   // EDIT POSTS
	{
		$post->Load($postid);
		echo '<input type="hidden" name="action2" value="edit" />';   // the 1st field will get lost on IE with unicode postings...
		echo '<input type="hidden" name="action" value="edit" />';
		echo '<input type="hidden" name="id" value="' . $postid . '" />';
		echo '<input type="hidden" name="page" value="' . $page . '" />';

		$first_post = $db->FastQuery1st('Posts', 'COUNT(*)', 'Date<' . $post->GetDate() . ' AND Thread=' . $post->GetThread()) == 0;
		$only_post = $db->FastQuery1st('Posts', 'COUNT(*)', 'Thread=' . $post->GetThread()) <= 1;

		// Save current 'have a poll' status in a hidden input field and
		// update its value from previous 'Poll' or 'NoPoll' button click
		if ($p)
			$x = $_POST['Poll'] ? 1 : 0;
		else
			$x = $thread->HasPoll() ? 1 : 0;
		if ($_POST['AddPoll'] != '') $x = 1;
		if ($_POST['RemovePoll'] != '') $x = 0;
		$_POST['Poll'] = $x;
		if ($first_post) echo '<input type="hidden" name="Poll" value="' . $x . '" />';

		if ($post->GetUser() > 0)
		{
			$user->Load($post->GetUser());
			$usernameinput = t2h($user->GetName());
		}
		else
		{
			$username = ($p ? $_POST['UserName'] : trim($post->GetUserName()));
			$usernameinput = '<input type="text" class="text" name="UserName" size="20"  maxlength="40" value="' . t2h($username, false) . '" />';
		}

		// Change the user of this post
		if (CheckRights('is_admin'))
			$usernameinput .= ' - ID: <input type="text" class="text" name="NewUser" size="5" maxlength="10" value="' .
				($p ? htmlquote($_POST['NewUser']) : $post->GetUser()) .
				'" />';
		// Move this post to another thread
		if (!$only_post &&
		    CheckRights('closethread', $thread->GetForum(), $thread->GetID()) &&
		    CheckRights('importantthread', $thread->GetForum(), $thread->GetID()))
			$usernameinput .= ' - ' . $T['thread'] . ': <input type="text" class="text" name="NewThread" size="5" maxlength="10" value="' .
				($p ? htmlquote($_POST['NewThread']) : $thread->GetID()) .
				'" />';

		$subject = ($p ? trim($_POST['Subject']) : $post->GetSubject());
		$desc = ($p ? trim($_POST['Desc']) : $thread->GetDesc());
		$msg = ($p ? trim($_POST['Msg0']) : $post->GetMsg());
	}
	elseif (!$m)   // NEW POSTS
	{
		echo '<input type="hidden" name="action" value="add" />';
		echo '<input type="hidden" name="forum" value="' . $forumid . '" />';

		$first_post = ($threadid == 0);

		// Save current 'have a poll' status in a hidden input field and
		// update its value from previous 'Poll' or 'NoPoll' button click
		if ($p)
			$x = $_POST['Poll'] ? 1 : 0;
		else
			$x = $thread->HasPoll() ? 1 : 0;
		if ($_POST['AddPoll'] != '') $x = 1;
		if ($_POST['RemovePoll'] != '') $x = 0;
		$_POST['Poll'] = $x;
		if ($first_post) echo '<input type="hidden" name="Poll" value="' . $x . '" />';

		if ($LoginUserID)
		{
			echo '<input type="hidden" name="user" value="' . $LoginUserID . '" />';
			$usernameinput = t2h($LoginUserName);
		}
		else
		{
			echo '<input type="hidden" name="user value="0" />';
			$usernameinput = '<input type="text" class="text" name="UserName" size="20" maxlength="40" value="' . t2h($_POST['UserName'], false) . '" />';
		}

		$subject = ($p ? trim($_POST['Subject']) : '');
		$desc = ($p ? trim($_POST['Desc']) : '');
		$msg = ($p ? trim($_POST['Msg0']) : '');

		if ($_REQUEST['quote'] > 0)
		{
			$post->Load($_REQUEST['quote']);
			if ($post->GetUser() > 0)
			{
				$user->Load($post->GetUser());
				$username = $user->GetName();
			}
			else
			{
				$username = $post->GetUserName();
			}
			echo '<input type="hidden" name="thread" value="' . $post->GetThread() . '" />';
			echo '<input type="hidden" name="replyto" value="' . $_REQUEST['quote'] . '" />';

			// add timestamp to quotation if it's older than x days
			if (time() - $post->GetDate() > rc('quote_with_date') * 24 * 3600)
				$msg = '[quote=' . $username . ':' . $post->GetDate() . "]\n" . $post->GetMsg() . "\n[/quote]\n\n";
			else
				$msg = '[quote=' . $username . "]\n" . $post->GetMsg() . "\n[/quote]\n\n";
		}
		else
		{
			echo '<input type="hidden" name="thread" value="' . $threadid . '" />';
			echo '<input type="hidden" name="replyto" value="' . (isset($_REQUEST['replyto']) ? $_REQUEST['replyto'] : 0) . '" />';
		}
	}
	else   // this means $m; MESSAGES
	{
		if ($messageid > 0)   // EDIT
		{
			echo '<input type="hidden" name="action" value="editmessage" />';
			echo '<input type="hidden" name="message" value="' . $messageid . '" />';
			$message->Load($messageid);
			$subject = ($p ? trim($_POST['Subject']) : $message->GetSubject());
			$msg = ($p ? trim($_POST['Msg0']) : $message->GetMsg());
			$newuserid = ($p ? $_POST['NewUser'] : $message->GetUser());
		}
		else   // NEW
		{
			echo '<input type="hidden" name="action" value="addmessage" />';
			echo '<input type="hidden" name="message" value="-1" />';
			echo '<input type="hidden" name="forum" value="' . $forumid . '" />';
			$subject = ($p ? trim($_POST['Subject']) : '');
			$msg = ($p ? trim($_POST['Msg0']) : '');
			$newuserid = ($p ? $_POST['NewUser'] : '');
		}

		$usernameinput = t2h($LoginUserName);
		if ($messageid > 0 && CheckRights('is_admin'))
			$usernameinput .= ' - ID: <input type="text" class="text" name="NewUser" size="5" maxlength="10" value="' . htmlquote($newuserid) . '" />';
	}

	// display smilies/special_chars table in the right column?
	// if set to false, both tables will be displayed left to the message textbox, if enabled
	$use_rcol = rc('post_use_rcol');

	echo '<table cellspacing="' . $TABLE['post']['space'] . '" cellpadding="' . $TABLE['post']['pad'] . '" border="' . $TABLE['post']['border'] . '" width="100%"' . $TABLE['post']['fixed'] . '>';
	echo '<colgroup>';
	echo '<col width="125" />';
	echo '<col width="470" />';   // TODO: this is a place to update when we offer a bigger textbox...
	echo '<col />';
	echo '</colgroup>';

	echo '<tr valign="top">';
	echo '<td class="edit_post_head_left" colspan="2">';
	if ($m) if ($messageid) echo $T['edit_message'];
	        else            echo $T['compose_message'];
	else    if ($postid) echo $T['edit_post'];
	        else         echo $T['compose_post'];
	echo '</td>';
	echo '<td class="edit_post_head_right">&nbsp;</td>';
	echo '</tr>';

	# USER
	echo '<tr>';
	echo '<td class="edit_post">' . $T['username'] . '</td>';
	echo '<td class="edit_post" colspan="2">' . $usernameinput . '</td>';
	echo '</tr>';

	# SUBJECT
	echo '<tr>';
	if ($m || $first_post)
		echo '<td class="edit_post">' . $T['subject'] . ':</td>';
	else
		echo '<td class="edit_post"><i>' . $T['subject'] . ':</i></td>';
	echo '<td class="edit_post" colspan="2">';
	echo '<input type="text" class="text" name="Subject" size="71" maxlength="150" value="' . t2h($subject, false) . '" tabindex="1" />';
	echo '</td>';
	echo '</tr>';

	if (!$m && $first_post)
	{
		echo '<tr>';
		echo '<td class="edit_post"><i>' . $T['description'] . ':</i></td>';
		echo '<td class="edit_post" colspan="2">';
		echo '<input type="text" class="text" name="Desc" size="71" maxlength="150" value="' . t2h($desc, false) . '" tabindex="2" />';
		echo '</td>';
		echo '</tr>';
	}

	# BBCODE:
	echo '<tr valign="top">';
	echo '<td class="edit_post">' . $T['advanced_bbcode'] . ':</td>';
	echo '<td class="edit_post" colspan="2">';

	echo '<input type="button" class="button" value="' . $T['f_b'] . '" style="width:23px; font-weight:bold;" title="' . $T['f_bold'] . '" onClick="doCmd(\'bold\');" />';
	echo '<input type="button" class="button" value="' . $T['f_i'] . '" style="width:23px; font-style:italic;" title="' . $T['f_italic'] . '" onClick="doCmd(\'italic\');" />';
	echo '<input type="button" class="button" value="' . $T['f_u'] . '" style="width:23px; text-decoration:underline;" title="' . $T['f_underline'] . '" onClick="doCmd(\'underline\');" />';
	echo '<input type="button" class="button" value="' . $T['f_s'] . '" style="width:23px; text-decoration:line-through;" title="' . $T['f_strike'] . '" onClick="doCmd(\'strike\');" />';
	echo '<input type="button" class="button" value="' . $T['f_m'] . '" style="width:23px; font-family:monospace;" title="' . $T['f_fixed'] . '" onClick="doCmd(\'mono\');" />'; // TODO: Firefox/linux aligns this button a fex pixels under
	echo '&nbsp;';
	echo '<input type="button" class="button" value="' . $T['f_quote'] . '" onClick="doCmd(\'quote\');" />';
	echo '<input type="button" class="button" value="' . $T['f_code'] . '" onClick="doCmd(\'code\');" />';
	echo '<input type="button" class="button" value="' . $T['f_link'] . '" onClick="doCmd(\'url\');" />';
	echo '<input type="button" class="button" value="' . $T['f_image'] . '" onClick="doCmd(\'img\');" />';
	echo '&nbsp;';
	echo '<input type="button" class="button" value="" style="width:13px; background:red;" title="' . $T['f_red'] . '" onClick="doCmd(\'color\', \'red\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:orange;" title="' . $T['f_orange'] . '" onClick="doCmd(\'color\', \'orange\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:yellow;" title="' . $T['f_yellow'] . '" onClick="doCmd(\'color\', \'yellow\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:green;" title="' . $T['f_green'] . '" onClick="doCmd(\'color\', \'green\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:darkcyan;" title="' . $T['f_cyan'] . '" onClick="doCmd(\'color\', \'darkcyan\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:blue;" title="' . $T['f_blue'] . '" onClick="doCmd(\'color\', \'blue\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:magenta;" title="' . $T['f_magenta'] . '" onClick="doCmd(\'color\', \'magenta\');" />';
	echo '<input type="button" class="button" value="" style="width:13px; background:gray;" title="' . $T['f_gray'] . '" onClick="doCmd(\'color\', \'gray\');" />';
	echo '&nbsp; ';
	echo '<a href="' . htmlquote('javascript:popup("' . $MODULE['abbc_help'] . '", 430, 550);') . '">' . $T['f_more'] . '</a>';

	echo '</td>';
	echo '</tr>';

	# SMILEYS:
	$out_smilies = '';
	if (rc('smilies'))
	{
		$out_smilies .= $T['smilies'] . ':<br />';

		$out_smilies .= '<table cellspacing="' . $TABLE['smiley']['space'] . '" cellpadding="' . $TABLE['smiley']['pad'] . '" border="' . $TABLE['smiley']['border'] . '" class="smiley_table">';

		$c = sizeof($abbc_smileys);
		$cols = rc('smilie_cols');
		$rows = rc('smilie_rows');
		$max_c = $rows * $cols;
		if ($c > $max_c) $c = $max_c;   // restrict number of smilies to table size
		for ($y = 0; $y < $rows; $y++)
		{
			$out_smilies .= '<tr>';
			for ($x = 0; $x < $cols; $x++)
			{
				if ($y * $cols + $x < $c)
				{
					$code = $abbc_smileys[$y * $cols + $x]['code'];
					$code_js = str_replace('\'', '\\x27', $code);
					$out_smilies .= '<td class="smiley_cell_0" onmouseover="this.className=\'smiley_cell_1\';" onmouseout="this.className=\'smiley_cell_0\';" onClick="insert_text(\' ' . $code_js . ' \');" title="' . $code . '">';
					$out_smilies .= abbc_reg_replace_smiley('', $y * $cols + $x);
					$out_smilies .= '</td>';
				}
				else
				{
					$out_smilies .= '<td class="smiley_cell_0">&nbsp;</td>';
				}
			}
			$out_smilies .= '</tr>';
		}
		$out_smilies .= '</table><br />';
	}

	# SPECIAL CHARACTERS:
	$out_special = '';
	if (rc('special_chars'))
	{
		if ($out_smilies != '') $out_smilies .= '&nbsp;';
		$out_special .= $T['special_chars'] . ':<br />';
		$out_special .= '<table cellspacing="' . $TABLE['smiley']['space'] . '" cellpadding="' . $TABLE['smiley']['pad'] . '" border="' . $TABLE['smiley']['border'] . '" class="smiley_table">';

		$c = sizeof($special_char_list);
		$cols = rc('special_cols');
		$rows = rc('special_rows');
		$max_c = $rows * $cols;
		if ($c > $max_c) $c = $max_c;   // restrict number of characters to table size
		for ($y = 0; $y < $rows; $y++)
		{
			$out_special .= '<tr>';
			for ($x = 0; $x < $cols; $x++)
			{
				if ($y * $cols + $x < $c)
				{
					$n = $y * $cols + $x;
					$out_special .= '<td class="smiley_cell_0" onmouseover="this.className=\'smiley_cell_1\';" onmouseout="this.className=\'smiley_cell_0\';" onClick="insert_text(\'' . $special_char_list[$n] . '\');" title="' . $special_char_list[$n] . '"><span class="special_char">' . $special_char_list[$n] . '</span></td>';
				}
				else
				{
					$out_special .= '<td class="smiley_cell_0">&nbsp;</td>';
				}
			}
			$out_special .= '</tr>';
		}

		$out_special .= '</table>';
	} // if special_chars

	# MESSAGE
	echo '<tr valign="top">';
	echo '<td class="edit_post">' . $T['text'] . ':';
	if (!$use_rcol)
	{
		echo '<br /><br />';
		echo $out_smilies;
		echo $out_special;
	}
	echo '</td>';
	echo '<td class="edit_post"' . ($use_rcol ? '' : ' colspan="2"') . '>';
	// cols=70 rows=19
	// TODO: this is a place to implement a bigger textbox...
	$height = 250;
	if (strlen($msg) > 1500) $height = 500;

	echo '<textarea class="text" name="Msg" style="width:450px; height:' . $height . 'px;" onkeydown="return text_keydown(this)" onkeypress="return checklength(this, ' . $maxpostlen . ')" onkeyup="document.form.Length.value=this.value.length; checklength(this, ' . $maxpostlen . ')" onblur="return checklength(this, ' . $maxpostlen . ')" onfocus="document.form.Length.value=this.value.length;" tabindex="3">' . t2h($msg, false) . '</textarea>';
	echo '</td>';
	if ($use_rcol)
	{
		echo '<td class="edit_post">';
		echo $out_smilies;
		echo $out_special;
		echo '</td>';
	}
	echo '</tr>';

	# SUBMIT / PREVIEW CONTROLS
	echo '<tr valign="top">';
	echo '<td class="edit_post">&nbsp;</td>';
	echo '<td class="edit_post" colspan="2">';
	echo '<input type="submit" class="button" name="save" value="' . $T['save'] . '" accesskey="' . $T['save_key'] . '" onclick="disableObj(\'document.form.save\');" tabindex="4" /> <small>' . $T['save_key_name'] . '</small> ';
	echo '<input type="submit" class="button" name="preview" value="' . $T['preview'] . '" accesskey="' . $T['preview_key'] . '" onclick="disableObj(\'document.form.save\');" tabindex="5" /> <small>' . $T['preview_key_name'] . '</small> ';

	if (!$m && $first_post && CheckRights('createpoll', $thread->GetForum(), $thread->GetID()))
	{
		if ($p)
			$chk = $_POST['Poll'];
		else
			$chk = $thread->HasPoll();

		if ($chk)
			echo '<input type="submit" class="button" name="RemovePoll" value="' . $T['poll_remove'] . '" /> ';
		else
			echo '<input type="submit" class="button" name="AddPoll" value="' . $T['poll_add'] . '" /> ';
	}

	//echo '<a href="javascript:getlength(document.form.Msg, $maxpostlen)">Textlnge prfen</a>';
	if (rc('post_show_textlength'))
	{
		if (rc('small_layout')) echo '<br />';
		else                    echo '&nbsp; ';
		echo $T['text_length'] . ': <input type="text" class="transparent" name="Length" value="..." readonly="readonly" size="' . ceil(log10($maxpostlen + 1)) . '" style="text-align:right;" /> ' . $T['chars'] . ' <small>(' . $T['maximum'] . ': ' . $maxpostlen . ')</small>';

		echo '<script type="text/javascript">' /*. '<![CDATA['*/;
		echo 'document.form.Length.value=document.form.Msg.value.length;';
		echo /*']]>' .*/ '</script>';
	}
	echo '</td>';
	echo '</tr>';

	# OPTIONS
	echo '<tr valign="top">';
	echo '<td class="edit_post">' . $T['options'] . ':</td>';
	echo '<td class="edit_post" colspan="2">';

	if (!$m)
	{
		# OPTIONS FOR NEW POSTS
		if (!$postid)
		{
			# is user already watching this thread?
			$mode = $thread->IsWatched($threadid);
			if (!($mode & 7)) $mode = $LoginUser->GetDefaultNotify();

			if ($LoginUser->GetEMail() != '' ||
				($LoginUser->GetICQ() != '' && $CFG['enable_icq']) ||
				($LoginUser->GetJabber() != '' && $CFG['enable_jabber']) ||
				$mode)
			{
				if ($LoginUser->GetEMail() != '' ||
					($LoginUser->GetICQ() != '' && $CFG['enable_icq']) ||
					($LoginUser->GetJabber() != '' && $CFG['enable_jabber']))
				{
					echo $T['notify_via'] . ' ';

					if ($LoginUser->GetEMail() != '')
					{
						if ($p)
							$chk = ($_POST['NotifyEMail'] == 1) ? ' checked="checked"' : '';
						else
							$chk = ($mode & 1) ? ' checked="checked"' : '';
						echo ' <input type="checkbox" class="radio" name="NotifyEMail" id="NotifyEMail" value="1"' . $chk . ' /><label for="NotifyEMail">' . $T['e_mail'] . '</label>';
					}
					if ($LoginUser->GetJabber() != '' && $CFG['enable_jabber'])
					{
						if ($p)
							$chk = ($_POST['NotifyJabber'] == 1) ? ' checked="checked"' : '';
						else
							$chk = ($mode & 4) ? ' checked="checked"' : '';
						echo ' <input type="checkbox" class="radio" name="NotifyJabber" id="NotifyJabber" value="1"' . $chk . ' /><label for="NotifyJabber">' . $T['jabber'] . '</label>';
					}
					if ($LoginUser->GetICQ() != '' && $CFG['enable_icq'])
					{
						if ($p)
							$chk = ($_POST['NotifyICQ'] == 1) ? ' checked="checked"' : '';
						else
							$chk = ($mode & 2) ? ' checked="checked"' : '';
						echo ' <input type="checkbox" class="radio" name="NotifyICQ" id="NotifyICQ" value="1"' . $chk . ' /><label for="NotifyICQ">' . $T['icq'] . '</label>';
					}
				}

				if ($mode)
				{
					$x = false;
					if ($mode & 1)
					{
						$x .= '<b>' . $T['e_mail'] . '</b>';
					}
					if ($mode & 4 && $CFG['enable_jabber'])
					{
						if ($x) $x .= ' ' . $T['and'] . ' ';
						$x .= '<b>' . $T['jabber'] . '</b>';
					}
					if ($mode & 2 && $CFG['enable_icq'])
					{
						if ($x) $x .= ' ' . $T['and'] . ' ';
						$x .= '<b>' . $T['icq'] . '</b>';
					}
					if ($x) echo ' <small>' . str_replace('%s', $x, $T['you_watch_this_by_x']) . '</small>';
				}

				echo '<br />';
			}
		}

		// Checkbox was replaced by AddPoll and RemovePoll buttons
		/*if ($first_post && CheckRights('createpoll', $thread->GetForum(), $thread->GetID()))
		{
			if ($p)
				$chk = ($_POST['Poll'] == 1) ? ' checked="checked"' : '';
			else
				$chk = $thread->HasPoll() ? ' checked="checked"' : '';
			echo '<input type="checkbox" class="radio" name="x-Poll" id="Poll" value="1"' . $chk . ' /><label for="Poll">' . $T['poll'] . '</label> <small>' . $T['update_with_preview'] . '</small><br />';
		}*/

		if ($p)
			$chk = ($_POST['NoSmileys'] == 1) ? ' checked="checked"' : '';
		else
			if ($postid)
				$chk = ($post->GetOptions() & 1) ? ' checked="checked"' : '';
			else
				$chk = '';
		echo '<input type="checkbox" class="radio" name="NoSmileys" id="NoSmileys" value="1"' . $chk . ' /><label for="NoSmileys">' . $T['no_smilies'] . '</label> ';

		if ($p)
			$chk = ($_POST['NoSpecialABBC'] == 1) ? ' checked="checked"' : '';
		else
			if ($postid)
				$chk = ($post->GetOptions() & 2) ? ' checked="checked"' : '';
			else
				$chk = '';
		echo '<input type="checkbox" class="radio" name="NoSpecialABBC" id="NoSpecialABBC" value="1"' . $chk . ' /><label for="NoSpecialABBC">' . $T['no_special_abbc_syntax'] . '</label><br />';

		# OPTIONS FOR POST EDITING
		if ($postid > 0)
		{
			if (CheckRights('noeditnote', $thread->GetForum(), $thread->GetID()))
			{
				if ($p)
					$chk = ($_POST['NoEditNote'] == 1) ? ' checked="checked"' : '';
				else
					$chk = '';
				echo '<input type="checkbox" class="radio" name="NoEditNote" id="NoEditNote" value="1"' . $chk . ' /><label for="NoEditNote">' . $T['no_edit_note'] . '</label> ' . $T['leaves_existing'];

				if (CheckRights('removeeditnote', $thread->GetForum(), $thread->GetID()))
				{
					if ($p)
						$chk = ($_POST['RemoveEditNote'] == 1) ? ' checked="checked"' : '';
					else
						$chk = '';
					echo ' &nbsp;<input type="checkbox" class="radio" name="RemoveEditNote" id="RemoveEditNote" value="1"' . $chk . ' /><label for="RemoveEditNote">' . $T['remove_edit_note'] . '</label>';
				}

				echo '<br />';
			}
		}

		if (CheckRights('closethread', $thread->GetForum(), $thread->GetID())
			|| CheckRights('importantthread', $thread->GetForum(), $thread->GetID()))
		{
			echo $T['thread_is'];

			if (CheckRights('closethread', $thread->GetForum(), $thread->GetID()))
			{
				if ($p)
					$chk = ($_POST['CloseThread'] == 1) ? ' checked="checked"' : '';
				else
					$chk = ($thread->GetOptions() & 1) ? ' checked="checked"' : '';
				echo ' <input type="checkbox" class="radio" name="CloseThread" id="CloseThread" value="1"' . $chk . ' /><label for="CloseThread">' . $T['closed'] . '</label>';
			}

			if (CheckRights('importantthread', $thread->GetForum(), $thread->GetID()))
			{
				if ($p)
					$chk = ($_POST['ImportantThread'] == 1) ? ' checked="checked"' : '';
				else
					$chk = ($thread->GetOptions() & 2) ? ' checked="checked"' : '';
				echo ' <input type="checkbox" class="radio" name="ImportantThread" id="ImportantThread" value="1"' . $chk . ' /><label for="ImportantThread">' . $T['important'] . '</label>';
			}

			echo '<br />';
		}
	}

	if ($m)
	{
		if ($p)
			$chk = ($_POST['ImportantMessage'] == 1) ? ' checked="checked"' : '';
		else
			$chk = ($messageid > 0 ? $message->GetOptions() & 1 : 0) ? ' checked="checked"' : '';
		echo '<input type="checkbox" class="radio" name="ImportantMessage" id="ImportantMessage" value="1"' . $chk . ' /><label for="ImportantMessage">' . $T['important_message'] . '</label><br />';

		if ($p)
			$chk = ($_POST['RecursiveMessage'] == 1) ? ' checked="checked"' : '';
		else
			$chk = ($messageid > 0 ? $message->GetOptions() & 2 : 0) ? ' checked="checked"' : '';
		echo '<input type="checkbox" class="radio" name="RecursiveMessage" id="RecursiveMessage" value="1"' . $chk . ' /><label for="RecursiveMessage">' . $T['recursive_message'] . '</label><br />';

		if ($p)
			$chk = ($_POST['ShowInThreads'] == 1) ? ' checked="checked"' : '';
		else
			$chk = ($messageid > 0 ? $message->GetOptions() & 4 : 0) ? ' checked="checked"' : '';
		echo '<input type="checkbox" class="radio" name="ShowInThreads" id="ShowInThreads" value="1"' . $chk . ' /><label for="ShowInThreads">' . $T['message_show_in_threads'] . '</label><br />';

		if ($p)
			$val = $_POST['Access'];
		else
			$val = ($messageid > 0 ? $message->GetOptAccess() : 0);
		echo $T['display_for'] . ' ';
		echo '<input type="radio" class="radio" name="Access" id="Access_0" value="0"' . ($val == 0 ? ' checked="checked"' : '') . ' /><label for="Access_0">' . $T['all'] . '</label> ';
		echo '<input type="radio" class="radio" name="Access" id="Access_1" value="1"' . ($val == 1 ? ' checked="checked"' : '') . ' /><label for="Access_1">' . $T['guests'] . '</label> ';
		echo '<input type="radio" class="radio" name="Access" id="Access_2" value="2"' . ($val == 2 ? ' checked="checked"' : '') . ' /><label for="Access_2">' . $T['users'] . '</label> ';
		echo '<input type="radio" class="radio" name="Access" id="Access_3" value="3"' . ($val == 3 ? ' checked="checked"' : '') . ' /><label for="Access_3">' . $T['moderators'] . '</label><br />';
	}

	if (!$m && $postid > 0 && CheckRights('removepost', $thread->GetForum(), $thread->GetID(), $post->GetUser(), $post->GetDate(), $thread->GetLastPostDate() == $post->GetDate() /* isLastPost */)
		|| $m && $messageid > 0 && CheckRights('editmessage', $message->GetForum()))
	{
		if ($p)
			$chk = ($_POST['Remove'] == 1) ? ' checked="checked"' : '';
		else
			$chk = '';

		$what = ($m ? $T['delete_message'] : $T['delete_post']);

		echo '<input type="checkbox" class="radio" name="Remove" id="Remove" value="1"' . $chk . ' ' . suredelete() . ' /><label for="Remove">' . $what . '</label><br />';
	}

	echo '</td>';
	echo '</tr>';

	# FILE ATTACHMENT

	if (!$m)
	{
		if ($postid > 0 && $post->GetAttachFile() != '')
		{
			echo '<tr valign="top">';
			echo '<td class="edit_post"><i>' . $T['attach_file'] . ':</i></td>';
			echo '<td class="edit_post" colspan="2">';
			echo $T['file_is_attached'] . ': ' . t2h($post->GetAttachFileName()) . ' <small>(' .
				@filesize($attach_path . $post->GetAttachFile()) . ' ' . $T['bytes'] . ', ' .
				$post->GetAttachDLCount() . $T['downloads'] . ')</small> &nbsp; ';
			echo '<input type="checkbox" class="radio" name="RemoveAttach" id="RemoveAttach" value="1"' .
				($_POST['RemoveAttach'] == 1 ? ' checked="checked"' : '') .
				' /><label for="RemoveAttach">' . $T['remove_attach'] . '</label>';
			echo '</td>';
			echo '</tr>';
		}
		elseif (CheckRights('maxattachsize', $thread->GetForum(), $threadid))
		{
			// maximum upload file size if minimum of applied ACL and PHP conf setting
			$maxsize = CheckRights('maxattachsize', $thread->GetForum(), $threadid);
			$phpsize = unverb_size(ini_get('upload_max_filesize'));
			$maxsize = min($phpsize, $maxsize);

			echo '<tr valign="top">';
			echo '<td class="edit_post"><i>' . $T['attach_file'] . ':</i></td>';
			echo '<td class="edit_post" colspan="2">';
			//echo '<input type=hidden name='MAX_FILE_SIZE' value='" .
			//	CheckRights('maxattachsize', $thread->GetForum(), $threadid) . "' />';
			// NOTE: this directive causes PHP to throw a warning before programme execution
			//       that I couldn't prevent. moreover doesn't browser support seem to prove
			//       the need for this feature.
			echo '<input type="file" class="text" name="AttachFile" size="52" /> &nbsp; <small>(' . $T['maximum'] . ': ' .
				format_number($maxsize, 1, 1024, ' ') .
				'B)</small><br />';
			echo '<small>' . $T['select_file_on_save'] . '</small>';
			echo '</td>';
			echo '</tr>';
		}
	}

	# POLL DEFINITION
	if ($first_post && $p && $_POST['Poll'] ||
		$first_post && !$p && $thread->HasPoll())
	{
		echo '<tr valign="top">';
		echo '<td class="edit_post">' . $T['poll'];
		echo '<input type="hidden" name="PollDef" value="1" />';
		echo '</td>';
		echo '<td class="edit_post" colspan="2">';

		$settitle = !$threadid || CheckRights('editpoll', $thread->GetForum, $thread->GetID(), $thread->CountVotes());

		echo $T['poll_question'] . '<br />';
		echo '<input type="text" class="' . ($settitle ? 'text' : 'transparent') . '" name="Question" size="50" maxlength="150" value="' . htmlquote($p ? trim($_POST['Question']) : $thread->GetQuestion()) . '" /><br />';

		if ($threadid)
			echo $T['poll_replies_sort'] . '<br />';
		else
			echo $T['poll_replies'] . '<br />';

		$votes = $thread->GetVotes();
		for ($n = 1; $n <= 10; $n++)
		{
			if ($votes[$n - 1]['Title'] == '' && !$settitle) break;   // don't show empty, non-editable lines

			echo '<input type="hidden" name="ID' . $n . '" value="' . $votes[$n - 1]['ID'] . '" />';
			echo '<input type="text" class="' . ($settitle ? 'text' : 'transparent') . '" name="Poll' . $n . '" size="50" maxlength="100" value="' . ($p ? htmlquote(trim($_POST["Poll$n"])) : htmlquote($votes[$n - 1]['Title'])) . '"' . ($settitle ? '' : ' readonly="readonly"') . ' /> ';
			if ($threadid)
			{
				echo '<input type="text" class="text" name="Sort' . $n . '" size="3" maxlength="3" value="' . ($p ? trim($_POST["Sort$n"]) : $votes[$n - 1]['Sort']) . '" /> ';
				if ($votes[$n - 1]['Title'] != '')
				{
					$x = $votes[$n - 1]['Votes'];
					if (!$x) echo $T['no_vote'];
					elseif ($x > 1) echo $x . ' ' . $T['votes'];
					else echo $x . ' ' . $T['vote'];
				}
			}
			echo '<br />';
		}

		if ($thread->HasPoll() && $thread->GetPollTimeout())
		{
			$timeleft = round(($thread->GetPollTimeout() - time()) / 24 / 3600, 3);
			if ($timeleft <= 0) $timeleft = 'x';
		}
		else
		{
			$timeleft = 'i';
		}
		if ($p) $timeleft = htmlquote(trim($_POST['PollTimeout']));
		if (!$timeleft) $timeleft = 'i';

		echo $T['poll_timeout'] . '<br />';
		echo '<input type="text" class="text" name="PollTimeout" size="7" maxlength="7" value="' . $timeleft . '" /><br />';
		echo '<small>' . $T['poll_timeout_no_limit'] . '<br />';
		echo $T['poll_timeout_now'] . '<br />';
		echo $T['poll_timeout_minutes'] . '</small>';

		echo '</td>';
		echo '</tr>';
	}

	echo '</table>';
	echo '</form>';

	# SHOW LATEST POSTS
	if (!$postid && !$m)
	{
		CountUserPosts();

		$c = 0;
		$start = ($page - 1) * $posts_per_page;
		if ($post->Find('Thread=' . $threadid, 'Date desc', '10')) do
		{
			if (!$c)
			{
				echo '<br />' . $T['last_posts_in_thread'] . '<br />';
				BeginPostTable();
			}

			if ($c) PostDelimiter();

			ShowPost($post, $thread, $c % 2, 1, false);
			$c++;
		}
		while ($post->FindNext());

		if ($c)
		{
			if ($NEWLAYOUT) PostDelimiter(-1);
			EndPostTable();
		}
	}
}


// Begin page
//
BeginHTML($T['title_posting']);

echo '<script type="text/javascript">' /*. '<![CDATA['*/;
include($libpath . 'editor.js.php');
echo /*']]>' .*/ '</script>';
echo '<script type="text/javascript" src="' . $libpath . 'editor.js"></script>';

if ($error)
{
	echo '<p class="error">' . $error . '</p>';
	errorlog($error);
}

if ($messageid || $forumid > 0)
{
	ShowPath($forumid);
	if ($LoginUserID) $LoginUser->SetLastActivity($forumid);
	else SetGuestLastForum($forumid);
}
elseif (!$messageid)
{
	ShowPath($thread->GetForum(), $thread->GetSubject(), $thread->GetDesc(), $threadid, true);
	if ($LoginUserID) $LoginUser->SetLastActivity($thread->GetForum());
	else SetGuestLastForum($thread->GetForum());
}

if ($_POST['preview'] != '') addlog('preview_post in ' . $threadid);

if ($messageid != 0)
	PostForm($forumid, 0, 0, $messageid, $thread);
elseif (!$forumid && !$threadid && !$postid)
	echo '<p class="error">' . $T['err_invalid_thread_for_post'] . '</p>';
else
	PostForm($forumid, $threadid, $postid, 0, $thread);

UpdateStat('PageHits', 1);

echo '<script type="text/javascript">' /*. '<![CDATA['*/;
echo 'textbox = document.form.Msg;';
echo /*']]>' .*/ '</script>';

EndHTML();
?>
