<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// public.php
// Public BlackBoard library file

define('PUBLIC_LIB', 1);

// Version definition, don't change this
$VERSION = '1.5-rc2';

// ----- BEGIN CONFIG AREA -----

// Enable UTF-8 Unicode encoding
//
// If you want to use ISO-8859-1 charset, you may want to remove the line
// '$unicode = false' from the t2h() function in this file.
//
// CHARSET variable's value must be all UPPERCASE
// Currently only ISO-8859-1 and UTF-8 are recommended, others may or may not work.
//
#$CHARSET = 'ISO-8859-1';
$CHARSET = 'UTF-8';

// You may want to change the board's configuration filename
#$conf_filename = 'board.conf.php';
#$conf_tempname = 'board.conf.tmp.php';

// These are the special characters that are displayed with the post editor
// board.conf setting 'special_chars' must be 1 for this to be relevant
// board.conf settings 'special_cols' and 'special_rows' are used for the table dimensions
$special_char_list = array(
	#'&auml;',    # ae (a umlaut)
	#'&ouml;',    # oe (o umlaut)
	#'&uuml;',    # ue (u umlaut)
	#'&szlig;',   # sz
	'&#xA9;',    # (c)
	'&#xAE;',    # (r)
	'&#x99;',    # (tm)
	'&#xB1;',    # +/-
	'&#xBC;',    # 1/4
	'&#xBD;',    # 1/2
	'&#x221A;',  # sqrt
	'&#8747;',   # int
	'&#x221E;',  # inf.
	'&#x2248;',  # almost eq.
	'&#x2260;',  # not eq.
	'&#x2264;',  # <=
	'&#x2265;',  # >=
	'&#x3B1;',   # alpha
	'&#x3B2;',   # beta
	'&#x3B3;',   # gamma
	'&#x3B4;',   # delta
	'&#x3B5;',   # epsilon
	'&#x3BB;',   # lambda
	'&#x3BC;',   # mu
	'&#x3C0;',   # pi
	'&#x3C6;',   # phi
	'&#x3C9;',   # omega
	'&#x394;',   # Delta
	'&#x3A3;',   # Sigma
	'&#x3A9;',   # Omega
	'&#8712;',   # isin
	'&#8707;',   # exist
	'&#8704;',   # forall
	'&#8743;',   # and
	'&#8744;',   # or
	'&#8745;',   # cap
	'&#8746;',   # cup
	'&#8801;',   # equiv
	'&#8834;',   # sub
	'&#8835;',   # sup
	'&#8869;',   # perp
	'&#8658;');  # rArr

// This will be used to separate forums in ShowPath() or the search result
//
#$CFG['forum_separator'] = ' &raquo; ';
$CFG['forum_separator'] = ', ';

// Specify if you're using Apache's mod_rewrite address rewriting for this board.
//
$CFG['mod_rewrite'] = rc('mod_rewrite_urls');

// ----- END CONFIG AREA -----


// scan a line for the UTF-8 marker and remove it
function is_utf8(&$line)
{
	if (substr($line, 0, 3) == "\xEF\xBB\xBF")
	{
		$line = substr($line, 3);   // remove UTF-8 marker
		return true;
	}
	else return false;
}

// read some data from the static config text file
if (!isset($conf_filename)) $conf_filename = 'board.conf.php';
if (!isset($conf_tempname)) $conf_tempname = 'board.conf.tmp.php';

$cf = file($conf_filename);
$conf_utf8 = is_utf8($cf[0]);

$CONFFILE = array();
if (is_array($cf)) foreach ($cf as $line)
{
	$line = ltrim($line);
	if ($line{0} === '#' || trim($line) === '<?php' || trim($line) === '?>') continue;
	$pos = strpos($line, '=');
	$value = trim(substr($line, $pos + 1));
	if ($CHARSET === 'UTF-8' && !$conf_utf8) $value = utf8_encode($value);
	if ($pos > 0) $CONFFILE[rtrim(substr($line, 0, $pos))] = $value;
}
unset($cf);

// read config
function rc($key, $array = false)
{
	global $CONFFILE;
	$value = $CONFFILE[$key];
	if (!$array) return $value;
	if ($value === '') return array();
	return array_map('trim', explode('|', $value));
}

// write config from modified $CONFFILE array
function rebuild_conffile()
{
	global $CONFFILE, $CHARSET, $conf_filename, $conf_tempname;
	if ($CONFFILE['nowrite']) return false;

	$nf = $CONFFILE;              // new file (array)
	$out = '';                    // new file (string)
	$cf = file($conf_filename);   // current config file (array)
	if (!is_array($cf)) return false;
	$conf_utf8 = is_utf8($cf[0]);

	foreach ($cf as $line)
	{
		if (trim($line) === '<?php' || trim($line) === '?>') continue;   // skip PHP code tags
		if (substr(ltrim($line), 0, 1) === '#')   // copy comment lines
		{
			$out .= rtrim($line) . "\n";
			continue;
		}
		$pos = strpos($line, '=');
		if ($pos === false)
		{
			$out .= rtrim($line) . "\n";   // there's no key=value in this line
		}
		else
		{
			$key = trim(substr($line, 0, $pos));
			if (isset($nf[$key]))   // this key is still present
			{
				$value = $nf[$key];
				if ($CHARSET === 'UTF-8' && !$conf_utf8) $value = utf8_decode($value);
				$out .= $key . ' = ' . trim($value) . "\n";   // write line with new value
				unset($nf[$key]);   // remove this key from input array
			}
		}
	}
	if (sizeof($nf))   // are there new keys left that didn't appear in the current config file?
	{
		foreach ($nf as $key => $value)
		{
			if ($CHARSET === 'UTF-8' && !$conf_utf8) $value = utf8_decode($value);
			$out .= $key . ' = ' . $value . "\n";
		}
	}

	$out = "<?php\n" . $out . "?>\n";
	if ($conf_utf8) $out = "\xEF\xBB\xBF" . $out;

	$fp = @fopen($conf_tempname, 'w+t');
	if ($fp === false) return false;
	if (!@fwrite($fp, $out)) return false;
	if (!@fclose($fp)) return false;

	if (!@rename($conf_tempname, $conf_filename))
	{
		// Windows compatibility (?): try again, but first delete target file for rename()
		if (!@unlink($conf_filename)) return false;
		if (!@rename($conf_tempname, $conf_filename)) return false;
	}
	return true;
}


// "Program ID"  *** SECURITY RELEVANT ***
// This string is used to build the session name and should be set to a unique value for
// the current domain at setup time. If you have two BlackBoards or PHP session dependant
// environments using the same session name, this could lead to users being able to login
// to this board (or the other environment) as someone else. I've experienced this especially
// with my WMS environment.
//
// This variable will only be set if it was not set at another location before. So you can
// leave this up to your page framework, if you want to.
//
if (!isset($prog_id)) $prog_id = rc('prog_id');

// BlackBoard runs independantly and should therefore create its own <html></html> tags and so on
$CFG['indep_page'] = rc('indep_page');

// Module filenames definition
// You can use this map to rename some or all of the mentioned PHP files
// Warning: Some of these request codes may be hard-coded somewhere else... So better don't change them!
$page = rc('indep_file');

$MODULE = array();
$MODULE['abbc_help'] = $page . '?req=abbc_help';
$MODULE['admin'] = $page . '?req=admin';
$MODULE['cp'] = $page . '?req=cp';
$MODULE['derefer'] = $page . '?req=derefer';
$MODULE['editpoll'] = $page . '?req=editpoll';
$MODULE['help'] = $page . '?req=help';
$MODULE['main'] = $page . '?req=main';
$MODULE['post'] = $page . '?req=post';
$MODULE['register'] = $page . '?req=register';
$MODULE['search'] = $page . '?req=search';
$MODULE['setuser'] = $page . '?req=setuser';
$MODULE['showip'] = $page . '?req=showip';
$MODULE['textfilter'] = $page . '?req=textfilter';
$MODULE['stat'] = $page . '?req=stat';
$MODULE['thread'] = $page . '?req=thread';
$MODULE['users'] = $page . '?req=users';

// This is the Parameter Delimiter for URLs.
//
// Background to this story:
// Once, every page was in its own PHP file. This filename was already taken from the $MODULE array,
// so that it was flexible and the webmaster could change it as needed. The first parameter for URLs
// to these pages needs to be separated by "?". Today, we have all pages called through the forum.php
// file, with already one URL parameter (i.e. forum.php?req=search). More parameters must be added
// after a "&" in this case. At this point, PD was introduced to give an easy way of changing these
// HTTP GET parameters, in case you change the page/filename structure.
//
define(PD, '&amp;');

// The following are some functions that provide general database connection and User login
// functionality that is then available in the outer page framework.

// Time measuring functions
function getmicrotime()
{
	list($usec, $sec) = explode(' ', microtime());
	return ((float)$usec + (float)$sec);
}
function perform_msec()
{
	global $perform_time_start;
	return round((getmicrotime() - $perform_time_start) * 1000, 1);
}
function perform_cputime()
{
	if (!function_exists('getrusage')) return 0;   // only accessable in unix/linux, not in windows
	$a = @getrusage();
	return round((float)$a['ru_utime.tv_sec'] * 1000 + (float)$a['ru_utime.tv_usec'] / 1000, 1);
}

// debug performance measuring and output
//
// text = -1: reset | false: start | 1: pause | 2/text: end/text
//
function dp($text = false)
{
	global $perform_time_start_tmp, $perform_time_tmp, $perform_count_tmp;
	global $db, $perform_dbtime_start_tmp, $perform_dbtime_tmp;

	if ($text === -1)
	{
		$perform_time_start_tmp = $perform_count_tmp = $perform_time_tmp = 0;
		$perform_dbtime_start_tmp = $perform_dbtime_tmp = 0;
		echo '<b>dp</b>: (reset)<br />';
	}
	else if ($text === false)
	{
		if (!isset($perform_time_start_tmp)) $perform_count_tmp = $perform_time_tmp = 0;
		if (!isset($perform_dbtime_start_tmp)) $perform_dbtime_tmp = 0;
		$perform_count_tmp++;
		$perform_time_start_tmp = getmicrotime();
		if ($db) $perform_dbtime_start_tmp = $db->GetTime();
		echo '<b>dp</b>: (start)<br />';
	}
	else if ($text === 1)
	{
		$perform_time_tmp += getmicrotime() - $perform_time_start_tmp;
		if ($db) $perform_dbtime_tmp += $db->GetTime() - $perform_dbtime_start_tmp;
		$x = round((getmicrotime() - $perform_time_start_tmp) * 1000, 2);
		if ($db) $y = round(($db->GetTime() - $perform_dbtime_start_tmp), 2);
		echo '<b>dp</b>: ' . $x . ' - ' . $y . ' = ' . ($x - $y) . ' ms (pause)<br />';
		$perform_time_start_tmp = 0;
		$perform_dbtime_start_tmp = 0;
	}
	else
	{
		if ($perform_time_start_tmp) $perform_time_tmp += getmicrotime() - $perform_time_start_tmp;
		if ($db) if ($perform_dbtime_start_tmp) $perform_dbtime_tmp += $db->GetTime() - $perform_dbtime_start_tmp;
		$x = round($perform_time_tmp * 1000, 1);
		$y = round($perform_dbtime_tmp, 1);
		echo '<b>dp</b>: ' . $x . ' - ' . $y . ' = ' . ($x - $y) . ' ms' .
			($perform_count_tmp > 1 ? " ($perform_count_tmp x " . round($x / $perform_count_tmp, 3) . " ms)" : '') .
			(is_string($text) ? ' - ' . $text : '') . ' (end)<br />';
		$perform_time_tmp = 0;
		$perform_count_tmp = 0;
		$perform_dbtime_tmp = 0;
	}
}

$perform_time_start = getmicrotime();

// This function is pre-defined from PHP 4.2.0 on
if (!function_exists('array_fill'))
{
	function array_fill($start, $num, $value)
	{
		$out = array();
		for ($n = 0; $n < $num; $n++)
		{
			$out[$n + $start] = $value;
		}
		return $out;
	}
}

// This function is pre-defined from PHP 4.3.0 on
if (!function_exists('html_entity_decode'))
{
	function html_entity_decode($str)
	{
		$str = str_replace('&quot;', '"', $str);
		$str = str_replace('&lt;', '<', $str);
		$str = str_replace('&gt;', '>', $str);
		$str = str_replace('&amp;', '&', $str);
		return $str;
	}
}

function TrailingSlash($path, $add = true)
{
	if ($path == '') return '';
	if (substr($path, strlen($path) - 1, 1) != '/' && $add) $path .= '/';
	if (substr($path, strlen($path) - 1, 1) == '/' && !$add) $path = substr($path, 0, strlen($path) - 1);
	return $path;
}

// PHP5 support
if (intval(phpversion()) >= 5) define('PHP5', 1); else define('PHP5', 0);

// ----- BEGIN: Read configuration settings -----
$extra_names = rc('extra_names', true);
$extra_count = sizeof($extra_names);

// All pathnames need a trailing slash ('/') if they're not empty!
// Library Path. Will be used to include all BlackBoard PHP library files. Normally set to lib/
$libpath = TrailingSlash(rc('lib_path'));
// Path to design folder, relative to executed PHP script
$design_path = TrailingSlash(rc('design_path'));
// Path to user avatars and post attachments
$avatar_path = TrailingSlash(rc('avatar_path'));
$photo_path  = TrailingSlash(rc('photo_path'));
$attach_path = TrailingSlash(rc('attach_path'));
// Where to write log files
$logpath = TrailingSlash(rc('log_path'));

$show_sql_verb = rc('show_sql');
$show_sql = !strcasecmp($show_sql_verb, 'on');

// Allow ICQ/Jabber notifications?
$CFG['enable_icq'] = rc('enable_icq');
$CFG['enable_jabber'] = rc('enable_jabber');
#$CFG['enable_jabber'] = false;   // it doesn't work anyway

// Automatic language scan
$CFG['languages'] = array();
$CFG['language_names'] = array();
$handle = @opendir($libpath . 'lang/');
if ($handle !== false)
{
	while ($file = readdir($handle))
		if (preg_match('/^([A-Za-z0-9\-]{2,6})\.php$/', $file, $m))
		{
			array_push($CFG['languages'], $m[1]);
			$f = file($libpath . 'lang/' . $m[1] . '.txt');
			if (!is_utf8($f[0])) $f[0] = utf8_encode($f[0]);
			$CFG['language_names'][$m[1]] =  $f[0];
		}
	sort($CFG['languages']);
	closedir($handle);
}

// Select default language. This language code MUST be listed in the $CFG['languages'] array.
$LANG = rc('def_lang');

// Select default timezone
// offset = UTC + n/4 hours
// withdst = with daylight saving time enabled
$TIMEZONE = array();
$TIMEZONE['local'] = intval(date('Z'));
$TIMEZONE['offset'] = 900 * rc('tz_offset');
$TIMEZONE['withdst'] = rc('tz_dst');

// ----- END: Read configuration settings -----

// User's LastForum special codes
define('ULF_HELP', -1);
define('ULF_CONFIG', -2);
define('ULF_PROFILE', -3);
define('ULF_SEARCH', -4);
define('ULF_STAT', -5);
define('ULF_USERS', -6);

// Send character set header
@header('Content-type: text/html; charset=' . $CHARSET);
#@header('Content-type: application/xhtml+xml; charset=' . $CHARSET);
// Setting content-type to xhtml will cause many errors (XML, Javascript, DOM) that would need to be fixed.
// Fixing these errors will break compatibility with most currently available browsers like for example
// Internet Explorer and is thus NOT RECOMMENDED at the moment. You MAY switch to xhtml setting for syntax
// checking of the pages with a strict XHTML capable browser like Mozilla/Firefox, though.

// Include all BlackBoard libraries needed to perform these early functions
require_once($libpath . 'database.lib.php');
require_once($libpath . 'stat.lib.php');
require_once($libpath . 'session.lib.php');
require_once($libpath . 'user.lib.php');
require_once($libpath . 'ua.inc.php');

// Cache arrays
// Every User/Thread/Forum Load() result is stored here for subsequent Load()s of the same record (saves DB queries)
$forum_cache = array();
$thread_cache = array();
$user_cache = array();

// Open database
$db = new IDatabase;
$db->server = rc('db_server');
$db->user = rc('db_user');
$db->password = base64_decode(rc('db_pass'));
$db->dbname = rc('db_name');
$db->tblprefix = rc('db_prefix');
if (!$NODB) $db->Open() or die('Database Open Error: ' . htmlspecialchars($db->LastError()));

// Determine Login Information
$LoginUserID = 0;
$LoginUserName = '';
$LoginUserGroups = array();

$LoginUser = new IUser;
if (check_session())
{
	// current session is OK
	$LoginUserID = intval($_SESSION['UserID']);
	if ($LoginUser->Load($LoginUserID))
	{
		$LoginUserName = $LoginUser->GetName();
	}
	else
	{
		$LoginUserID = 0;
		$LoginUserName = '';
		$LoginUserGroups = array();
	}
}
if (!$LoginUserID)
{
	// try to login with cookie information
	if (is_numeric($_COOKIE['UserID']) && $_COOKIE['UserID'] > 0)
	{
		if ($LoginUser->Load($_COOKIE['UserID']))
		{
			if (create_session($_COOKIE['UserID'], $_COOKIE['Password'], $LoginUser->GetPassword(), $LoginUser->GetFlags() & 32))
			{
				$LoginUserID = $LoginUser->GetID();
				$LoginUserName = $LoginUser->GetName();

				$LoginUser->SetLastLogin();
			}
		}
	}
}
if (!$LoginUserID)
{
	// still not logged in -> must be a guest
	// check all saved session IDs for their timeout (24h)
	$db->RemoveRecord('LastActivity<' . (time() - 24 * 3600), 'Guests');

	// check this session ID's status
	$lastact = $db->FastQuery1st('Guests', 'LastActivity', 'Session="' . session_id() . '"');
	if (!$lastact)
	{
		// session id is NEW, so add it to the list
		$db->AddRecord(array('Session' => session_id(), 'LastActivity' => time()), 'Guests');
		// update Stat table
		UpdateUserStat();
	}
	else
	{
		// session id is OLD, so update the time in the list
		$db->ChangeRecord(array('LastActivity' => time()), 'Session="' . session_id() . '"', 'Guests');
	}
}

require_once($libpath . 'abbc.lib.php');

$DEFLANG = $LANG;

// Evaluate Accept-Language header and guests' session/cookie language selection
$lang_arr = explode(',', $host_lang);
if (!$LoginUserID) $lang_arr = array_merge(array($_SESSION['pref_language'], $_COOKIE['pref_language']), $lang_arr);
$setlang = false;
foreach ($lang_arr as $a)
{
	foreach ($CFG['languages'] as $lang)
	{
		if (!$setlang && !strncasecmp($a, $lang, strlen($lang)))
		{
			$LANG = $lang;
			$setlang = true;
		}
	}
}

// Debug setting
if (!strcasecmp($show_sql_verb, 'on')) $show_sql = true;
elseif (is_numeric($show_sql_verb) && $show_sql_verb > 0) $show_sql = ($LoginUserID == $show_sql_verb);
else $show_sql = false;

$NEWLAYOUT = rc('new_layout');
$SHOWHIDECODE = true;

if ($LoginUserID)
{
	$lang = $LoginUser->GetLanguage();
	if ($lang != '' && in_array($lang, $CFG['languages'])) $LANG = $lang;

	$timezone = $LoginUser->GetTimezone();
	if ($timezone != 99)
	{
		$TIMEZONE['offset'] = $timezone * 900;
		$timezoneds = $LoginUser->GetTimezoneDS();
		if ($timezoneds != -1) $TIMEZONE['withdst'] = $timezoneds;
	}

	// Find all groups which current user is member of
	$LoginUserGroups = $db->FastQuery1stArray('GroupMembers', '`Group`', 'User=' . $LoginUserID)
	or $LoginUserGroups = array();

	$SHOWHIDECODE = $LoginUser->GetFlags() & 4;
	$NEWLAYOUT = $LoginUser->GetFlags() & 256;
}

// Include language file
#if ($LANG !== $DEVLANG) require_once($libpath . 'lang/' . $DEVLANG . '.php');
require_once($libpath . 'lang/' . $LANG . '.php');

if ($LoginUserID)
{
	$dateformat = $LoginUser->GetDateFormat();
	if ($dateformat != '') $T['dateformat_short'] = $dateformat;
}

// Design definition
// Needs database to be opened for config setting and Login User to be determined for User's private design setting

// Read registered designs list
$_count = 0;
$designs = rc('design_list', true);
foreach ($designs as $name)
{
	$info = @file($design_path . $name . '/name.txt');
	if ($info !== false && is_array($info) && sizeof($info) > 0 && trim($info[0]) != '')
	{
		$DESIGN['name'][$_count] = $info[0];
		$DESIGN['path'][$_count] = $name;
		$_count++;
	}
}
$DESIGN['count'] = $_count;

// ThreadIcons: IM icons and actions like 'reply', 'quote' etc.
// This is the default setting. It may be first overridden by the user, then by the selected design
if ($LoginUserID)
{
	$ICON = ($LoginUser->GetThreadIcons() != 1);
	$TEXT = ($LoginUser->GetThreadIcons() > 0);
}
else
{
	$ICON = true;
	$TEXT = true;
}

// Determine current design from board config and user profile
$current_design = rc('design');
if ($LoginUserID && $LoginUser->GetDesign() != -1) $current_design = $LoginUser->GetDesign();
if (isset($_REQUEST['setdesign'])) $current_design = $_REQUEST['setdesign'];

if ($INSTALLING) $current_design = 1;
if ($current_design >= sizeof($DESIGN['path'])) $current_design = sizeof($DESIGN['path']) - 1;

// Switch back to Text-Only design for some browsers
if ($host_browser === 'NN' && $host_bver < 6) $TEXTONLY = true;
if ($host_browser === 'TEXT') $TEXTONLY = true;
if ($TEXTONLY) $current_design = rc('design_textonly');

// Load currently selected design
$_path = $DESIGN['path'][$current_design];
include($design_path . $_path . '/cfg.php');


// build the 'fixed' settings
$TABLE['forum']['fixed'] = $TABLE['forum']['fixed'] ? ' style="table-layout:fixed;"' : '';
$TABLE['thread']['fixed'] = $TABLE['thread']['fixed'] ? ' style="table-layout:fixed;"' : '';
$TABLE['post']['fixed'] = $TABLE['post']['fixed'] ? ' style="table-layout:fixed;"' : '';
$TABLE['cp']['fixed'] = $TABLE['cp']['fixed'] ? ' style="table-layout:fixed;"' : '';
$TABLE['msg']['fixed'] = $TABLE['msg']['fixed'] ? ' style="table-layout:fixed;"' : '';


// ----- GLOBAL FUNCTIONS -----

// Return session ID for <a/> Links.
//
// n = 0       : sid is supposed to be the first parameter (use the PD version for board links!)
// n unset or 1: there are already parameters before sid
// n = 2       : sid is first parameter, delimiting symbol depends on PD definition
// n = 3       : delimiting symbol is ';'. Use this for shorter mod_rewrite URLs
// n = 4       : output is an <input/> tag for <form/>s
//
function sid($n = 1)
{
	if (isset($_REQUEST['setdesign']))
	{
		$design = 'setdesign=' . $_REQUEST['setdesign'];
		if ($n == 0)
			return '?setdesign=' . $_REQUEST['setdesign'] . (SID ? '&' . SID : '');
		elseif ($n == 2)
			return PD . 'setdesign=' . $_REQUEST['setdesign'] . (SID ? '&' . SID : '');
		elseif ($n == 3)
			return ';setdesign=' . $_REQUEST['setdesign'] . (SID ? '&' . SID : '');
		elseif ($n == 4)
			return SID ?
				'<input type="hidden" name="' . session_name() . '" value="' . session_id() . '" />' .
				'<input type="hidden" name="setdesign" value="' . htmlquote($_REQUEST['setdesign']) . '" />'
				: '';
		else
			return PD . 'setdesign=' . $_REQUEST['setdesign'] . (SID ? '&' . SID : '');
	}
	else
	{
		if ($n == 0)
			return (SID ? '?' . SID : '');
		elseif ($n == 2)
			return (SID ? PD . SID : '');
		elseif ($n == 3)
			return (SID ? ';' . SID : '');
		elseif ($n == 4)
			return SID ? '<input type="hidden" name="' . session_name() . '" value="' . session_id() . '" />' : '';
		else
			return (SID ? PD . SID : '');
	}
}

// Text-to-HTML conversion
// Masks special HTML characters: & < >
// Unicode-Safe (optional: $unicode): "&#...;" will not be replaced since this is
//     the alternative description of Unicode characters
// if ($spaces), multiple subsequent " " will be masked in a way that no space character gets lost
//
function t2h($text, $spaces = true, $quotes = true, $nl2br = false, $unicode = true)
{
	$unicode = false;   // we use utf-8 instead

	if ($unicode) $text = preg_replace('/&(?!#x?[\dABCDEF]+?;)/i', '&amp;', $text);
	else $text = preg_replace('/&/', '&amp;', $text);
	$text = preg_replace('/</', '&lt;', $text);
	$text = preg_replace('/>/', '&gt;', $text);
	if ($quotes) $text = str_replace('"', '&quot;', $text);

	// multiple spaces and tabs
	if ($spaces)
	{
		$text = preg_replace('/\t/', '    ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
	}

	if ($nl2br)
	{
		$text = nl2br($text);
	}

	return $text;
}

function h2t($text)
{
	$text = str_replace('&amp;', '&', $text);
	$text = str_replace('&lt;', '<', $text);
	$text = str_replace('&gt;', '>', $text);
	$text = str_replace('&quot;', '"', $text);
	$text = str_replace('&nbsp;', ' ', $text);   // TODO: this one is subject to test...

	return $text;
}

// UTF-8-aware String length counting
//
function utf8_strlen($str)
{
	$count = 0;
	$len = strlen($str);
	for ($i = 0; $i < $len; $i++)
	{
		$value = ord($str[$i]);
		if ($value > 127)
			if     ($value >= 224 && $value <= 239) $i += 2;
			elseif ($value >= 240 && $value <= 247) $i += 3;
			else   $i++;  /* 192...223 */
		$count++;
	}
	return $count;
}

// UTF-8-aware String extraction method
// Works like standard substr() and cuts off incomplete multibyte characters from the end
//
function utf8_substr($str, $start, $len = false)
{
	$pos = 0;     // current ascii string index
	$index = 0;   // current multibyte symbol index
	$s = false;   // where to start copy then
	$l = 0;       // number of bytes to copy
	$slen = strlen($str);
	while ($pos < $slen)
	{
		if ($s !== false) $l += $bytes;   // if we're in-index, add number of bytes of previous symbol
		if ($index == $start) $s = $pos;   // this may be our starting symbol index
		if ($len !== false && $index - $start + 1 == $len) break;   // stop condition

		$value = ord($str[$pos]);
		if ($value > 127)
			if     ($value >= 224 && $value <= 239) $bytes = 3;
			elseif ($value >= 240 && $value <= 247) $bytes = 4;
			else   $bytes = 2;  /* 192...223 */
		else $bytes = 1;

		$index++;
		$pos += $bytes;
	}
	if ($s === false) return '';
	return substr($str, $s, $l);

	/* $str = ($len === false ? substr($str, $start) : substr($str, $start, $len));
	// Check the end bound is a multi-byte first byte or not
	while (preg_match('/[\x80-\xBF]$/', $str)) $str = substr($str, 0, -1);
	return $str; */
}

// Limit a string to a given length and add '...' if it was truncated
//
// multibyte = true: use utf8 aware string split functions. only if current character set if UTF-8
//
function str_limit($str, $len, $multibyte = false)
{
	global $CHARSET;

	if ($multibyte && $CHARSET == 'UTF-8')
	{
		if (utf8_strlen($str) <= $len) return $str;
		return utf8_substr($str, 0, $len - 2) . '...';
	}
	else
	{
		if (strlen($str) <= $len) return $str;
		return substr($str, 0, $len - 2) . '...';
	}
}

// Limit a URL to a given length and add '...' to query/path/filename parts if they were truncated
//
function str_limit_url($url, $max)
{
	$qpos = strpos($url, '?');
	if ($qpos === false)
	{
		$path = $url;
		$query = '';
	}
	else
	{
		$path = substr($url, 0, $qpos);
		$query = substr($url, $qpos);
	}

	if (!preg_match('~^(.+?:)(//.+?)(/.+)?(/.+)$~', $path, $m)) return $url;

	// shorten query part
	$len = strlen($m[1] . $m[2] . $m[3] . $m[4] . $query);
	if ($len > $max) $query = str_limit($query, max($max - ($len - strlen($query) + 1), 8));

	// shorten path part
	$len = strlen($m[1] . $m[2] . $m[3] . $m[4] . $query);
	if ($len > $max) $m[3] = str_limit($m[3], max($max - ($len - strlen($m[3]) + 1), 10));

	// shorten filename part
	$len = strlen($m[1] . $m[2] . $m[3] . $m[4] . $query);
	if ($len > $max) $m[4] = str_limit($m[4], max($max - ($len - strlen($m[4]) + 1), 10));

	$url = $m[1] . $m[2] . $m[3] . $m[4] . $query;
	return $url;
}

// Change " into &quot;
// needed for <input value="...">
//
function htmlquote($str)
{
	return t2h($str, false);   // no spaces conversion here!
	#return str_replace('"', '&quot;', $str);
}

// Check string against a valid mail address
//
function is_mailaddr($email)
{
	list($local, $domain) = explode('@', $email);

	$pattern_local = '^([0-9a-z]*([-|_]?[0-9a-z]+)*)(([-|_]?)\.([-|_]?)[0-9a-z]*([-|_]?[0-9a-z]+)+)*([-|_]?)$';
	$pattern_domain = '^([0-9a-z]+([-]?[0-9a-z]+)*)(([-]?)\.([-]?)[0-9a-z]*([-]?[0-9a-z]+)+)*\.[a-z]{2,4}$';

	$match_local = eregi($pattern_local, $local);
	$match_domain = eregi($pattern_domain, $domain);

	return ($match_local && $match_domain);
}

// Format a number
//
// x = number for format
// decimals = maximum number of decimals | false
// range_factor = 1000 | 1024...
// sep = separator between number and range sign (will always be added)
// force_dec = true: even add something like '.00'
//
function format_number($x, $decimals = false, $range_factor = 0, $sep = '', $force_dec = false)
{
	if ($x === '') return '';

	global $T;

	$decimal = $T['num_decimal'];
	$thousand = $T['num_thousands'];
	$exp_names = array(-4 => 'p', -3 => 'n', -2 => '', -1 => 'm', 0 => '', 1 => 'k', 2 => 'M', 3 => 'G', 4 => 'T', 5 => 'P');

	// adjust range, if requested
	$exp = 0;
	if ($range_factor && $x)
	{
		if ($range_factor != 1024)
		{
			// 1024 is likely to adjust bytes' range. don't go below 1 in that case...
			// else:
			while ($x < 0.0012 * $range_factor)
			{
				$x *= $range_factor;
				$exp--;
			}
		}
		while ($x > 1.2 * $range_factor)
		{
			$x /= (float) $range_factor;
			$exp++;
		}
	}
	$exp = $exp_names[$exp];

	// limit decimals
	if ($decimals > 0) $x = round($x, $decimals);

	// split up number into integer and decimal parts
	list($int, $dec) = explode('.', $x);

	// insert thousands separators
	$a = '';
	for ($n = 0; $n < strlen($int); $n++)
	{
		if (($n % 3 == 0) && ($n > 0) && ($n < strlen($int))) $a = $thousand . $a;
		$a = substr($int, strlen($int) - 1 - $n, 1) . $a;
	}
	if ($a == '') $a = '0';

	// re-combine with correct decimal separator
	if ($decimals === false)
	{
		if ($dec != '') $a .= $decimal . $dec;
	}
	elseif ($decimals > 0)
	{
		if ($force_dec)
		{
			if (strlen($dec) < $decimals) $dec = str_pad($dec, $decimals, '0');
			if (strlen($dec)) $a .= $decimal . $dec;
		}
		else
		{
			if (strlen($dec) && intval($dec)) $a .= $decimal . $dec;
		}
	}

	if ($sep != '') $a .= $sep;
	if ($exp != '') $a .= $exp;
	return $a;
}

// exactly like date(), but...
// - knows all of BlackBoard's languages
// - can also accept an array for the date:
//   (year, month, day, hour, min, sec) -- but then, no too special format codes are available...
//
function bb_date($fmt, $time = false)
{
	global $T;

	$len = strlen($fmt);
	$out = '';

	if ($time === false) $time = time();
	$arr = is_array($time);

	for ($n = 0; $n < $len; $n++)
	{
		$c = $fmt{$n};
		switch ($c)
		{
			case '\\':
				$n++;
				$out .= $fmt{$n};
				break;
			case 'a':
				if ($arr)
					$out .= $time['hour'] < 12 ? 'am' : 'pm';
				else
					$out .= date($c, $time);
				break;
			case 'A':
				if ($arr)
					$out .= $time['hour'] < 12 ? 'AM' : 'PM';
				else
					$out .= date($c, $time);
				break;
			case 'B':
				if ($arr)
					$out .= '?';
				else
					$out .= date($c, $time);
				break;
			case 'd':
				if ($arr)
					$out .= $time['day'] < 10 ? '0' . $time['day'] : $time['day'];
				else
					$out .= date($c, $time);
				break;
			case 'D':
				if ($arr)
					$out .= '?';
				else
					switch (date('w', $time))
					{
						case 0: $out .= $T['sun']; break;
						case 1: $out .= $T['mon']; break;
						case 2: $out .= $T['tue']; break;
						case 3: $out .= $T['wed']; break;
						case 4: $out .= $T['thu']; break;
						case 5: $out .= $T['fri']; break;
						case 6: $out .= $T['sat']; break;
						default: $out .= '?';
					}
				break;
			case 'F':
				if ($arr)
					$mon = $time['month'];
				else
					$mon = date('m', $time);
				switch ($mon)
				{
					case 1: $out .= $T['january']; break;
					case 2: $out .= $T['february']; break;
					case 3: $out .= $T['march']; break;
					case 4: $out .= $T['april']; break;
					case 5: $out .= $T['may']; break;
					case 6: $out .= $T['june']; break;
					case 7: $out .= $T['july']; break;
					case 8: $out .= $T['august']; break;
					case 9: $out .= $T['september']; break;
					case 10: $out .= $T['october']; break;
					case 11: $out .= $T['november']; break;
					case 12: $out .= $T['december']; break;
					default: $out .= '?';
				}
				break;
			case 'g':
				if ($arr)
					$out .= ($time['hour'] + 11) % 12 + 1;
				else
					$out .= date($c, $time);
				break;
			case 'G':
				if ($arr)
					$out .= $time['hour'];
				else
					$out .= date($c, $time);
				break;
			case 'h':
				if ($arr)
				{
					$hr = ($time['hour'] + 11) % 12 + 1;
					$out .= $hr < 10 ? '0' . $hr : $hr;
				}
				else
					$out .= date($c, $time);
				break;
			case 'H':
				if ($arr)
					$out .= $time['hour'] < 10 ? '0' . $time['hour'] : $time['hour'];
				else
					$out .= date($c, $time);
				break;
			case 'i':
				if ($arr)
					$out .= $time['min'] < 10 ? '0' . $time['min'] : $time['min'];
				else
					$out .= date($c, $time);
				break;
			case 'I':
				if ($arr)
					$out .= '?';
				else
					$out .= date($c, $time);
				break;
			case 'j':
				if ($arr)
					$out .= $time['day'];
				else
					$out .= date($c, $time);
				break;
			case 'l':
				if ($arr)
					$out .= '?';
				else
					switch (date('w', $time))
					{
						case 0: $out .= $T['sunday']; break;
						case 1: $out .= $T['monday']; break;
						case 2: $out .= $T['tuesday']; break;
						case 3: $out .= $T['wednesday']; break;
						case 4: $out .= $T['thursday']; break;
						case 5: $out .= $T['friday']; break;
						case 6: $out .= $T['saturday']; break;
						default: $out .= '?';
					}
				break;
			case 'L':
				if ($arr)
				{
					if ($time['year'] % 400 == 0) $out .= '1';
					elseif ($time['year'] % 100 == 0) $out .= '0';
					elseif ($time['year'] % 4 == 0) $out .= '1';
					else $out .= '0';
				}
				else
					$out .= date($c, $time);
				break;
			case 'm':
				if ($arr)
					$out .= $time['month'] < 10 ? '0' . $time['month'] : $time['month'];
				else
					$out .= date($c, $time);
				break;
			case 'M':
				if ($arr)
					$mon = $time['month'];
				else
					$mon = date('m', $time);
				switch ($mon)
				{
					case 1: $out .= $T['jan']; break;
					case 2: $out .= $T['feb']; break;
					case 3: $out .= $T['mar']; break;
					case 4: $out .= $T['apr']; break;
					case 5: $out .= $T['may2']; break;
					case 6: $out .= $T['jun']; break;
					case 7: $out .= $T['jul']; break;
					case 8: $out .= $T['aug']; break;
					case 9: $out .= $T['sep']; break;
					case 10: $out .= $T['oct']; break;
					case 11: $out .= $T['nov']; break;
					case 12: $out .= $T['dec']; break;
					default: $out .= '?';
				}
				break;
			case 'n':
				if ($arr)
					$out .= $time['month'];
				else
					$out .= date($c, $time);
				break;
			case 'O':
			case 'r':
				if ($arr)
					$out .= '?';
				else
					$out .= date($c, $time);
				break;
			case 's':
				if ($arr)
					$out .= $time['sec'] < 10 ? '0' . $time['sec'] : $time['sec'];
				else
					$out .= date($c, $time);
				break;
			case 'S':
				if ($arr)
					switch ($time['day'])
					{
						case 1:
						case 21:
						case 31:
							$out .= 'st'; break;
						case 2:
						case 22:
							$out .= 'nd'; break;
						case 3:
						case 23:
							$out .= 'rd'; break;
						default:
							$out .= 'th';
					}
				else
					$out .= date($c, $time);
				break;
			case 't':
				if ($arr)
					switch ($time['month'])
					{
						case 1:
						case 3:
						case 5:
						case 7:
						case 8:
						case 10:
						case 12:
							$out .= '31'; break;
						case 4:
						case 6:
						case 9:
						case 11:
							$out .= '30'; break;
						case 2:
							$out .= bb_date('L', $time) ? '29' : '28'; break;
					}
				else
					$out .= date($c, $time);
				break;
			case 'T':
				$out .= date($c);
				break;
			case 'U':
				if ($arr)
					$out .= '?';
				else
					$out .= $time;
				break;
			case 'w':
			case 'W':
				if ($arr)
					$out .= '?';
				else
					$out .= date($c, $time);
				break;
			case 'Y':
				if ($arr)
					$out .= $time['year'];
				else
					$out .= date($c, $time);
				break;
			case 'y':
				if ($arr)
					$out .= $time['year'] % 100;
				else
					$out .= date($c, $time);
				break;
			case 'z':
			case 'Z':
				if ($arr)
					$out .= '?';
				else
					$out .= date($c, $time);
				break;
			default:
				$out .= $c;
		}
	}
	return $out;
}

// Convert a timestamp from local time to user's timezone
//
// back = true: convert from user's TZ to local
//
function convert_timezone($time, $back = false)
{
	global $TIMEZONE;

	$offset = $TIMEZONE['offset'];
	if ($TIMEZONE['withdst']) $offset += (date('I', $time) ? 3600 : 0);
	if ($back)
		return $time + $TIMEZONE['local'] - $offset;
	else
		return $time - $TIMEZONE['local'] + $offset;
}

// Format a date/timestamp
//
// opt = 1: date, 2: time, 4: seconds, 8: weekday
// sep = separator between date and time
// tz_done = t: timezone offset is already considered | f: timezone will be converted
//
function FormatTime($date, $opt = 1, $sep = ', ', $tz_done = false)
{
	global $T;

	$str = '';

	if (!$tz_done) $date = convert_timezone($date);

	if ($opt & 1)
	{
		if ($opt & 8)
		{
			$str .= bb_date('D', $date) . $sep;
		}
		$str .= bb_date($T['dateformat_short'], $date);
	}
	if ($opt & 2)
	{
		if ($opt & 1) $str .= $sep;
		if ($opt & 4)
		{
			$str .= bb_date($T['timeformat_long'], $date);
		}
		else
		{
			$str .= bb_date($T['timeformat_short'], $date);
		}
	}

	return $str;
}

// level = 1: only today/yesterday, 2+: longer time intervals, 3: seconds/mins/hours
// opt: see FormatTime function above
// tonow = t: compare time to now, f: $date is already a time difference (in seconds)
// grammar = 1: timespan past-now | 2: timespan present | 3: point of time in future | 4: point of time in past
//
function FriendlyDate($date, $level = 1, $opt = 1, $tonow = true, $grammar = 1)
{
	global $T;

	$date = convert_timezone($date);

	if (!$level) return FormatTime($date, $opt, ', ', true);

	$diff = ($tonow ? abs(convert_timezone(time()) - $date) : $date);

	if ($level == 3)
	{
		if ($diff <= 40)
			return $T['time' . $grammar . '_some_seconds'];
		if ($diff <= 100)
			return $T['time' . $grammar . '_one_minute'];
		if ($diff <= 12 * 60)
			return $T['time' . $grammar . '_some_minutes'];
		if ($diff <= 25 * 60)
			return $T['time' . $grammar . '_one_quarter_of_an_hour'];
		if ($diff <= 45 * 60)
			return $T['time' . $grammar . '_half_an_hour'];
		if ($diff <= 2 * 3600)
			return $T['time' . $grammar . '_two_hours'];
		if ($diff <= 20 * 3600)
			return $T['time' . $grammar . '_some_hours'];
		if ($diff <= 32 * 3600)
			return $T['time' . $grammar . '_one_day'];
	}

	$last_0h = getdate(convert_timezone(time()));
	$last_0h = mktime(0, 0, 0, $last_0h['mon'], $last_0h['mday'], $last_0h['year']);

	if ($level == 1)
	{
		if ($date >= $last_0h)
			return '<b>' . $T['today'] . '</b>' . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');
		if ($date >= $last_0h - 3600 * 24)
			return $T['yesterday'] . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');

		return FormatTime($date, $opt, ', ', true);
	}

	if ($grammar == 1)
	{
		if ($date >= $last_0h)
			return '<b>' . $T['since_today'] . '</b>' . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');
		if ($date >= $last_0h - 3600 * 24)
			return $T['since_yesterday'] . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');
	}
	elseif ($grammar == 4)
	{
		if ($date >= $last_0h)
			return '<b>' . $T['today'] . '</b>' . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');
		if ($date >= $last_0h - 3600 * 24)
			return $T['yesterday'] . ($opt & ~1 ? ', ' . FormatTime($date, $opt & ~1, ', ', true) : '');
	}

	$diff /= 3600 * 24;    // reduce seconds to days
	if ($diff <= 5)
		return $T['time' . $grammar . '_some_days'];
	if ($diff <= 10)
		return $T['time' . $grammar . '_one_week'];
	if ($diff <= 19)
		return $T['time' . $grammar . '_two_weeks'];
	if ($diff <= 45)
		return $T['time' . $grammar . '_one_month'];
	if ($diff <= 75)
		return $T['time' . $grammar . '_two_months'];
	if ($diff <= 105)
		return $T['time' . $grammar . '_three_months'];

	return str_replace('%s', bb_date($T['dateformat_m_y'], $date), $T['time' . $grammar . '_absolute']);
}

// Format a User's gender verbose
//
function GetGenderVerbose($gender)
{
	global $T;

	if ($gender == 'm') return $T['male'];
	if ($gender == 'f') return $T['female'];
	return '';
}

function GetGenderImage($gender)
{
	global $IMG, $TEXTONLY;

	switch ($gender)
	{
		case 'm': $img = 'male'; break;
		case 'f': $img = 'female'; break;
		default: return '';
	}

	if ($TEXTONLY)
		return '<span title="' . t2h(GetGenderVerbose($gender)) . '">[' . $gender . ']</span>';
	else
		return '<img ' . $IMG[$img] . ' title="' . t2h(GetGenderVerbose($gender)) . '" />';
}

// Return the descriptive User Status
//
// userid = user id to get status text for
// mask = string mask to insert status description into
// notnormal = t: only return not "user" text | f: return any text, also "user"
// short = use short form of text?
// plural = use plural form of text?
//
function GetUserStatusText($userid, $mask = '%s', $notnormal = false, $short = false, $plural = false)
{
	global $T;

	if ($userid == -1) return $T['former_user'];

	// Read user's groups
	if ($userid > 0) $groups = GetUserGroups($userid);
	else             $groups = array();

	// Find appropriate group/status
	if (!$userid) $status = 0;
	elseif (in_array(3, $groups)) $status = 3;
	elseif (in_array(2, $groups)) $status = 2;
	elseif (in_array(1, $groups)) $status = 1;
	else $status = -1;

	if ($status == 0) $text = ($plural ? $T['guests'] : $T['guest']);
	elseif ($status == 1 && !$notnormal) $text = ($plural ? $T['users'] : $T['user']);
	elseif ($status == 2) $text = ($short ? $T['mod_short'] : ($plural ? $T['moderators'] : $T['moderator']));
	elseif ($status == 3) $text = ($short ? $T['admin_short'] : ($plural ? $T['administrators'] : $T['administrator']));
	elseif (!$notnormal) $text = $T['unknown'];
	else $text = '';

	if ($text != '') $text = str_replace('%s', $text, $mask);
	return $text;
}

// Return the names of all groups the current user is a member of
//
function GetUserGroupNames($userid = false)
{
	global $db, $LoginUserGroups, $LoginUserID;

	if ($userid === false || $userid == $LoginUserID)
		$groups = $LoginUserGroups;
	else
		$groups = $db->FastQuery1stArray('GroupMembers', '`Group`', 'User=' . $userid);

	if (!$groups) return false;
	return $db->FastQuery1stArray('GroupNames', 'Name', 'ID IN (' . join(',', $groups) . ')', 'Name');
}

// Return all groups the current user is a member of
//
function GetUserGroups($userid = false)
{
	global $db, $LoginUserGroups, $LoginUserID;

	if ($userid === false || $userid == $LoginUserID)
		return $LoginUserGroups;
	else
	{
		$g = $db->FastQuery1stArray('GroupMembers', '`Group`', 'User=' . $userid);
		if ($g === false) $g = array();
		return $g;
	}
}

// Return the names of all known groups, as array ID=>Name
//
// only_team_visible = true, to only return groups with ShowInTeam=1
//
function GetGroupNames($only_team_visible = false)
{
	global $db;

	$arr = $db->FastQueryArray('GroupNames', 'ID, Name', ($only_team_visible ? 'ShowInTeam = 1' : ''));
	$groups = array();
	if ($arr) foreach ($arr as $rec) $groups[$rec['ID']] = $rec['Name'];

	return $groups;
}

// Set all groups the current user should be a member of
//
function SetUserGroups($userid, $groups)
{
	global $db, $LoginUserGroups, $LoginUserID;

	if (!is_array($groups)) return false;
	if (!$db->RemoveRecord('User=' . $userid, 'GroupMembers')) return false;
	$ok = true;
	foreach ($groups as $g)
		if (!$db->AddRecord(array('User' => $userid, 'Group' => $g), 'GroupMembers')) $ok = false;
	if (!$ok) return false;

	if ($userid == $LoginUserID) $LoginUserGroups = $groups;
	return true;
}

// Return all users of a group
//
function GetGroupMembers($groupid)
{
	global $db;

	$g = $db->FastQuery1stArray('GroupMembers', 'User', '`Group`=' . $groupid);
	if ($g === false) $g = array();
	return $g;
}

// Return the MIME content-type of a filename
//
function get_mimetype($file)
{
	$ext = array_pop(explode('.', $file));

	if (!strcasecmp('txt', $ext)) return 'text/plain';
	if (!strcasecmp('htm', $ext)) return 'text/html';
	if (!strcasecmp('html', $ext)) return 'text/html';
	if (!strcasecmp('gif', $ext)) return 'image/gif';
	if (!strcasecmp('jpg', $ext)) return 'image/jpeg';
	if (!strcasecmp('png', $ext)) return 'image/png';
	if (!strcasecmp('zip', $ext)) return 'application/x-zip-compressed';
	if (!strcasecmp('rar', $ext)) return 'application/x-rar-compressed';
	if (!strcasecmp('gz', $ext)) return 'application/x-gzip';
	if (!strcasecmp('tar', $ext)) return 'application/x-tar';
	if (!strcasecmp('exe', $ext)) return 'application/x-msdownload';
	if (!strcasecmp('com', $ext)) return 'application/x-msdownload';

	return 'unknown';
}

// Build a JavaScript confirmation message for Delete? checkboxes
//
function suredelete()
{
	global $T;

	return 'onclick="if (this.checked) if (!confirm(\'' . htmlquote($T['sure_delete']) . '\')) this.checked = false;"';
}

function GetUserOnlineImg($online)
{
	global $IMG, $T, $TEXTONLY;

	if ($TEXTONLY)
		return $online ? '<span title="' . $T['online'] . '">[on]</span>' : '<span title="' . $T['offline'] . '">[off]</span>';
	else
		return $online ? '<img ' . $IMG['online'] . ' title="' . $T['online'] . '" />' : '<img ' . $IMG['offline'] . ' title="' . $T['offline'] . '" />';
}

function avatar_url($user)
{
	global $avatar_path;
	if (!is_object($user)) return '';

	// fake avatar check: rotate image - disabled!
	//if ($user->GetFlags() & 1)
	//	return "avatar.php?file=" . urlencode(($user->AvatarFromURL() ? "" : $avatar_path) . $user->GetAvatar());
	//else
		return ($user->AvatarFromURL() ? '' : $avatar_path) . $user->GetAvatar();
}

// user = IUser object
// halfsize = t: allow halfsized avatars | f: always show correct image size
//
function avatar_size(&$user, $halfsize = true)
{
	global $avatar_path, $LoginUserID, $LoginUser;
	if (!is_object($user)) return '';
	if (!$user->GetAvatarX() || !$user->GetAvatarY()) return '';

	if ($LoginUserID &&
		$halfsize &&
		($LoginUser->GetFlags() & 512) &&
		($user->GetAvatarY() > rc('avatar_y') / 2))
	{
		$div = 2;
	}
	else
	{
		$div = 1;
	}

	return ' width="' . ($user->GetAvatarX() / $div) . '" height="' . ($user->GetAvatarY() / $div) . '"';
}

function photo_url($user)
{
	global $photo_path;
	if (!is_object($user)) return '';

	return ($user->PhotoFromURL() ? '' : $photo_path) . $user->GetPhoto();
}

// Count Posts per User for ALL Users
//
function CountUserPosts()
{
	global $PostsByUser, $db;

	$pre = $db->tblprefix;
	$sql = "SELECT {$pre}Posts.User, COUNT(*) " .
		"FROM {$pre}Posts " .
		"GROUP BY User";

	$db->Exec($sql);

	$PostsByUser = array();
	while ($record = $db->GetRecord())
	{
		$PostsByUser[$record[0]] = $record[1];
	}
}

// Return number of Posts for a User
// Reads value from global array, so expects CountUserPosts() to be called before
//
function GetPostsByUser($userid)
{
	global $PostsByUser;
	if (isset($PostsByUser[$userid]))
		return $PostsByUser[$userid];
	else
		return 0;
}

// Returns one or more Last Posts depending on given parameter
//
// where =
//   (int): Thread ID
//   (array): Array of Thread IDs
//   (string): Entire WHERE definition (e.g. "User=...")
//
// Pay attention to correct string/integer type! You may need to user intval() or strval()
//
function GetLastPost($where)
{
	global $db;

	if (is_int($where))
	{
		// single record version
		return $db->FastQuery('Posts', '*', 'Thread=' . $where, 'Date desc', 1);
	}
	elseif (is_array($where))
	{
		// multi-record version
		if (!sizeof($where)) return false;

		$ids = join(',', $where);

		$pre = $db->tblprefix;
		$db->Exec("SELECT {$pre}Posts.*, {$pre}Users.Name AS UserName2 FROM {$pre}Threads LEFT JOIN {$pre}Posts ON {$pre}Posts.Date={$pre}Threads.LastPostDate AND {$pre}Posts.Thread={$pre}Threads.ID LEFT JOIN {$pre}Users ON {$pre}Posts.User={$pre}Users.ID WHERE {$pre}Posts.Thread IN ($ids)");

		$a = array();
		#while ($record = $db->GetRecord())
		while ($record = @mysql_fetch_array($db->result))
		{
			if (!$record['UserName']) $record['UserName'] = $record['UserName2'];
			$a[$record['Thread']] = $record;
		}

		return $a;
	}
	elseif (is_string($where))
	{
		// single record version, with WHERE definition
		return $db->FastQuery('Posts', '*', $where, 'Date desc', 1);
	}

	// input type not supported
	return false;
}


// ----- GENERAL PAGE LAYOUT -----

// Create a little redirection page
//
// url = Address to redirect to. Should not contain " characters
//
function ForwardHTML($url)
{
	global $CSSFILE, $CHARSET, $T, $CFG, $LANG;

	echo '<?xml version="1.0" encoding="' . $CHARSET . '"?>';
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
	echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="' . $LANG . '" lang="' . $LANG . '">';
	echo '<head><title>Redirect</title>';
	if ($CSSFILE) echo '<link rel="stylesheet" href="' . $CSSFILE . '" />';
	if (!rc('no_forward')) echo '<meta http-equiv="refresh" content="0; URL=' . $url . '" />';
	echo '</head><body class="simple">';
	echo '<div class="margin">' . $T['forwarding_to'] . ':<br /><a href="' . $url . '">' . $url . '</a></div>';
	echo '</body></html>';

	#if ($CFG['code_before_exit']) eval($CFG['code_before_exit']);
	exit();
}


// Begin a general HTML page
//
// title = <title> content
// css = additional CSS definitions
// simple = t|f: create header and navigation
//
function BeginHTML($title = '', $simple = false)
{
	global $db, $LoginUserID, $LoginUserName, $IMG, $MODULE, $CSSFILE, $DESIGN, $TABLE, $ME, $CFG, $CHARSET, $T, $TIMEZONE, $INSTALLING, $LANG;

	# BEGIN PAGE

	// title is left in its original form for use in the frame page

	if (!$simple) eval($CFG['frm_beginpage']);

	if ($title != '') $title .= ' - ';
	if ($INSTALLING) $title .= 'BlackBoard';
	else             $title .= rc('forum_title');
	if ($CFG['indep_page'] || $simple)
	{
		echo '<?xml version="1.0" encoding="' . $CHARSET . '"?>';
		echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
		#echo '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
		echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="' . $LANG . '" lang="' . $LANG . '">';
		echo '<head><title>' . $title . '</title>';
		if (function_exists('abbc_css')) echo '<style type="text/css">' /*. '<![CDATA['*/ . abbc_css() . /*']]>' .*/ '</style>';
		if ($CSSFILE) echo '<link rel="stylesheet" href="' . $CSSFILE . '" />';
		if ($IMG['icon']) echo '<link rel="shortcut icon" href="' . $IMG['icon'] . '" />';

		echo '</head><body' . ($simple ? ' class="simple"' : '') . '>';
	}
	if ($TABLE['outer']['use']) echo '<table cellspacing="0" cellpadding="0" width="100%">';

	if (!$simple)
	{
		if ($IMG['toplogo'])
		{
			if ($TABLE['outer']['use']) echo '<tr><td class="head">';

			# BEGIN HEAD

			if ($TABLE['outer']['use']) echo '<table cellspacing="0" cellpadding="0" width="100%"><tr valign="bottom"><td class="logo">';
			if ($CFG['mod_rewrite'])
				echo '<a href="0' . sid(3) . '"><img ' . $IMG[toplogo] . ' alt="' . t2h(rc('forum_title')) . '" /></a>';
			else
				echo '<a href="' . $MODULE[main] . sid(2) . '"><img ' . $IMG[toplogo] . ' alt="' . t2h(rc('forum_title')) . '" /></a>';
			if ($TABLE['outer']['use']) echo '</td><td align="right" class="time">'; else echo ' &nbsp; &nbsp; ';

			echo FormatTime(time(), 3) . ' ';
			$offset = $TIMEZONE['offset'];
			if ($TIMEZONE['withdst']) $offset += (date('I') ? 3600 : 0);
			echo ($offset >= 0 ? '+' : '-');
			$offset = abs($offset) / 60;
			echo str_pad(intval($offset / 60), 2, '0', STR_PAD_LEFT);
			echo str_pad($offset % 60, 2, '0', STR_PAD_LEFT);

			if ($TABLE['outer']['use']) echo '</td></tr></table>'; else echo '<br />&nbsp;';

			# END HEAD

			if ($TABLE['outer']['use']) echo '</td></tr>';
		}

		# BEGIN NAVI

		$nav_line = rc('nav_line');
		$show_login = rc('show_login');
		$login_top = rc('login_top');
		if ($nav_line || $show_login && $login_top && !$INSTALLING)
		{
			if ($TABLE['outer']['use']) echo '<tr><td class="navi">';

			echo '<table cellspacing="0" cellpadding="0" width="100%"><tr>';

			if ($nav_line)
			{
				echo '<td>';

				echo '<table class="navlinks" cellspacing="0" cellpadding="0"><tr>';
				if (rc('parent_url') != '')
				{
					echo '<td><a href="' . rc('parent_url') . '" title="' . t2h(rc('parent_url')) . '"><img ' . $IMG['back'] . ' hspace="1" /> ' . $T['back'] . '</a></td>';
					echo '<td class="sep"><span class="sep"></span></td>';
				}

				echo '<td ' . ($ME == $MODULE['main'] ? 'class="sel"' : '') . '><a href="' .
					($CFG['mod_rewrite'] ?
						'0' . sid(3) :
						$MODULE['main'] . sid(2)) .
					'"><img ' . $IMG['overview'] . ' hspace="1" /> ' . $T['overview'] . '</a></td>';
				if ($LoginUserID)
				{
					echo '<td ' . ($ME == $MODULE['cp'] ? 'class="sel"' : '') . '><a href="' .
						($CFG['mod_rewrite'] ?
							'cp' . sid(3) :
							$MODULE['cp'] . PD . 'item=1' . sid()) .
						'"><img ' . $IMG['user_cp'] . ' hspace="1" /> ' . $T['user_cp'] . '</a></td>';
				}
				echo '<td ' . ($ME == $MODULE['users'] ? 'class="sel"' : '') . '><a href="' .
					($CFG['mod_rewrite'] ?
						'users' . sid(3) :
						$MODULE['users'] . sid(2)) .
					'"><img ' . $IMG['users'] . ' hspace="1" /> ' . $T['members'] . '</a></td>';
				echo '<td ' . ($ME == $MODULE['search'] ? 'class="sel"' : '') . '><a href="' .
					($CFG['mod_rewrite'] ?
						'search' . sid(3) :
						$MODULE['search'] . sid(2)) .
					'"><img ' . $IMG['search'] . ' hspace="1" /> ' . $T['search'] . '</a></td>';
				if (CheckRights('showstat'))
				{
					echo '<td ' . ($ME == $MODULE['stat'] ? 'class="sel"' : '') . '><a href="' .
						($CFG['mod_rewrite'] ?
							'stat' . sid(3) :
							$MODULE['stat'] . sid(2)) .
						'"><img ' . $IMG['stat'] . ' hspace="1" /> ' . $T['statistics'] . '</a></td>';
				}
				echo '<td ' . ($ME == $MODULE['help'] ? 'class="sel"' : '') . '><a href="' .
					($CFG['mod_rewrite'] ?
						'help' . sid(3) :
						$MODULE['help'] . sid(2)) .
					'"><img ' . $IMG['help'] . ' hspace="1" /> ' . $T['help'] . '</a></td>';

				// here you can add more links to the navigation line:
				#echo '<td><span class="sep">&nbsp;</span></td>';
				#echo '<td><a href="myurl' . sid(0) . '">myurl</a></td>';
				#echo '<td><a href="myurl?parameters' . sid() . '">myurl</a></td>';

				echo '</tr></table>';

				echo '</td>';
			}

			if ($show_login && $login_top && !$INSTALLING)
			{
				echo '<td class="loginctl" align="right">';

				LoginControl();

				echo '</td>';
			}

			echo '</tr></table>';

			if (!$TABLE['outer']['use']) echo '<br />';

			if ($TABLE['outer']['line'] == 1) echo '<hr />';
			if ($TABLE['outer']['use']) echo '</td></tr>';
		} // if nav_line || login_top

		# END NAVI

	} // if !simple

	if ($TABLE['outer']['use']) echo '<tr><td class="content">';

	# ADMIN-LOCK CHECK
	if (rc('admin_lock') && !CheckRights('is_admin') && !$INSTALLING)
	{
		echo '<p class="error">' . $T['board_locked'];
		$message = new IMessage;
		if ($message->Find(-1))
		{
			// Show maintenance message
			$GLOBALS['abbc_cfg']['output_div'] = 2;
			echo '<br /><br />' . abbc_proc($message->GetMsg());
		}
		echo '</p>';
		EndHTML($simple);
		#if ($CFG['code_before_exit']) eval($CFG['code_before_exit']);
		exit();
	}

	# BEGIN CONTENT
}


// End general HTML page
//
// simple = t|f: show timing information
//
function EndHTML($simple = false)
{
	global $db, $VERSION, $TABLE, $CFG, $TIMEZONE, $T, $INSTALLING;

	# END CONTENT

	if (!$simple && rc('show_login') && !rc('login_top') && !$INSTALLING)
	{
		echo '<br /><div align="right">';

		LoginControl();

		echo '</div>';
	}

	if (rc('foot_line'))
	{
		if ($TABLE['outer']['use'])
		{
			echo '</td></tr>';
			echo '<tr><td class="foot">';
		}
		else
		{
			echo '<br />';
			if ($TABLE['outer']['line'] == 1) echo '<hr />';
			echo '<div class="foot">';
		}
		echo '<a class="foot" href="http://blackboard.unclassified.de" target="_blank">BlackBoard</a> ' . $VERSION . ' | ';
		echo '&copy; 2003-4 by Yves Goergen';
		if (!$simple && rc('foot_db_time'))
		{
			echo ' | ' . $T['time'] . ': ' . perform_msec() . ' msec, ';
			if ($cputime = perform_cputime()) echo $T['cpu_time'] . ': ' . $cputime . ' msec, ';
			echo $db->GetCount() . ' ' . $T['db_queries_in'] . ' ' . $db->GetTime() . ' msec';

			$c = ob_get_contents();
			if ($c !== false)
				echo ' | ' . format_number(strlen(gzcompress($c)), 1, 1024, ' ') . 'B (' . format_number(strlen($c), 1, 1024, ' ') . 'B)';
			else
				echo ' | ' . $T['no_compression'];

			echo ' | ' . $T['timezone'] . ': ';
			$offset = $TIMEZONE['offset'];
			if ($TIMEZONE['withdst']) $offset += (date('I') ? 3600 : 0);
			echo ($offset >= 0 ? '+' : '-');
			$offset = abs($offset) / 60;
			echo str_pad(intval($offset / 60), 2, '0', STR_PAD_LEFT);
			echo str_pad($offset % 60, 2, '0', STR_PAD_LEFT);
		}

		if ($TABLE['outer']['use']) echo '</td></tr></table>'; else echo '</div>';
	}
	else
	{
		if ($TABLE['outer']['use'])
		{
			echo '</td></tr></table>';
		}
		else
		{
			echo '</div>';
		}
	}

	if ($CFG['indep_page'] || $simple) echo '</body></html>';

	# END PAGE

	// database is left open so that frame page can access it
	if (!$simple) eval($CFG['frm_endpage']);

	$db->Close();
}

// ----- SOME LITTLE PAGE ELEMENTS -----

// Display the Login form or current Login status (from com_htmlout.inc.php)
//
function LoginControl()
{
	global $ME, $MODULE, $LoginUserID, $LoginUserName, $T;

	switch ($ME)
	{
		case $MODULE['main']:
			$params = 'module=main&amp;id=' . $_GET['id'] . '&amp;page=' . $_GET['page'];
			break;
		case $MODULE['thread']:
			$params = 'module=thread&amp;id=' . $_GET['id'] . '&amp;page=' . $_GET['page'];
			break;
		case $MODULE['stat']:
			$params = 'module=stat&amp;page=' . $_GET['page'];
			break;
		default:
			$params = 'module=';
	}

	if ($LoginUserID)
	{
		echo $T['logged_in_as'] . ' <b>' . t2h($LoginUserName) . '</b> (<a href="' . $MODULE[setuser] . PD . $params . sid() . '">' . $T['logout'] . '</a>)';
	}
	else
	{
		echo '<form action="' . $MODULE[setuser] . PD . $params . sid() . '" method="post">';
		echo $T['not_logged_in'] . ' <input type="text" class="text" name="Name" value="Name/ID" size="10" maxlength="40" onfocus="if (this.value==\'Name/ID\') this.value=\'\';" onblur="if (this.value==\'\') this.value=\'Name/ID\';" /> ';
		echo '<input type="password" class="text" name="Password" value="****" size="10" maxlength="40" onfocus="if (this.value==\'****\') this.value=\'\';" onblur="if (this.value==\'\') this.value=\'****\';" /> ';
		echo '<input type="submit" class="button" value="Login" /> ';
		echo '<a href="' . $MODULE[register] . sid(2) . '">' . $T['register'] . '</a>';
		echo '</form>';
	}
}

// Show the current Forum/Thread hierarchy
//
// id = Forum ID
// thread = Thread's subject (and display Thread subject)
// threadid = Thread ID (and make a link to there)
//
function ShowPath($id, $thread = '', $desc = '', $threadid = 0, $top = false)
{
	global $ME, $MODULE, $T, $CFG;

	$forum = new IForum();
	$path = '';
	$fdesc = '';   // forum description
	$n = 0;
	$sep = $CFG['forum_separator'];

	if ($id > 0) if ($forum->Load($id)) do
	{
		$path =
			(($path == '') ? '<b>' : '') .
			($CFG['mod_rewrite'] ?
				'<a href="f.$id' . sid(3) . '">' :
				'<a href="' . $MODULE['main'] . PD . 'id=' . $id . sid() . '">') .
			t2h($forum->GetName()) . '</a>' .
			(($path == '') ? '</b>' : '') .
			(($path == '') ? '' : $sep) .
			$path;
		$id = $forum->GetParent();
		if (!$fdesc) $fdesc = $forum->GetDescription();
	}
	while ($id > 0 && $forum->Load($id));

	$path = ($CFG['mod_rewrite'] ?
			'<a href="0' . sid(3) . '">' :
			'<a href="' . $MODULE['main'] . sid(2) . '">') .
		$T['overview'] . '</a>' . (($path == '') ? '' : $sep) . $path;

	echo '<div class="path">';
	if ($thread == '')
	{
		echo $T['forum'] . ': ' . $path;
		if ($fdesc) echo '<br /><small>' . abbc_proc($fdesc) . '</small>';
	}
	elseif ($top)
	{
		$threadid = 0;   // don't show the thread title in this line!
		if ($threadid > 0)
			$thread = '<a href="' . $MODULE[thread] . PD . 'id=' . $threadid . sid() . '">' . t2h($thread) . '</a>';
		else
			$thread = t2h($thread);
		echo $T['forum'] . ': ' . $path . '<div class="thread">' . $thread . '</div>';
		if ($desc != '') echo '<div class="desc">' . $desc . '</div>';
	}
	else
	{
		echo $T['forum'] . ': ' . $path . ' <span class="sep">';
	}
	echo '</div>';
}

// Make a link to a User's profile displaying the username
//
// userid = User ID (no link if 0)
// name = UserName
// bold = t|f: make <b> tags
// email = t|f: go to email page
// t2h = t|f: t2h() the name
//
function MakeUserLink($userid, $name, $bold = false, $email = false, $t2h = true)
{
	global $MODULE, $CFG;

	$link = '';
	if ($bold) $link .= '<b>';
	if ($userid > 0)
	{
		if ($CFG['mod_rewrite'] && !$email)
			$link .= '<a href="u.' . $userid . sid(3) . '">';
		else
			$link .= '<a href="' . $MODULE[cp] . PD . 'id=' . $userid . ($email ? '&amp;action=email' : '') . sid() . '">';

		$link .= ($t2h ? t2h($name) : $name) . '</a>';
	}
	else
		$link .= ($t2h ? t2h($name) : $name);
	if ($bold) $link .= '</b>';

	return $link;
}

// Make a link to a specified Post
// that contains correct Thread ID, Page and inner-page Post ID target.
//
// post = Post to go to (can be Integer (PostID), Array(ID, Thread, Date, *) or an IPost object)
// right = 0|1: make the <img> right-aligned (main.php)
// page = Page the Post is on, if already known or e.g. -1 (saves some database queries, again)
// customtag = t: only create the <a> tag. Callee must set the </a> tag alone.
//
function MakePostLink($post, $right = 0, $page = 0, $customtag = false)
{
	global $IMG, $MODULE, $CFG, $T;

	if (is_int($post))
	{
		$postid = $post;
		$threadid = 0;
		$date = 0;
	}
	elseif (is_array($post))
	{
		$postid = $post['ID'];
		$threadid = $post['Thread'];
		$date = $post['Date'];
	}
	elseif (is_object($post))
	{
		$postid = $post->GetID();
		$threadid = $post->GetThread();
		$date = $post->GetDate();
	}
	else
	{
		// neither int nor array nor object given
		return false;
	}

	$posts_per_page = rc('posts_per_page');
	if (!$threadid || !$date && $posts_per_page)   // we only need the Date for Page determination
	{
		$post = new IPost();
		$post->Load($postid);
		$threadid = $post->GetThread();
		$date = $post->GetDate();
	}

	if (!$page)
	{
		if (!$posts_per_page)
			$page = 1;
		else
		{
			// determine Page of requested Post
			$post = new IPost();
			$page = ceil($post->Count('Thread=' . $threadid . ' AND Date<=' . $date) / $posts_per_page);
		}
	}

	if ($page == -1 || $page > 1)
		$pagelink = '&amp;page=' . $page;
	else
		$pagelink = '';

	$out = ($CFG['mod_rewrite'] ?
		'<a href="' . $threadid . ',' . $page . sid(3) . '#' . $postid . '">' :
		'<a href="' . $MODULE['thread'] . PD . 'id=' . $threadid . $pagelink . sid() . '#' . $postid . '">');

	$out .= ($customtag ?
		'' :
		'<img ' . $IMG['goto_post'] . ' title="' . $T['goto_post'] . '"' . ($right ? ' align="right" vspace="7" style="margin-left:9px;"' : '') . ' /></a>');

	return $out;
}

?>