<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// register.inc.php
// New User Registration

require($libpath . 'lang/' . $LANG . '_more.php');

// List of disallowed UserNames and Mail addresses for Registration
// These string must not appear entirely or partly in requested UserNames or EMail Addresses at registration
// Comparison is done case-insensitive
$disallowed_usernames = rc('disallowed_usernames', true);
$disallowed_emails = rc('disallowed_emails', true);
$allowed_email_domains = rc('allowed_email_domains', true);


// Returns true on successful delivery
//
function SendValidationMail(&$user, $newkey = false)
{
	global $MODULE;

	$key = $user->GetActivateKey();
	if ($newkey || $key == '')
	{
		mt_srand((double) microtime() * 1000000);
		$key = md5($user->GetName() . mt_rand(100000, 99999999) . time());
		$user->SetActivateKey($key);
	}

	// This is a bit of a hack, but the easiest way to send the mail to another address.
	// We modify the user cache and rely on it being used in the NotifyUser function.
	global $user_cache;
	$user_cache[$id]['EMail'] = $user_cache[$id]['RegEMail'];

	// NO &amp;s here! this is sent by e-mail
	$c = NotifyUser($user->GetID(), 1, 'reg_validation_subject', array(), 'reg_validation_mail',
		array(
			'%u' => rc('home_url') . '/' . $MODULE['register'] . h2t(PD) . 'id=' . $user->GetID() . '&validate=1&key=' . $key,
			'%n' => $user->GetName()));

	if (!$c) return false;
	return true;
}


$error = false;

// "Step 2": Check User data and add user
//
if ($_POST['action'] == 'step2' && rc('new_user_validation') > 0)
{
	$user = new IUser;

	// import request variables
	$name = trim($_POST['Name']);
	$password = trim($_POST['Password']);
	$password2 = trim($_POST['Password2']);
	$email = trim($_POST['EMail']);

	// check data
	if ($name == '' || $password == '' || $email == '')
	{
		$error .= $T['err_form_not_complete'] . '<br />';
	}

	// check for disallowed data
	foreach ($disallowed_usernames as $d_name)
	{
		if (!strcasecmp($name, $d_name) ||      // entire match
		    (substr($d_name, 0, 1) == '*' ?
		        stristr($name, $d_name) :       // partial match
		        false))
		{
			$error .= $T['username_disallowed'] . '<br />';
			break;
		}
	}
	if (is_numeric($name)) $error .= $T['username_disallowed'] . '<br />';

	foreach ($disallowed_emails as $d_mail)
	{
		if (!strcasecmp($email, $d_mail) ||      // entire match
		    (substr($d_mail, 0, 1) == '*' ?
		        stristr($email, $d_mail) :       // partial match
		        false))
		{
			$error .= $T['email_disallowed'] . '<br />';
			break;
		}
	}

	if (sizeof($allowed_email_domains) > 0)
	{
		$n = 0;
		foreach ($allowed_email_domains as $domain)
		{
			if (preg_match("/@$domain$/i", $email)) $n++;
		}
		if (!$n)
		{
			$error .= $T['email_disallowed'] . '<br />';
		}
	}

	// both passwords equal?
	if ($password != $password2)
	{
		$error .= $T['err_passwords_different'] . '<br />';
	}

	// password insecure?
	$secure = IsSecurePassword($password, $name);
	switch ($secure)
	{
		case 0: break;
		case 1: $error .= str_replace('%n', rc('pass_minlength'), $T['err_password_too_short']) . '<br />'; break;
		case 2: $error .= $T['err_password_is_username'] . '<br />'; break;
		case 3: $error .= $T['err_password_need_number'] . '<br />'; break;
		case 4: $error .= $T['err_password_need_special'] . '<br />'; break;
		default: $error .= $T['err_password_generic'] . '<br />'; break;
	}

	// user name too loang?
	if (strlen($name) > 40)
	{
		$error .= $T['err_username_too_long'] . '<br />';
	}

	// e-mail vaild?
	if (!is_mailaddr($email))
	{
		$error .= $T['err_invalid_email'] . '<br />';
	}

	// username available?
	if ($user->FindByName($name))
	{
		$error .= $T['err_username_assigned'] . '<br />';
	}

	if (!$error)
	{
		if (!$user->Add($name, $password, $email))
		{
			$error .= $T['err_user_not_created'] . '<br />';
			addlog('register_user ' . $name . ' error');
		}
		else
		{
			addlog('register_user ' . $name . ' (' . $user->GetID() . ') ok');
			$next_smtperror = false;

			// check activation...
			switch (rc('new_user_validation'))
			{
				case 1:  // immediately
					addlog('debug: register: set user group');
					SetUserGroups($user->GetID(), array(1));
					break;
				case 2:  // link via E-Mail
					addlog('debug: register: send email');
					if (!SendValidationMail($user, true))
					{
						$next_smtperror = true;
					}
					break;
				case 3:  // manually by mod/admin
					$admins = GetGroupMembers(3);   // Send the notification to all Admins for now
					$c = NotifyUser($admins, 1, 'reg_manual_subject', array(), 'reg_manual_mail',
						array('%u' => rc('home_url') . '/' . $MODULE['cp'] . h2t(PD) . 'id=' . $user->GetID() . '&item=1'));
					if (!$c)
					{
						$next_smtperror = true;
					}
					break;
				case 4:  // recommendation by a user
					// TODO: hmm, how shall this work? (-> see TODO.txt)
					break;
			}

			ForwardHTML($ME . PD . 'id=' . $user->GetID() . ($next_smtperror ? '&amp;smtperror=1' : '') . sid());
			exit();
		}
	}
}


// Begin page
//
BeginHTML($T['register']);

if (rc('new_user_validation') == 0)
{
	echo '<p class="error">' . $T['reg_temp_disabled'] . '</p>';
}
elseif (isset($_GET['id']))
{
	if ($_REQUEST['smtperror'] == 1) $error .= $T['err_smtp'] . '<br />';

	if ($error)
	{
		echo '<p class="error">' . $error . '</p>';
		errorlog($error);
	}

	$user = new IUser;
	$user->Load($_GET['id']);

	if ($_GET['validate'] == 1)
	{
		// Validate User with E-Mail link

		if ($_GET['key'] == $user->GetActivateKey())
		{
			// should we check '!sizeof(GetUserGroups($user->GetID()))' here?
			addlog('debug: register: validate, set user group');
			if (SetUserGroups($user->GetID(), array(1)))
			{
				echo "<p class=list_cp>" . $T['reg_validated'] . '</p>';

				$user->SetActivateKey('');
			}
			else
			{
				echo '<p class="error">' . $T['err_validation_failed'] . ' (' . t2h($user->db->LastError()) . ')</p>';
			}
		}
		else
		{
			echo '<p class="error">' . $T['err_validation_failed_admin'] . '</p>';
		}
	}
	elseif ($_GET['resend'] == 1)
	{
		addlog('debug: register: re-send email');
		if (!SendValidationMail($user))
		{
			$error = $T['err_smtp'] . '<br />';
			echo '<p class="error">' . $error . '</p>';
			errorlog($error);
		}
		else
		{
			echo '<p>' . $T['reg_validation_resent'] . '</p>';
		}
	}
	else
	{
		// Show success and welcome message

		echo '<p class="list_cp">' . $T['reg_welcome_created'] . '<br />';
		if (in_array(1, GetUserGroups($user->GetID())))
		{
			echo $T['reg_acc_validated'] . '<br />';
		}
		else
		{
			echo $T['reg_acc_not_validated'] . ' ';
			switch (rc('new_user_validation'))
			{
				case 2:
					echo $T['reg_acc_mail'] . ' ';
					break;
				case 3:
					echo $T['reg_acc_manual'] . ' ';
					break;
				default:
					echo $T['reg_acc_error'] . ' ';
			}
			echo $T['reg_acc_login'] . '<br />';
		}
		echo '<br />' . $T['reg_welcome_profile'] . '</p>';
	}
}
else
{
	// Show initial registration <form>

	if ($error)
	{
		echo '<p class="error">' . $error . '</p>';
		errorlog($error);
	}

	if ($LoginUserID) $LoginUser->SetLastActivity(0);

	echo '<form action="' . $ME . sid(2) . '" method="post">';
	echo '<input type="hidden" name="action" value="step2" />';

	echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad'] . '" border="' . $TABLE['cp']['border'] . '" width="100%">';
	echo '<tr>';
	echo '<td class="list_cp_head_left">' . $T['reg_agreement'] . '</td>';
	echo '<td class="list_cp_head_right">&nbsp;</td>';
	echo '</tr>';
	echo '</table>';

	echo '<table cellspacing="' . $TABLE['cp']['space'] . '" cellpadding="' . $TABLE['cp']['pad'] . '" border="' . $TABLE['cp']['border'] . '" width="100%">';
	echo '<colgroup>';
	echo '<col width="150" />';
	echo '<col />';
	echo '</colgroup>';

	echo '<tr>';
	echo '<td class="list_cp_l" colspan="2">' . $T['reg_agree_msg'] . '<br />&nbsp;</td>';
	echo '</tr>';

	echo '<tr>';
	echo '<td class="list_cp_n" colspan="2">' . $T['reg_data_required'] . '</td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['username'] . ':</td>';
	echo '<td class="list_cp_n"><input type="text" class="text" name="Name" ' . ($error != '' ? 'value="' . htmlquote($_POST['Name']) . '"' : '') . ' size="40" /><br />';
	echo '<small>' . $T['reg_username_desc'] . '</small></td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['password'] . ':</td>';
	echo '<td class="list_cp_n"><input type="password" class="text" name="Password" size="40" /><br />';
	echo '<small>' . $T['reg_pass_desc'] . '</small></td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['password_repeat'] . ':</td>';
	echo '<td class="list_cp_n"><input type="password" class="text" name="Password2" size="40" /><br />';
	echo '<small>' . $T['reg_pass2_desc'] . '</small></td>';
	echo '</tr>';

	echo '<tr valign="top">';
	echo '<td class="list_cp_n">' . $T['email_address'] . ':</td>';
	echo '<td class="list_cp_n"><input type="text" class="text" name="EMail" ' . ($error != '' ? 'value="' . htmlquote($_POST['EMail']) . '"' : '') . ' size="40" /><br />';
	echo '<small>' . $T['reg_mail_desc'];
	if (rc('new_user_validation') == 2)
		echo '<br />' . $T['reg_mail_note'];
	echo '</small></td>';
	echo '</tr>';

	echo '<tr>';
	echo '<td class="list_cp_n">&nbsp;</td>';
	echo '<td class="list_cp_n"><br /><input type="submit" class="button" value="' . $T['reg_continue'] . '" /></td>';
	echo '</tr>';

	echo '</table>';

	echo '</form>';
}

UpdateStat('PageHits', 1);

EndHTML();
?>
