<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// stat.lib.php
// Statistics table access functiuons


// Read out one day's Statistic values
//
// key = Value to read or '' for an array with all values
// day = 0h timestamp of requested day or 0 for current day
//
function ReadStat($key = '', $date = 0)
{
	global $db;

	// set current time if needed
	if (!$date) $date = time();
	// calculate last 0h from that time on
	$last_0h = getdate(time());
	$last_0h = mktime(0, 0, 0, $last_0h['mon'], $last_0h['mday'], $last_0h['year']);

	#$lt = localtime();
	#$last_0h = $date - (($lt[2] * 60 + $lt[1]) * 60 + $lt[0]);

	// there can possibly be a second change between both above lines!
	// then $last_0h is ...1 what we have to correct now
	#$last_0h -= $last_0h % 5;

	if ($key != '')
	{
		return $db->FastQuery1st('Stat', $key, 'Date=' . $last_0h);
	}
	else
	{
		return $db->FastQuery('Stat', '*', 'Date=' . $last_0h);
	}
}

// Update a Statistics value in the Database
//
// key = NewThreads, NewPosts, OnlineUsers, OnlineGuests, NewUsers
// value =
//   for OnlineUsers/Guests: New absolute Value, will be saved if it's greater than current value
//   else                  : Value to add/subtract to current value
//
function UpdateStat($key, $offset)
{
	global $db;

	// calculate last 0h of today
	$last_0h = getdate(time());
	$last_0h = mktime(0, 0, 0, $last_0h['mon'], $last_0h['mday'], $last_0h['year']);

	#$lt = localtime();
	#$last_0h = time() - (($lt[2] * 60 + $lt[1]) * 60 + $lt[0]);

	// there can possibly be a second change between both above lines!
	// then $last_0h is ...1 what we have to correct now
	#$last_0h -= $last_0h % 5;

	$ret = $db->FastQuery1st('Stat', 'COUNT(*)', 'Date=' . $last_0h);

	if ($ret !== false && $ret == 0)
	{
		// we are on another day now than we ware the last page hit.
		// let's see if we missed some days in the past?
		$maxdate = $db->FastQuery1st('Stat', 'MAX(Date)');
		if ($maxdate > 0 && $maxdate < $last_0h - 86400)
		{
			// some lines are missing. insert them now...
			$maxdate += 86400;
			while ($maxdate < $last_0h)
			{
				if (!$db->AddRecord("Date = $maxdate", 'Stat')) break;
				$maxdate += 86400;
			}
		}

		return $db->AddRecord("Date = $last_0h, $key = $offset", 'Stat');
	}
	else
	{
		if ($key == 'OnlineUsers' || $key == 'OnlineGuests')
		{
			return $db->ChangeRecord("$key = $offset", 'Date=' . $last_0h, 'Stat');
		}
		else
		{
			return $db->ChangeRecord("$key = $key + $offset", 'Date=' . $last_0h, 'Stat');
		}
	}
}

// Update user/guest count in Stat table
//
function UpdateUserStat()
{
	global $db;

	// count today's online users
	$last_0h = getdate(time());
	$last_0h = mktime(0, 0, 0, $last_0h['mon'], $last_0h['mday'], $last_0h['year']);

	#$a = localtime();
	#$last_0h = time() - (($a[2] * 60 + $a[1]) * 60 + $a[0]);

	// there can possibly be a second change between both above lines!
	// then $last_0h is ...1 what we have to correct now
	#$last_0h -= $last_0h % 5;

	$count = $db->FastQuery1st('Users', 'COUNT(*)', 'LastActivity > ' . $last_0h);
	$guests = $db->FastQuery1st('Guests', 'COUNT(*)', 'LastActivity > ' . $last_0h);

	// update statistics table, no error detection here
	UpdateStat('OnlineUsers', $count);
	UpdateStat('OnlineGuests', $guests);
}

?>