<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// thread.lib.php
// Thread Library, provides the IThread interface


class IThread
{

// ------------------------------------------------------------ PUBLIC VARIABLES

// ------------------------------------------------------------ PRIVATE VARIABLES
var $db;

var $ID = 0;
var $Forum = 0;
var $LastPostDate = 0;
var $Date = 0;
var $User = 0;
var $UserName = '';
var $Subject = '';
var $Desc = '';
var $Views = 0;
var $Options = 0;
	//	1	closed
	//	2	important (sticky)
	//	4	poll
	//	8	moved
	//	16	has an attachment
var $Question = '';
var $PollTimeout = 0;


// ------------------------------------------------------------ CONSTRUCTOR

function IThread()
{
	global $db;
	if (PHP5)
		$this->db = $db->__clone();
	else
		$this->db = $db;
}

function Reset()
{
	$this->ID = 0;
	$this->Forum = 0;
	$this->LastPostDate = 0;
	$this->Date = 0;
	$this->User = 0;
	$this->UserName = '';
	$this->Subject = '';
	$this->Desc = '';
	$this->Views = 0;
	$this->Options = 0;
	$this->Question = '';
	$this->PollTimeout = 0;
}

// ------------------------------------------------------------ FIND

function Find($id = 0, $order = '', $limit = '')
{
	if ($id)
		$where = 'Forum=' . $id;
	else
		$where = '';
	$record = $this->db->FastQuery('Threads', '*', $where, $order, $limit);
	return $this->LoadFromRecord($record);
}

function FindNext()
{
	$record = $this->db->GetRecord();
	return $this->LoadFromRecord($record);
}

function FindArray($where = 0, $order = '', $limit = '')
{
	if (is_int($where))
	{
		if ($where > 0)
			$where = 'Forum=' . $where;
		else
			$where = '';
	}
	elseif (!is_string($where)) return false;   // input type not supported

	$a = array();
	if ($record = $this->db->FastQuery('Threads', '*', $where, $order, $limit)) do
	{
		array_push($a, $record);
	}
	while ($record = $this->db->GetRecord());
	return $a;

}

// Find the first unread post in a/this thread
//
// LastRead = User's LastRead timestamp for this thread
//
function FirstUnreadPost($LastRead, $id = 0)
{
	if (!$id) $id = $this->ID;

	return $this->db->FastQuery('Posts', '*', 'Thread=' . $id . ' AND Date>' . $LastRead, 'Date', '1');
}


// ------------------------------------------------------------ READ ACCESS

function Load($id)
{
	global $thread_cache;

	if (array_key_exists($id, $thread_cache))
	{
		$record = $thread_cache[$id];
		$add = false;
	}
	else
	{
		$record = $this->db->FastQuery('Threads', '*', 'ID=' . $id);
		$add = true;
	}

	return $this->LoadFromRecord($record, $add);
}

function LoadFromRecord($record, $add = false)
{
	global $thread_cache;

	if ($record)
	{
		$this->ID = intval($record['ID']);
		$this->Forum = intval($record['Forum']);
		$this->LastPostDate = intval($record['LastPostDate']);
		$this->Date = intval($record['Date']);
		$this->User = intval($record['User']);
		$this->UserName = $record['UserName'];
		$this->Subject = $record['Subject'];
		$this->Desc = $record['Desc'];
		$this->Views = intval($record['Views']);
		$this->Options = intval($record['Options']);
		$this->Question = $record['Question'];
		$this->PollTimeout = intval($record['PollTimeout']);

		if ($add) $thread_cache[$this->ID] = $record;

		return true;
	}
	else
	{
		$this->Reset();
		return false;
	}
}


function GetID()
{
	return $this->ID;
}

function GetForum($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Forum;
}

function GetLastPostDate($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->LastPostDate;
}

function GetDate($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Date;
}

function GetUser($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->User;
}

function GetUserName($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->UserName;
}

function GetSubject($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Subject;
}

function GetDesc($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Desc;
}

function GetViews($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Views;
}

function GetOptions($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Options;
}

function GetQuestion($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Question;
}

function GetPollTimeout($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->PollTimeout;
}


function Count($where = '')
{
	return intval($this->db->FastQuery1st('Threads', 'COUNT(*)', $where));
}

function IsClosed($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 1) ? true : false;
}

function IsImportant($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 2) ? true : false;
}

function HasPoll($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 4) ? true : false;
}

function IsPollEnded($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->PollTimeout && $this->PollTimeout <= time()) ? true : false;
}

function IsMoved($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 8) ? true : false;
}

function IsMovedExpired($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 8 && $this->PollTimeout <= time()) ? true : false;
}

function HasAttach($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return ($this->Options & 16) ? true : false;
}


// ------------------------------------------------------------ WRITE ACCESS

// Add a new thread
// after adding, all saved values are instantly accessible in this object
//
function Add($Forum, $UserName, $Subject, $Options, $date = 0)
{
	global $LoginUserID;

	$max = $this->db->FastQuery1st('Threads', 'MAX(ID)');
	if (!$date) $date = time();

	$this->ID = $max + 1;
	$this->Forum = $Forum;
	$this->LastPostDate = time();
	$this->Date = $date;
	$this->User = $LoginUserID;
	$this->UserName = trim($UserName);
	$this->Subject = trim($Subject);
	$this->Views = 0;
	$this->Options = $Options;
	$this->Question = '';
	$this->PollTimeout = 0;

	$ok = $this->db->AddRecord(array(
			'ID' => $this->ID,
			'Forum' => $this->Forum,
			'Date' => $this->Date,
			'User' => $this->User,
			'UserName' => $this->UserName,
			'Subject' => $this->Subject,
			'Views' => $this->Views,
			'Options' => $this->Options,
			'Question' => $this->Question
		), 'Threads');

	// update statistics table, no error detection here
	if ($ok) UpdateStat('NewThreads', 1);
	return $ok;
}

// Change a thread
//
// if $id = 0, currently loaded thread will be changed and updated in the database,
// else currently loaded thread won't be changed and data will be written to another thread in the database.
// $User and $UserName are only updated if $User != -1.
// $Options is only updated if it's != -1.
//
function Change($id, $Forum, $User, $UserName, $Subject, $Options = -1)
{
	if (!$id)
	{
		$id = $this->ID;
		$this->Forum = intval($Forum);
		if ($User != -1)
		{
			$this->User = intval($User);
			$this->UserName = trim($UserName);
		}
		$this->Subject = trim($Subject);
		if ($Options != -1) $this->Options = intval($Options);
	}
	$arr = array('Forum' => $Forum,
		'Subject' => trim($Subject));
	if ($User != -1)
		$arr = array_merge($arr, array('User' => $User, 'UserName' => trim($UserName)));
	if ($Options != -1)
		$arr = array_merge($arr, array('Options' => $this->Options));

	return $this->db->ChangeRecord($arr, 'ID=' . $id, 'Threads');
}

// Only update the Subject value of the current thread
//
function SetUser($User = false, $UserName = false)
{
	$arr = array();

	if ($User !== false)
	{
		$this->User = $User;
		$arr['User'] = $User;
	}
	if ($UserName !== false)
	{
		if (isset($arr['User'])) $UserName = '';   // Set UserName to '' if a User ID is given!
		$this->UserName = trim($UserName);
		$arr['UserName'] = trim($UserName);
	}
	if (!$arr) return true;

	return $this->db->ChangeRecord($arr, 'ID=' . $this->ID, 'Threads');
}

// Only update the Subject value of a thread
// $id is optional, default is currently loaded thread
//
function SetSubject($Subject, $id = 0)
{
	if (trim($Subject) == '') $Subject = '---';


	if (!$id)
	{
		$id = $this->ID;
		$this->Subject = trim($Subject);
	}
	return $this->db->ChangeRecord(array('Subject' => trim($Subject)), 'ID=' . $id, 'Threads');
}

// Only update the Desc value of a thread
// $id is optional, default is currently loaded thread
//
function SetDesc($Desc, $id = 0)
{
	if (!$id)
	{
		$id = $this->ID;
		$this->Desc = trim($Desc);
	}
	return $this->db->ChangeRecord(array('Desc' => trim($Desc)), 'ID=' . $id, 'Threads');
}

// Only update the Options value of a thread
// $id is optional, default is currently loaded thread
//
function SetOptions($Options, $id = 0)
{
	if (!$id)
	{
		$id = $this->ID;
		$this->Options = intval($Options);
	}
	return $this->db->ChangeRecord(array('Options' => $Options), 'ID=' . $id, 'Threads');
}

// Increase number of views of a thread
// $id is optional, default is currently loaded thread
//
function IncViews($id = 0, $count = 1)
{
	$count = intval($count);
	if (!$id)
	{
		$id = $this->ID;
		$this->Views += $count;
	}
	$pre = $this->db->tblprefix;
	return $this->db->Exec("UPDATE `{$pre}Threads` SET `Views` = `Views` + $count WHERE ID=$id");
}

// Set the time of the last post in this thread
// if $LastPostDate is not set, it is automatically determined
// $id is optional, default is currently loaded thread
//
function SetLastPostDate($LastPostDate = false, $id = 0)
{
	if (!$id)
	{
		$id = $this->ID;
	}

	if ($LastPostDate === false)

	{
		// Load the last post of the old thread and update LastPostDate
		$post = new IPost;
		$post->Find('Thread=' . $id, 'Date DESC', '1');
		$LastPostDate = $post->GetDate();
	}


	if (!$id)
	{
		$this->LastPostDate = intval($LastPostDate);
	}
	return $this->db->ChangeRecord(array('LastPostDate' => $LastPostDate), 'ID=' . $id, 'Threads');
}


// Remove a thread
// $id is optional, default is currently loaded thread
//
// note: all posts in this thread must already be removed or they won't be accessible anymore!
//
function Remove($id = 0)
{
	if (!$id) $id = $this->ID;

	// note: there's no error detection on these operations
	$this->RemoveAllThreadWatchs($id);
	$this->RemoveAllVotes($id);

	$ok = $this->db->RemoveRecord('ID=' . $id, 'Threads');

	// update statistics table, no error detection here
	if ($ok) UpdateStat('NewThreads', -1);
	return $ok;
}

// Extension to Remove()
// deletes all posts in (this) thread and implicitly subsequently removes thread entry
//
function RemoveAllPosts($id = 0)
{
	if (!$id) $id = $this->ID;
	if ($id <= 0) return false;

	// remove all posts in (this) thread, without counting remaining posts every time
	$post = new IPost;
	if ($post->Find('Thread=' . $id)) do
	{
		if (!$post->Remove(0, false)) return false;
	}
	while ($post->FindNext());

	// remove thread 'manually' now
	return $this->Remove($id);
}


// ------------------------------------------------------------ POLL FUNCTIONS

// Tells whether current user has already voted at a poll
// $id is optional, default is currently loaded thread
//
function HasUserVoted($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;
	return ($this->db->FastQuery1st('PollUsers', 'COUNT(*)', 'Thread=' . $id . ' AND User=' . $LoginUserID));
}

// Get an array of votes (possible answers) for a poll
// $id is optional, default is currently loaded thread
//
function GetVotes($id = 0)
{
	if (!$id) $id = $this->ID;
	return ($this->db->FastQueryArray('PollVotes', '*', 'Thread=' . $id, 'Sort'));
}

// Count votes for a poll
// $id is optional, default is currently loaded thread
//
function CountVotes($id = 0)
{
	if (!$id) $id = $this->ID;
	return ($this->db->FastQuery1st('PollUsers', 'COUNT(*)', 'Thread=' . $id));
}

// Get an array of users, that have already voted for a poll
// $id is optional, default is currently loaded thread
// returns an array(userid, username)
//
function GetUsersVoted($id = 0)
{
	if (!$id) $id = $this->ID;
	$db = $this->db;
	$pre = $db->tblprefix;
	$db->Exec("SELECT {$pre}PollUsers.User, {$pre}Users.Name, {$pre}PollUsers.VoteNum " .
		"FROM {$pre}PollUsers, {$pre}Users " .
		"WHERE {$pre}PollUsers.User = {$pre}Users.ID and {$pre}PollUsers.Thread = $id " .
		"ORDER BY {$pre}Users.Name");
	$arr = array();
	while ($record = $db->GetRecord()) array_push($arr, $record);
	return $arr;
}

// Add current user's vote
// $id is optional, default is currently loaded thread
//
function AddVote($id = 0, $voteid)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;

	if (!$this->db->FastQuery1st('PollUsers', 'COUNT(*)', 'Thread=' . $id . ' AND User=' . $LoginUserID))
	{
		$this->db->AddRecord(array('Thread' => $id, 'User' => $LoginUserID, 'VoteNum' => $voteid), 'PollUsers');
	}
	else
		return false;

	$pre = $this->db->tblprefix;
	return $this->db->Exec("UPDATE `{$pre}PollVotes` SET `Votes` = `Votes` + 1 WHERE ID=$voteid");
}

// Remove current user's vote
// $id is optional, default is currently loaded thread
//
function RemoveVote($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;

	if ($voteid = $this->db->FastQuery1st('PollUsers', 'VoteNum', 'Thread=' . $id . ' AND User=' . $LoginUserID))
	{
		$this->db->RemoveRecord('Thread=' . $id . ' AND User=' . $LoginUserID, 'PollUsers');
	}
	else
		return false;

	$pre = $this->db->tblprefix;
	return $this->db->Exec("UPDATE `{$pre}PollVotes` SET `Votes` = `Votes` - 1 WHERE ID=$voteid");
}

// Create a new voting option (for this or another thread)
//
function CreateVoteOption($id = 0, $title, $sort)
{
	if (!$id) $id = $this->ID;

	$optionid = $this->db->FastQuery1st('PollVotes', 'MAX(ID)') + 1;
	return $this->db->AddRecord(array('ID' => $optionid, 'Thread' => $id, 'Sort' => $sort, 'Title' => $title, 'Votes' => 0), 'PollVotes');
}

// Changes a voting option
//
function ChangeVoteOption($voteid, $title, $sort, $settitle = true, $setsort = true)
{
	if (!$settitle && !$setsort) return true;

	$set = array();
	if ($settitle) $set['Title'] = $title;
	if ($setsort) $set['Sort'] = $sort;

	if ($settitle && $title == '')
		return $this->db->RemoveRecord('ID=' . $voteid, 'PollVotes') &&
		       $this->db->RemoveRecord('VoteNum=' . $voteid, 'PollUsers');
	else
		return $this->db->ChangeRecord($set, 'ID=' . $voteid, 'PollVotes');
}

// Set the poll question
//
function SetQuestion($Question, $PollTimeout, $setquestion = true, $settimeout = true, $id = 0)
{
	if (!$setquestion && !$settimeout) return true;

	$Question = trim($Question);
	if (!$id)
	{
		$id = $this->ID;
		if ($setquestion) $this->Question = $Question;
		if ($settimeout) $this->PollTimeout = $PollTimeout;
	}

	$set = array();
	if ($setquestion) $set['Question'] = $Question;
	if ($settimeout) $set['PollTimeout'] = $PollTimeout;

	return $this->db->ChangeRecord($set, 'ID=' . $id, 'Threads');
}

// Removes any PollVotes data for (this) thread
// $id is optional, default is currently loaded thread
//
function RemoveAllVotes($id = 0)
{
	if (!$id) $id = $this->ID;

	$ok = true;
	if (!$this->db->RemoveRecord('Thread=' . $id, 'PollVotes')) $ok = false;
	if (!$this->db->RemoveRecord('Thread=' . $id, 'PollUsers')) $ok = false;
	return $ok;
}

// same as RemoveAllVotes but for users
//
function RemoveAllUserVotes($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;
	if (!$id) $id = $LoginUserID;

	return $this->db->RemoveRecord('User=' . $id, 'PollUsers');
}


// ------------------------------------------------------------ THREAD WATCHING

// Set watching mode for current user
// $id is optional, default is currently loaded thread
// $Mode = 1: E-Mail, 2: ICQ  (both is possible)
//
function SetWatched($Mode = 1, $id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;
	$currentmode = $this->db->FastQuery1st('ThreadWatch', 'Mode', 'Thread=' . $id . ' AND User=' . $LoginUserID);
	if ($currentmode === false && $Mode)
	{
		return $this->db->AddRecord(array('Thread' => $id, 'User' => $LoginUserID, 'Mode' => $Mode), 'ThreadWatch');
	}
	else
	{
		if ($currentmode != $Mode)
		{
			return $this->db->ChangeRecord(array('Mode' => $Mode), 'User=' . $LoginUserID . ' AND Thread=' . $id, 'ThreadWatch');
		}
		else
		{
			return true;
		}
	}
}

// Returns watching mode for current user
// $id is optional, default is currently loaded thread
//
function IsWatched($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;
	return ($this->db->FastQuery1st('ThreadWatch', 'Mode', 'Thread=' . $id . ' AND User=' . $LoginUserID));
}

// Returns number of users that have a ThreadWatch line for this Thread
// $id is optional, default is currently loaded thread
//
function CountUsers($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;
	return ($this->db->FastQuery1st('ThreadWatch', 'COUNT(*)', 'Thread=' . $id));
}

// Removes any ThreadWatch data for (this) thread
// $id is optional, default is currently loaded thread
//
function RemoveAllThreadWatchs($id = 0)
{
	if (!$id) $id = $this->ID;
	return $this->db->RemoveRecord('Thread=' . $id, 'ThreadWatch');
}

// Removes any ThreadWatch data for (current) user
// $id is optional, default is current user
//
function RemoveAllUserWatchs($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;
	if (!$id) $id = $LoginUserID;
	return $this->db->RemoveRecord('User=' . $id, 'ThreadWatch');
}


// ------------------------------------------------------------ USER READ THREAD

// Set last-read time for current user
// $id is optional, default is currently loaded thread
//
function SetLastRead($Date, $id = 0, $force = false)
{
	global $LoginUserID;
	if (!$LoginUserID) return false;

	if (!$id) $id = $this->ID;
	$current = $this->db->FastQuery1st('ThreadWatch', 'LastRead', 'Thread=' . $id . ' AND User=' . $LoginUserID);
	if ($current === false)
	{
		return $this->db->AddRecord(array('Thread' => $id, 'User' => $LoginUserID, 'LastRead' => $Date), 'ThreadWatch');
	}
	elseif ($current < $Date || $force)
	{
		return $this->db->ChangeRecord(array('LastRead' => $Date), 'Thread=' . $id . ' AND User=' . $LoginUserID, 'ThreadWatch');
	}
}

// Returns last read time for current user
// $id is optional, default is currently loaded thread
//
function GetLastRead($id = 0)
{
	global $LoginUserID;
	if (!$LoginUserID) return 1;

	if (!$id) $id = $this->ID;
	$db = $this->db;
	return $db->FastQuery1st('ThreadWatch', 'LastRead', 'Thread=' . $id . ' AND User=' . $LoginUserID);
}


// ------------------------------------------------------------ USER NOTIFY TIMESTAMP

// Set last-notify time for current user
// $id is optional, default is currently loaded thread
// $userid is optional, default is currently logged in user
//
function SetLastNotify($Date, $id = 0, $userid = 0)
{
	global $LoginUserID;
	if (!$userid) $userid = $LoginUserID;
	if (!$userid) return false;

	if (!$id) $id = $this->ID;
	$current = $this->db->FastQuery1st('ThreadWatch', 'LastNotify', 'Thread=' . $id . ' AND User=' . $userid);
	if ($current === false)
	{
		return $this->db->AddRecord(array('Thread' => $id, 'User' => $userid, 'LastNotify' => $Date), 'ThreadWatch');
	}
	elseif ($current < $Date)
	{
		return $this->db->ChangeRecord(array('LastNotify' => $Date), 'Thread=' . $id . ' AND User=' . $userid, 'ThreadWatch');
	}
}

// Returns last notify time for current user
// $id is optional, default is currently loaded thread
// $userid is optional, default is currently logged in user
//
function GetLastNotify($id = 0, $userid = 0)
{
	global $LoginUserID;
	if (!$userid) $userid = $LoginUserID;
	if (!$userid) return 1;

	if (!$id) $id = $this->ID;
	$db = $this->db;
	return $db->FastQuery1st('ThreadWatch', 'LastNotify', 'Thread=' . $id . ' AND User=' . $userid);
}

}  // class

?>