<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/LICENSE.txt for distribution/license details
//
// Language synchronization tool

// Path to the BlackBoard libraries
$libpath = '../../bb_lib/';

// Switch to UTF-8 output
$CHARSET = 'UTF-8';

// Both languages to compare
$lang1 = 'en';
$lang2 = 'nl';


// Text-to-HTML conversion
// Masks special HTML characters: & < >
// Unicode-Safe (optional: $unicode): "&#...;" will not be replaced since this is
//     the alternative description of Unicode characters
// if ($spaces), multiple subsequent " " will be masked in a way that no space character gets lost
//
function t2h($text, $spaces = true, $quotes = true, $nl2br = true, $unicode = false)
{
	$unicode = false;   // we use utf-8 instead

	if ($unicode) $text = preg_replace('/&(?!#x?[\dABCDEF]+?;)/i', '&amp;', $text);
	else $text = preg_replace('/&/', '&amp;', $text);
	$text = preg_replace('/</', '&lt;', $text);
	$text = preg_replace('/>/', '&gt;', $text);
	if ($quotes) $text = str_replace('"', '&quot;', $text);

	// multiple spaces and tabs
	if ($spaces)
	{
		$text = preg_replace('/\t/', '    ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
	}

	if ($nl2br)
	{
		$text = nl2br($text);
		$text = preg_replace('~<br />~', '<br>', $text);
	}

	return $text;
}


@header("Content-type: text/html; charset=$CHARSET");
echo "<html><head><title>Language synchronization tool</title>";
echo "<meta http-equiv='Content-Type' content='text/html; charset=$CHARSET'>";
echo "<style type='text/css'>";
echo "td { border-bottom: solid 1px silver; border-right: solid 1px silver; }";
echo "td.odd { background-color: #F0F0FF; }";
echo "b.red { color: red; }";
echo "</style>";
echo "</head><body>";

// Read language arrays
require($libpath . 'lang/' . $lang1 . '.php');
$T1 = $T;
unset($T);

require($libpath . 'lang/' . $lang2 . '.php');
$T2 = $T;
unset($T);

echo "<table>";
echo "<tr><th>Key</th><th>1: $lang1</th><th>2: $lang2</th></tr>";
$odd = false;

// Find keys on both sides or left only
foreach ($T1 as $key => $val)
{
	$odd = !$odd;

	$cn = $odd ? ' class=odd' : '';



	if (array_key_exists($key, $T2))
	{
		echo "<tr><td$cn>" . t2h($key) . "</td><td$cn>" . t2h($T1[$key]) . "</td><td$cn>" . t2h($T2[$key]) . "</td></tr>";
		unset($T2[$key]);
	}
	else
	{
		echo "<tr><td$cn>" . t2h($key) . "</td><td$cn>" . t2h($T1[$key]) . "</td><td$cn><b class=red>-missing-</b></td></tr>";
	}
}

// Find keys right only
foreach ($T2 as $key => $val)
{
	$odd = !$odd;
	$cn = $odd ? ' class=odd' : '';

	echo "<tr><td$cn>" . t2h($key) . "</td><td$cn><b class=red>-missing-</b></td><td$cn>" . t2h($T2[$key]) . "</td></tr>";
}

echo "</table>";
echo "</body></html>";

?>
